//	(c) 2000 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.

#include "stdafx.h"
#include "sgraph.h"
#include "ArrangeGraphDialog.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CArrangeGraphDialog dialog


CArrangeGraphDialog::CArrangeGraphDialog(float width, float height,
		float startX, float startY,
		float vertical, float horizontal, int rows, int cols,
		CWnd* pParent /*=NULL*/)
	: CDialog(CArrangeGraphDialog::IDD, pParent)
{

	//{{AFX_DATA_INIT(CArrangeGraphDialog)
	m_height = _T("");
	m_start_x = _T("");
	m_start_y = _T("");
	m_width = _T("");
	m_row_edit = _T("");
	m_col_edit = _T("");
	//}}AFX_DATA_INIT

	m_height.Format("%.3g", height);
	m_start_x.Format("%.3g", startX);
	m_start_y.Format("%.3g", startY);
	m_width.Format("%.3g", width);

	initRow = rows;
	initCol = cols;

	hGap = horizontal;
	vGap = vertical;

	apply_to_all = TRUE;
}


void CArrangeGraphDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CArrangeGraphDialog)
	DDX_Control(pDX, IDC_ROW_SPIN, m_row_spin);
	DDX_Control(pDX, IDC_COL_SPIN, m_col_spin);
	DDX_Control(pDX, IDC_HORIZONTAL_EDIT, m_horizontal);
	DDX_Control(pDX, IDC_VERTICAL_EDIT, m_vertical);
	DDX_Text(pDX, IDC_HEIGHT_EDIT, m_height);
	DDX_Text(pDX, IDC_START_X_EDIT, m_start_x);
	DDX_Text(pDX, IDC_START_Y_EDIT, m_start_y);
	DDX_Text(pDX, IDC_WIDTH_EDIT, m_width);
	DDX_Text(pDX, IDC_ROW_EDIT, m_row_edit);
	DDX_Text(pDX, IDC_COL_EDIT, m_col_edit);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CArrangeGraphDialog, CDialog)
	//{{AFX_MSG_MAP(CArrangeGraphDialog)
	ON_EN_CHANGE(IDC_ROW_EDIT, OnChangeRowEdit)
	ON_EN_CHANGE(IDC_COL_EDIT, OnChangeColEdit)
	ON_BN_CLICKED(IDC_SELECTED_RADIO, OnSelectedRadio)
	ON_BN_CLICKED(IDC_ALL_RADIO, OnAllRadio)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CArrangeGraphDialog message handlers
BOOL CArrangeGraphDialog::OnInitDialog()
{
	CDialog::OnInitDialog();
	
	// Set the spinner buddies
	m_row_spin.SetBuddy(GetDlgItem(IDC_ROW_EDIT));
	m_col_spin.SetBuddy(GetDlgItem(IDC_COL_EDIT));

	// Set the allowed spinner range
	m_row_spin.SetRange32(1, UD_MAXVAL);
	m_col_spin.SetRange32(1, UD_MAXVAL);

	m_row_edit.Format("%d", initRow);
	m_col_edit.Format("%d", initCol);

	CString tmp;

	tmp.Format("%.3g", vGap);
	m_vertical.SetWindowText(tmp);

	tmp.Format("%.3g", hGap);
	m_horizontal.SetWindowText(tmp);


	if(apply_to_all){
		CheckRadioButton(IDC_SELECTED_RADIO,	// First button
			IDC_ALL_RADIO,						// Last bottom	
			IDC_ALL_RADIO);						// Button to check

		m_vertical.EnableWindow(TRUE);
		m_horizontal.EnableWindow(TRUE);
		GetDlgItem(IDC_ROW_EDIT)->EnableWindow(TRUE);
		GetDlgItem(IDC_COL_EDIT)->EnableWindow(TRUE);
		m_row_spin.EnableWindow(TRUE);
		m_col_spin.EnableWindow(TRUE);
	}
	else{
		CheckRadioButton(IDC_SELECTED_RADIO,	// First button
			IDC_ALL_RADIO,						// Last bottom	
			IDC_SELECTED_RADIO);				// Button to check
		
		m_vertical.EnableWindow(FALSE);
		m_horizontal.EnableWindow(FALSE);
		GetDlgItem(IDC_ROW_EDIT)->EnableWindow(FALSE);
		GetDlgItem(IDC_COL_EDIT)->EnableWindow(FALSE);
		m_row_spin.EnableWindow(FALSE);
		m_col_spin.EnableWindow(FALSE);
	}

	UpdateData(FALSE);

	return TRUE;
}

BOOL CArrangeGraphDialog::GetData(float &width, float &height,
		float &startX, float &startY,
		float &vertical, float &horizontal, int &rows, int &cols)
{	
	if(!ValidateString(m_height, isPositive)){
		AfxMessageBox("Can't convert the \"Graph height\" entry "
			"to a positive number. No Changes made.");
		return FALSE;
	}

	if(!ValidateString(m_width, isPositive)){
		AfxMessageBox("Can't convert the \"Graph width\" entry "
			"to a positive number. No Changes made.");
		return FALSE;
	}


	if(!ValidateString(hGap_string, isPositive)){
		AfxMessageBox("Can't convert the \"Horizontal gap\" entry "
			"to a positive number. No Changes made.");
		return FALSE;
	}

	if(!ValidateString(vGap_string, isPositive)){
		AfxMessageBox("Can't convert the \"Vertical gap\" entry "
			"to a positive number. No Changes made.");
		return FALSE;
	}
	
	if(!ValidateString(m_start_x, isPositive)){
		AfxMessageBox("Can't convert the \"Start at x\" entry "
			"to a positive number. No Changes made.");
		return FALSE;
	}

	if(!ValidateString(m_start_y, isPositive)){
		AfxMessageBox("Can't convert the \"Start at y\" entry "
			"to a positive number. No Changes made.");
		return FALSE;
	}
	
	width = (float)(atof(m_width));
	height = (float)(atof(m_height));
	startX = (float)(atof(m_start_x));
	startY = (float)(atof(m_start_y));

	vertical = (float)(atof(vGap_string));
	horizontal = (float)(atof(hGap_string));

	rows = initRow;
	cols = initCol;
	
	return TRUE;
}

void CArrangeGraphDialog::UpdateLayout()
{
	UpdateData();

	int rows = atoi(m_row_edit);
	int cols = atoi(m_col_edit);

	if(rows < 1){
		AfxMessageBox("Invalid row value!");
		return;
	}

	if(cols < 1){
		AfxMessageBox("Invalid column value!");
		return;
	}

	m_horizontal.GetWindowText(hGap_string);

	if(!ValidateString(hGap_string, isPositive)){
		AfxMessageBox("Can't convert the \"Horizontal gap\" entry "
			"to a positive number. Is there a typing error?");
		return;
	}
	
	hGap = (float)(atof(hGap_string));

	m_vertical.GetWindowText(vGap_string);

	if(!ValidateString(vGap_string, isPositive)){
		AfxMessageBox("Can't convert the \"Vertical gap\" entry "
			"to a positive number. Is there a typing error?");
		return;
	}

	vGap = (float)(atof(vGap_string));
	
	// Set the graph width and height appropriately
	m_height.Format("%.3g", (DefaultHeight - (rows - 1)*hGap)/rows);
	m_width.Format("%.3g", (DefaultWidth - (cols - 1)*vGap)/cols);

	UpdateData(FALSE);
}

void CArrangeGraphDialog::OnOK() 
{
	UpdateData();

	initRow = atoi(m_row_edit);
	initCol = atoi(m_col_edit);
	
	if(initRow < 1){
		AfxMessageBox("Invalid row value!");
		return;
	}

	if(initCol < 1){
		AfxMessageBox("Invalid column value!");
		return;
	}

	m_vertical.GetWindowText(vGap_string);
	m_horizontal.GetWindowText(hGap_string);

	CDialog::OnOK();
}

void CArrangeGraphDialog::OnAllRadio() 
{
	CheckRadioButton(IDC_SELECTED_RADIO,	// First button
		IDC_ALL_RADIO,						// Last bottom	
		IDC_ALL_RADIO);						// Button to check

	m_vertical.EnableWindow(TRUE);
	m_horizontal.EnableWindow(TRUE);
	GetDlgItem(IDC_ROW_EDIT)->EnableWindow(TRUE);
	GetDlgItem(IDC_COL_EDIT)->EnableWindow(TRUE);
	m_row_spin.EnableWindow(TRUE);
	m_col_spin.EnableWindow(TRUE);

	apply_to_all = TRUE;
}

void CArrangeGraphDialog::OnSelectedRadio() 
{
	CheckRadioButton(IDC_SELECTED_RADIO,	// First button
		IDC_ALL_RADIO,						// Last bottom	
		IDC_SELECTED_RADIO);				// Button to check

	m_vertical.EnableWindow(FALSE);
	m_horizontal.EnableWindow(FALSE);
	GetDlgItem(IDC_ROW_EDIT)->EnableWindow(FALSE);
	GetDlgItem(IDC_COL_EDIT)->EnableWindow(FALSE);
	m_row_spin.EnableWindow(FALSE);
	m_col_spin.EnableWindow(FALSE);

	apply_to_all = FALSE;
}

void CArrangeGraphDialog::OnChangeRowEdit() 
{
	UpdateLayout();
}

void CArrangeGraphDialog::OnChangeColEdit() 
{
	UpdateLayout();
}
