// AudioDialog.cpp : implementation file
//

#include "stdafx.h"
#include "sgraph.h"
#include "AudioDialog.h"
#include "SGraphDoc.h"

#include <mmsystem.h>
#include <process.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CAudioDialog dialog


CAudioDialog::CAudioDialog(CDocument* pDoc, CWnd* pParent /*= NULL*/)
	: CDialog(CAudioDialog::IDD, pParent)
{

	num_channels = MONO;
	freq = FREQ_8;
	m_pDoc = pDoc;

	//{{AFX_DATA_INIT(CAudioDialog)
	m_left_lable = _T("Mono");
	m_right_lable = _T("");
	m_volume = 100;
	m_use_default_app = TRUE;
	m_duration = 1.0f;
	//}}AFX_DATA_INIT
}


void CAudioDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAudioDialog)
	DDX_Control(pDX, IDC_RIGHT_LIST, m_right_list);
	DDX_Control(pDX, IDC_LEFT_LIST, m_left_list);
	DDX_Control(pDX, IDC_CHANNELS_COMBO, m_channels_combo);
	DDX_Control(pDX, IDC_FREQ_COMBO, m_freq_combo);
	DDX_Text(pDX, IDC_LEFT_STATIC, m_left_lable);
	DDX_Text(pDX, IDC_RIGHT_STATIC, m_right_lable);
	DDX_Text(pDX, IDC_VOLUME_EDIT, m_volume);
	DDV_MinMaxUInt(pDX, m_volume, 1, 100);
	DDX_Check(pDX, IDC_SOUND_APP_CHECK, m_use_default_app);
	DDX_Text(pDX, IDC_DURATION_EDIT, m_duration);
	DDV_MinMaxFloat(pDX, m_duration, 0.0f, 100000.0f);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CAudioDialog, CDialog)
	//{{AFX_MSG_MAP(CAudioDialog)
	ON_LBN_SELCHANGE(IDC_LEFT_LIST, OnSelchangeLeftList)
	ON_LBN_SELCHANGE(IDC_RIGHT_LIST, OnSelchangeRightList)
	ON_CBN_SELCHANGE(IDC_CHANNELS_COMBO, OnSelchangeChannelsCombo)
	ON_BN_CLICKED(IDC_PLAY_BUTTON, OnPlayButton)
	ON_BN_CLICKED(IDC_SAVE_BUTTON, OnSaveButton)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CAudioDialog message handlers
BOOL CAudioDialog::OnInitDialog()
{	
	CDialog::OnInitDialog();

	// Initialize the list box
	leftListBox.OnInitDialog(m_left_list, m_pDoc);
	rightListBox.OnInitDialog(m_right_list, m_pDoc);

	// Fill the "channels" combo box with the appropriate choices
	// (i.e. mono or stero).
	m_channels_combo.SetCurSel(num_channels);

	// Fill the frequency combo box with various choices
	m_freq_combo.SetCurSel(freq);

	// Make sure at least one plot has been loaded
	if(leftListBox.SelectedPlot() >= 0){

		// Update the duration value
		m_duration = 
			((CSgraphDoc*) m_pDoc)->graph[((CSgraphDoc*) m_pDoc)->selected_graph].
			plot[rightListBox.SelectedPlot()].GetRangeX();
	}

	// Update the left and right channel names and
	// activate the appropriate list boxes.
	if(num_channels == MONO){
		m_left_lable = _T("Mono");
		m_right_lable = _T("");

		GetDlgItem(IDC_RIGHT_LIST)->EnableWindow(FALSE);
	}
	else{
		m_left_lable = _T("Left Channel");
		m_right_lable = _T("Right Channel");
	}


	UpdateData(FALSE);

	return TRUE;

}

void CAudioDialog::OnSelchangeLeftList() 
{
	leftListBox.OnSelectChange();

	// Make sure at least one plot has been loaded
	if(leftListBox.SelectedPlot() < 0)
		return;

	// Update the duration value
	m_duration = 
		((CSgraphDoc*) m_pDoc)->graph[((CSgraphDoc*) m_pDoc)->selected_graph].
		plot[leftListBox.SelectedPlot()].GetRangeX();

	UpdateData(FALSE);
}

void CAudioDialog::OnSelchangeRightList() 
{
	rightListBox.OnSelectChange();

	// Make sure at least one plot has been loaded
	if(rightListBox.SelectedPlot() < 0)
		return;

	// Update the duration value
	m_duration = 
		((CSgraphDoc*) m_pDoc)->graph[((CSgraphDoc*) m_pDoc)->selected_graph].
		plot[rightListBox.SelectedPlot()].GetRangeX();

	UpdateData(FALSE);
}

void CAudioDialog::OnSelchangeChannelsCombo() 
{
	 num_channels = m_channels_combo.GetCurSel();

	if(num_channels == MONO){
		m_left_lable = _T("Mono");
		m_right_lable = _T("");

		GetDlgItem(IDC_RIGHT_LIST)->EnableWindow(FALSE);
	}
	else{
		m_left_lable = _T("Left Channel");
		m_right_lable = _T("Right Channel");

		GetDlgItem(IDC_RIGHT_LIST)->EnableWindow(TRUE);
	}

	UpdateData(FALSE);
}

void CAudioDialog::OnPlayButton() 
{
	// Collect the audio output values
	if(UpdateData() == 0)
		return;

	// Make sure that the user doesn't accidently create a
	// VERY large file!
	if(m_duration > 60.0f){
		if(AfxMessageBox("You are about to create a large sound file. Continue?", 
			MB_YESNO) == IDNO)
			return;
	}

	CSgraphDoc* pGraphDoc = (CSgraphDoc*)m_pDoc;
	int samplesSec = 0;

	switch(freq){
		case FREQ_8:
			samplesSec = 8000;
			break;
		case FREQ_11:
			samplesSec = 11025;
			break;
		case FREQ_22:
			samplesSec = 22050;
			break;
		case FREQ_44:
			samplesSec = 44100;
			break;
	};

	// Generate a temporary file name with a *.wav extension
	CString m_name = tmpnam(NULL);
	
	m_name += ".wav";

	if(num_channels == MONO){
		if(pGraphDoc->graph[pGraphDoc->selected_graph].
			ValidateMonoAudio(leftListBox.SelectedPlot(),
			samplesSec, m_duration, m_volume) == FALSE){

			AfxMessageBox("Selected plot cannot be saved as audio");
			return;
		}
	}
	else{
		if(pGraphDoc->graph[pGraphDoc->selected_graph].
			ValidateStereoAudio(leftListBox.SelectedPlot(),
			rightListBox.SelectedPlot(), 
			samplesSec, m_duration, m_volume) == FALSE){
			AfxMessageBox("Selected plots cannot be saved as audio");
			return;
		}
	}

	// Save the selected plots to a temporary file
	try{
		// Save the selected plot(s)
		if(num_channels == MONO){
			pGraphDoc->graph[pGraphDoc->selected_graph].
				ExportMonoAudio(m_name, leftListBox.SelectedPlot(),
				samplesSec, m_duration, m_volume);
				
		}
		else{
			pGraphDoc->graph[pGraphDoc->selected_graph].
				ExportStereoAudio(m_name, leftListBox.SelectedPlot(), 
				rightListBox.SelectedPlot(), samplesSec, m_duration, m_volume);
		}
	}
	catch(Error &e){
		AfxMessageBox(e.message());
		return;
	}


	// Play the selected plot(s)
	HCURSOR prevCursor = ::SetCursor(LoadCursor(NULL, IDC_WAIT));

	if(m_use_default_app){
		
		// Rely on the default application that is associated with
		// *.wav files
		char DefaultAppName[1024];

		// Get the name of the default application from the shell
		HINSTANCE error = FindExecutable(
			LPCSTR(m_name),
			NULL,
			DefaultAppName
			);

		if(int(error) <= 32){

			// Restore the original cursor
			::SetCursor(prevCursor);

			AfxMessageBox("Unable to locate default sound application");

			// Delete the temporary file
			_unlink(m_name);

			return;
		}

		// Spawn the default application to play the sound. Focus is not
		// returned to PrestoPlot untill this application has finished.
		if(_spawnl(_P_WAIT, DefaultAppName, LPCSTR(m_name), LPCSTR(m_name), NULL) < 0){

			// Restore the original cursor
			::SetCursor(prevCursor);

			AfxMessageBox("Unable to execute default sound application");

			// Delete the temporary file
			_unlink(m_name);

			return;
		}
	}
	else{
		// Play the sound
		if(!PlaySound(LPCSTR(m_name), NULL, 
			SND_FILENAME | SND_NODEFAULT | SND_NOWAIT | SND_SYNC)){

			// Restore the original cursor
			::SetCursor(prevCursor);

			AfxMessageBox("Unable to play the selected data set(s)");

			// Delete the temporary file
			_unlink(m_name);

			return;
		} 
	}

	// Delete the temporary file
	_unlink(m_name);

	// Restore the original cursor
	::SetCursor(prevCursor);
}

void CAudioDialog::OnSaveButton() 
{
	// Collect the audio output values
	if(UpdateData() == 0)
		return;

	// Make sure that the user doesn't accidently create a
	// VERY large file!
	if(m_duration > 60){
		if(AfxMessageBox("You are about to create a large sound file. Continue?", 
			MB_YESNO) == IDNO)
			return;
	}

	CSgraphDoc* pGraphDoc = (CSgraphDoc*)m_pDoc;
	int samplesSec = 0;

	switch(freq){
		case FREQ_8:
			samplesSec = 8000;
			break;
		case FREQ_11:
			samplesSec = 11025;
			break;
		case FREQ_22:
			samplesSec = 22050;
			break;
		case FREQ_44:
			samplesSec = 44100;
			break;
	};

	if(num_channels == MONO){
		if(pGraphDoc->graph[pGraphDoc->selected_graph].
			ValidateMonoAudio(leftListBox.SelectedPlot(),
			samplesSec, m_duration, m_volume) == FALSE){

			AfxMessageBox("Selected plot cannot be saved as audio");
			return;
		}
	}
	else{
		if(pGraphDoc->graph[pGraphDoc->selected_graph].
			ValidateStereoAudio(leftListBox.SelectedPlot(),
			rightListBox.SelectedPlot(), 
			samplesSec, m_duration, m_volume) == FALSE){
			AfxMessageBox("Selected plots cannot be saved as audio");
			return;
		}
	}

	CString m_name;

	// Prompt the user to enter a file name
	CFileDialog fDlg(FALSE /* Save As*/ , "*.wav");

	fDlg.m_ofn.lpstrTitle = "Specify WAV file to create";
	fDlg.m_ofn.lpstrFilter = "WAVE files\0*.wav\0\0";

	// Set the current directory
	char dirBuffer[1024];

	// If the buffer is not big enought, DON'T set the current 
	// directory
	if(GetCurrentDirectory(1023, dirBuffer) != 0)
		fDlg.m_ofn.lpstrInitialDir = dirBuffer;

	if(fDlg.DoModal() == IDOK){
		m_name = fDlg.GetPathName();

		UpdateData(FALSE);
	}
	else{
		// No file to save (the user has cancelled the operation)
		return;
	}

	try{
		// Save the selected plot(s)
		if(num_channels == MONO){
			pGraphDoc->graph[pGraphDoc->selected_graph].
				ExportMonoAudio(m_name, leftListBox.SelectedPlot(),
				samplesSec, m_duration, m_volume);
				
		}
		else{
			pGraphDoc->graph[pGraphDoc->selected_graph].
				ExportStereoAudio(m_name, leftListBox.SelectedPlot(), 
				rightListBox.SelectedPlot(), samplesSec, m_duration, m_volume);
		}
	}
	catch(Error &e){
		AfxMessageBox(e.message());
		return;
	}

	CDialog::OnOK();
}
