//	(c) 2000 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.

#include "stdafx.h"
#include "sgraph.h"
#include "ErrorBarDialog.h"
#include "sgraphDoc.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CErrorBarDialog dialog


CErrorBarDialog::CErrorBarDialog(CDocument* pDoc,
								 CWnd* pParent /*=NULL*/)
	: CDialog(CErrorBarDialog::IDD, pParent)
{
	m_pDoc = pDoc;

	//{{AFX_DATA_INIT(CErrorBarDialog)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}


void CErrorBarDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CErrorBarDialog)
	DDX_Control(pDX, IDC_ERROR_BAR_LIST, m_list);
	DDX_Control(pDX, IDC_HAT_SIZE_COMBO, m_hat_size);
	DDX_Control(pDX, IDC_ERROR_LINE_TYPE, m_line_type);
	DDX_Control(pDX, IDC_ERROR_LINE_SIZE_COMBO, m_line_size);
	DDX_Control(pDX, IDC_ERROR_X_DISPLAY_COMBO, m_display_x);
	DDX_Control(pDX, IDC_ERROR_Y_DISPLAY_COMBO, m_display_y);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CErrorBarDialog, CDialog)
	//{{AFX_MSG_MAP(CErrorBarDialog)
		ON_LBN_SELCHANGE(IDC_ERROR_BAR_LIST, OnSelectChange)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CErrorBarDialog message handlers
BOOL CErrorBarDialog::OnInitDialog()
{	
	CDialog::OnInitDialog();

	// Initialize the list box
	PlotListBox::OnInitDialog(m_list, m_pDoc);
	
	if(PlotListBox::SelectedPlot() >= 0){

		// Cast the document pointer
		SGraph *m_ptr = &(((CSgraphDoc *)m_pDoc)->
			graph[((CSgraphDoc *)m_pDoc)->selected_graph]);

		// Read in the Selected plot options
		m_hat_size.
			SetCurSel(m_ptr->plot[PlotListBox::SelectedPlot()].error_bar_hat_size - 1);

		m_line_type.
			SetCurSel(m_ptr->plot[PlotListBox::SelectedPlot()].error_bar_line_type);

		m_line_size.
			SetCurSel(m_ptr->plot[PlotListBox::SelectedPlot()].error_bar_line_size - 1);

		m_display_x.
			SetCurSel(m_ptr->plot[PlotListBox::SelectedPlot()].display_x_error_bar);

		m_display_y.
			SetCurSel(m_ptr->plot[PlotListBox::SelectedPlot()].display_y_error_bar);
	}
	else{
		SetDefaultOptions();
	}

	return TRUE;
}

void CErrorBarDialog::SetDefaultOptions()
{
	// "hat" size of 1
	m_hat_size.SetCurSel(0);

	// Solid line type
	m_line_type.SetCurSel(0);

	// Line size of 1
	m_line_size.SetCurSel(0);

	m_display_x.SetCurSel(SPlot::BAR_BOTH);
	m_display_y.SetCurSel(SPlot::BAR_BOTH);
}

void CErrorBarDialog::OnSelectChange()
{
	PlotListBox::OnSelectChange();

	if(PlotListBox::SelectedPlot() >= 0){
		// Cast the document pointer
		SGraph *m_ptr = &(((CSgraphDoc *)m_pDoc)->
			graph[((CSgraphDoc *)m_pDoc)->selected_graph]);

		m_hat_size.SetCurSel(m_ptr->plot[PlotListBox::SelectedPlot()].
			error_bar_hat_size - 1);

		m_line_type.SetCurSel(m_ptr->plot[PlotListBox::SelectedPlot()].
			error_bar_line_type);

		m_line_size.SetCurSel(m_ptr->plot[PlotListBox::SelectedPlot()].
			error_bar_line_size - 1);

		m_display_x.SetCurSel(m_ptr->plot[PlotListBox::SelectedPlot()].
			display_x_error_bar);

		m_display_y.SetCurSel(m_ptr->plot[PlotListBox::SelectedPlot()].
			display_y_error_bar);
	}
	else{
		SetDefaultOptions();
	}
}

void CErrorBarDialog::OnOK() 
{
	int i, plot_count;

	// Cast the document pointer
	SGraph *m_ptr = &(((CSgraphDoc *)m_pDoc)->
		graph[((CSgraphDoc *)m_pDoc)->selected_graph]);
	
	// Read in all of the variables from the dialog box
	UpdateData();

	// Has the user selected a plot(s)? If so, apply the changes
	// requested.
	for(i = 0, plot_count = 0;i < m_ptr->plot.Length();i++)
		if(m_ptr->plot[i].isDrawn && m_ptr->plot[i].isSelected){
			
			m_ptr->plot[i].error_bar_hat_size = 
				m_hat_size.GetCurSel() + 1;

			m_ptr->plot[i].error_bar_line_type = 
				m_line_type.GetCurSel();

			m_ptr->plot[i].error_bar_line_size =
				m_line_size.GetCurSel() + 1;

			m_ptr->plot[i].display_x_error_bar = m_display_x.GetCurSel();
			m_ptr->plot[i].display_y_error_bar = m_display_y.GetCurSel();

			plot_count ++;
		}

	if(plot_count == 0){
		AfxMessageBox("No Plots have been selected!");
		return;
	}
	else{
		((CSgraphDoc *)m_pDoc)->UpdateAllViews(NULL);
	}

	((CSgraphDoc *)m_pDoc)->SetModifiedFlag();

	CDialog::OnOK();
}
