// GridStyleDialog.cpp : implementation file
//

#include "stdafx.h"
#include "sgraph.h"
#include "GridStyleDialog.h"
#include "AxisDialog.h"
#include "color.h"
#include "PrestoColor.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CGridStyleDialog dialog


CGridStyleDialog::CGridStyleDialog(CSgraphDoc* pDoc, int axis)
	: CDialog(CGridStyleDialog::IDD, NULL)
{
	m_pDoc = pDoc;
	selected_axis = axis;

	//{{AFX_DATA_INIT(CGridStyleDialog)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}


void CGridStyleDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CGridStyleDialog)
	DDX_Control(pDX, IDC_APPLY, m_apply);
	DDX_Control(pDX, IDC_LINE_TYPE, m_line_type);
	DDX_Control(pDX, IDC_LINE_SIZE, m_line_size);
	DDX_Control(pDX, IDC_LINE_COLOR, m_line_color);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CGridStyleDialog, CDialog)
	//{{AFX_MSG_MAP(CGridStyleDialog)
	ON_CBN_SELCHANGE(IDC_LINE_COLOR, OnSelchangeLineColor)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CGridStyleDialog message handlers

BOOL CGridStyleDialog::OnInitDialog()
{
	CDialog::OnInitDialog();
	
	SAxisObject *normal_ptr;
	
	if(selected_axis == CAxisDialog::XAxis){
		normal_ptr = &(m_pDoc->graph[m_pDoc->selected_graph].bottomAxis);
	}
	else{
		normal_ptr = &(m_pDoc->graph[m_pDoc->selected_graph].leftAxis);
	}

	// Set the selected axis combo box
	m_apply.SetCurSel(0 /* Current Axis */);
	
	// Set the current line type
	m_line_type.SetCurSel(normal_ptr->grid_type);

	// Set the current line size
	m_line_size.SetCurSel(normal_ptr->grid_size - 1);

	// Set the current line color
	int tmp_color = ColorToIndex(normal_ptr->grid_color);

	if(tmp_color < 0){
		// Custom color
		custom_line_color = -tmp_color;
		m_line_color.SetCurSel(S_CLR_CUSTOM);
	}
	else{
		m_line_color.SetCurSel(tmp_color);
	}

	UpdateData(FALSE /* Initialize dialog */);

	return TRUE;
}

void CGridStyleDialog::UpdateGrid(int graph, int axis)
{
	SAxisObject *normal_ptr;

	if(axis == CAxisDialog::XAxis){
		normal_ptr = &(m_pDoc->graph[graph].bottomAxis);
	}
	else{
		normal_ptr = &(m_pDoc->graph[graph].leftAxis);
	}

	// Turn on axis grids
	normal_ptr->isGridDrawn = TRUE;

	normal_ptr->grid_type = m_line_type.GetCurSel();
	normal_ptr->grid_size = m_line_size.GetCurSel() + 1;

	if(m_line_color.GetCurSel() == S_CLR_CUSTOM){
		normal_ptr->grid_color = (COLORREF)(custom_line_color);
	}
	else{
		normal_ptr->grid_color = 
			IndexToColor(m_line_color.GetCurSel());
	}
}

void CGridStyleDialog::OnOK() 
{
	// What graphs and axes do we apply the changes to?
	int apply = m_apply.GetCurSel();

	// The four different possibilities for the apply combo box
	if(apply == 0 /* Current Graph */){
		UpdateGrid(m_pDoc->selected_graph, selected_axis);
	}
	
	if(apply == 1 /* All Axes, Current Graph */){
		UpdateGrid(m_pDoc->selected_graph, CAxisDialog::XAxis);
		UpdateGrid(m_pDoc->selected_graph, CAxisDialog::YAxis);
	}
	
	if(apply == 2 /* Current Axis, All Graphs */){
		for(int i = 0;i < m_pDoc->num_graph;i++)
			UpdateGrid(i, selected_axis);
	}

	if(apply == 3 /* All Axes, All Graphs */){
		for(int i = 0;i < m_pDoc->num_graph;i++){
			UpdateGrid(i, CAxisDialog::XAxis);
			UpdateGrid(i, CAxisDialog::YAxis);
		}		
	}

	CDialog::OnOK();
}

void CGridStyleDialog::OnSelchangeLineColor() 
{
	// Allow the user to specify a custom line color
	if(m_line_color.GetCurSel() == S_CLR_CUSTOM){
		COLORREF tmp_color;

		if((custom_line_color == NO_COLOR) || (custom_line_color > 0))
			tmp_color = custom_line_color;
		else
			tmp_color = (COLORREF)(0);

		CPrestoColor pcDlg(tmp_color);

		if(pcDlg.DoModal() == IDOK)
			custom_line_color = (int)(pcDlg.GetColor());
	}
}
