//	(c) 2000-2002 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.
#include "stdafx.h"
#include "sgraph.h"
#include "sgraphDoc.h"
#include "HistogramDialog.h"
#include "PrestoUtil.h"
#include "color.h"
#include <math.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CHistogramDialog dialog


CHistogramDialog::CHistogramDialog(CDocument* pDoc, 
								   CWnd* pParent /*=NULL*/)
	: CDialog(CHistogramDialog::IDD, pParent)
{
	m_pDoc = pDoc;

	// Use bin width by default
	use_num_bin = FALSE;

	//{{AFX_DATA_INIT(CHistogramDialog)
	m_end = _T("");
	m_start = _T("");
	m_width = _T("");
	m_num_bin = _T("");
	m_normalize = FALSE;
	m_center_bin = TRUE;
	//}}AFX_DATA_INIT
}


void CHistogramDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CHistogramDialog)
	DDX_Control(pDX, IDC_GRAPH, m_graph);
	DDX_Control(pDX, IDC_HIST_LIST, m_list);
	DDX_Text(pDX, IDC_END, m_end);
	DDX_Text(pDX, IDC_START, m_start);
	DDX_Text(pDX, IDC_WIDTH, m_width);
	DDX_Text(pDX, IDC_NUM_BIN, m_num_bin);
	DDX_Check(pDX, IDC_CHECK_NORM, m_normalize);
	DDX_Check(pDX, IDC_CHECK_CENTER, m_center_bin);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CHistogramDialog, CDialog)
	//{{AFX_MSG_MAP(CHistogramDialog)
		ON_LBN_SELCHANGE(IDC_HIST_LIST, OnSelectChange)
	ON_BN_CLICKED(IDC_BIN_WIDTH_RADIO, OnBinWidthRadio)
	ON_BN_CLICKED(IDC_NUM_BIN_RADIO, OnNumBinRadio)
	ON_BN_CLICKED(IDC_CHECK_NORM, OnCheckNorm)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CHistogramDialog message handlers
BOOL CHistogramDialog::OnInitDialog()
{
	int i;
	
	CDialog::OnInitDialog();

	// Initialize the list box
	PlotListBox::OnInitDialog(m_list, m_pDoc);

	// Fill the combo box with various graph choices
	m_graph.AddString("Current");

	for(i = 0;i < ((CSgraphDoc *)m_pDoc)->num_graph;i++)
		if(((CSgraphDoc *)m_pDoc)->graph[i].isDrawn){
			CString counter;
			counter.Format("%d", i);
			m_graph.AddString(counter);
		}
	
	// Make the "Current" string the combo box default
	m_graph.SetCurSel(0);
	
	// Set the radio buttons
	if(use_num_bin){
		CheckRadioButton(IDC_BIN_WIDTH_RADIO,	// First button
						 IDC_NUM_BIN_RADIO,		// Last bottom	
						 IDC_NUM_BIN_RADIO);	// Button to check
		GetDlgItem(IDC_WIDTH)->EnableWindow(FALSE);
		GetDlgItem(IDC_NUM_BIN)->EnableWindow(TRUE);
	}
	else{
		CheckRadioButton(IDC_BIN_WIDTH_RADIO,	// First button
						 IDC_NUM_BIN_RADIO,		// Last bottom	
						 IDC_BIN_WIDTH_RADIO);	// Button to check
		GetDlgItem(IDC_WIDTH)->EnableWindow(TRUE);
		GetDlgItem(IDC_NUM_BIN)->EnableWindow(FALSE);
	}

	return TRUE;
}

void CHistogramDialog::OnSelectChange()
{
	PlotListBox::OnSelectChange();
}

void CHistogramDialog::OnOK() 
{
	int i, plot_count;
	int destination_graph;
	float start, end, width;
	int bins;

	// Cast the document pointer
	SGraph *m_ptr = &(((CSgraphDoc *)m_pDoc)->
		graph[((CSgraphDoc *)m_pDoc)->selected_graph]);
	
	// Read in all of the variables from the dialog box
	UpdateData();

	// Has the user selected a plot(s)?
	for(i = 0, plot_count = 0;i < m_ptr->plot.Length();i++)
		if(m_ptr->plot[i].isDrawn && m_ptr->plot[i].isSelected)
			plot_count ++;

	if(plot_count == 0){
		AfxMessageBox("No Plots have been selected!");
		return;
	}

	if(StringToFloat(m_end, end) == FALSE){
		AfxMessageBox("Invalid end value!");
		return;
	}
		
	if(StringToFloat(m_start, start) == FALSE){
		AfxMessageBox("Invalid start value!");
		return;
	}

	if(use_num_bin){
		float tmp_float;

		if(StringToFloat(m_num_bin, tmp_float) == FALSE){
			AfxMessageBox("Invalid number of bins");
			return;
		}
		
		bins = (int)(tmp_float);

		// Is the number of bins valid?
		if(bins <= 0){
			AfxMessageBox("# Bin <= 0. No histogram for you.");
			return;
		}
	}
	else{
		if(StringToFloat(m_width, width) == FALSE){
			AfxMessageBox("Invalid bin width");
			return;
		}

		// Is the width valid?
		if(width <= 0.0){
			AfxMessageBox("Bin Width value <= 0. No histogram for you.");
			return;
		}
	}
	
	///////////////////////////////////////////
	// Perform some sanity checks on the inputs
	///////////////////////////////////////////
	
	// Are the limits out of order?
	if(start > end){
		AfxMessageBox("Start value > Ending value.");
		return;
	}

	// Get the destination graph
	if((destination_graph = m_graph.GetCurSel()) == 0)
		// The current graph
		destination_graph = ((CSgraphDoc *)m_pDoc)->selected_graph;
	else
		destination_graph --;


	// Compute the histogram(s)
	for(i = 0;i < m_ptr->plot.Length();i++)
		if(m_ptr->plot[i].isDrawn && m_ptr->plot[i].isSelected){
			SPlot newPlot;

			if(use_num_bin)
				Histogram(newPlot, m_ptr->plot[i], i, start, end, bins);
			else
				Histogram(newPlot, m_ptr->plot[i], i, start, end, width);

			((CSgraphDoc *)m_pDoc)->graph[destination_graph].plot.
				Push(newPlot);
		}

	////////////////////////////////////////////////////////////////////////////////
	// Update the plot range and update the document
	////////////////////////////////////////////////////////////////////////////////
	SGraph *m_dest_ptr = &(((CSgraphDoc *)m_pDoc)->graph[destination_graph]);

	// Make sure that the X axis scale is set to the appropriate value for ALL
	// plots (not just the newly created histogram).
	if(m_dest_ptr->x_scaleType == SCALE_LOG){
		BOOL force_linear_scale = FALSE;

		for(i = 0;i < m_dest_ptr->plot.Length();i++){
			if(m_dest_ptr->plot[i].canXLog == TRUE){
				m_dest_ptr->plot[i].x_scaleType = SCALE_LOG;
			}
			else{
				force_linear_scale = TRUE;
				break;
			}
		}
		
		// The graph will not support a log scale!
		if(force_linear_scale){
			// Change the graph scale to linear
			m_dest_ptr->x_scaleType = SCALE_LINEAR;

			// Change all of the plot scales to linear
			for(i = 0;i < m_dest_ptr->plot.Length();i++)
				m_dest_ptr->plot[i].x_scaleType = SCALE_LINEAR;
		}
	}
	else{  // m_dest_ptr->x_scaleType == SCALE_LINEAR
		for(i = 0;i < m_dest_ptr->plot.Length();i++)
			m_dest_ptr->plot[i].y_scaleType = SCALE_LINEAR;
	}

	// Make sure that the Y axis is set to a linear scale
	m_dest_ptr->y_scaleType = SCALE_LINEAR;
	for(i = 0;i < m_dest_ptr->plot.Length();i++)
		m_dest_ptr->plot[i].y_scaleType = SCALE_LINEAR;

	m_dest_ptr->SetPlotRange();
	m_dest_ptr->SetBoundingBox();

	((CSgraphDoc *)m_pDoc)->SetModifiedFlag();

	CDialog::OnOK();
}

// Compute a histogram of a plot (and return the result as a histogram).
SPlot &CHistogramDialog::Histogram(SPlot &hist, SPlot &plot, int id, 
								   float start, float end, float width)
{
	int numBin = (int)((end - start)/width);
	int i;

	float *histArray = new float [numBin];
	
	// Zero out the fledgling histogram
	for(i = 0;i < numBin;i++)
		histArray[i] = 0.0;

	stackIterator<DataPoint> next(plot.data);
	DataPoint xy;

	while(next(xy)){
		double int_part;
		
		modf((xy.y - start)/width, &int_part);
		
		if(((int)int_part < numBin) && ((int)int_part >= 0))
			histArray[(int)int_part]++;
	}
	
	///////////////////////////
	// Build the histogram plot
	///////////////////////////
	// Give the plot a name
	hist.info.Format("Histogram of Set %d", id);

	// Set the plot style
	hist.pointType = SPlot::POINT_HISTOGRAM_X;
	hist.brush.lbStyle = BS_HOLLOW;
	hist.brush.lbColor = NO_COLOR;
	hist.lineColor = 0x00000000; // Black
	hist.lineType = SPlot::LINE_NONE;
	
	if(start <= 0.0)
		hist.canXLog = FALSE;
	else
		hist.canXLog = TRUE;

	hist.canYLog = FALSE;

	// Normalize the histogram if needed
	if(m_normalize){
		double norm = 0.0;
		
		for(i = 0;i < numBin;i++)
			norm += width*histArray[i];

		for(i = 0;i < numBin;i++)
			histArray[i] /= (float)norm;
	}

	for(i = 0;i < numBin;i++){
		if(m_center_bin)
			// Center the bin on the data point
			// x = left edge of bin
			xy.x = start + i*width;
		else
			// Align the left bin edge on the data point.
			// x = center of bin
			xy.x = start + (i + 0.5f)*width;

		xy.y = histArray[i];
		hist.data.Push(xy);
	}
	
	// "Terminate" the histogram by inserting a bin with zero counts
	// at the end.
	if(m_center_bin)
		// Center the bin on the data point
		// x = left edge of bin
		xy.x = start + numBin*width;
	else
		// Align the left bin edge on the data point.
		// x = center of bin
		xy.x = start + (numBin + 0.5f)*width;

	xy.y = 0.0f;
	hist.data.Push(xy);

	// Clean up
	delete [] histArray;

	return hist;
}

// Compute a histogram of a plot (and return the result as a histogram).
SPlot &CHistogramDialog::Histogram(SPlot &hist, SPlot &plot, int id, 
								   float start, float end, int numBin)
{
	//int numBin = (int)((end - start)/width);
	float width = (end - start)/numBin;

	int i;

	float *histArray = new float [numBin];
	
	// Zero out the fledgling histogram
	for(i = 0;i < numBin;i++)
		histArray[i] = 0.0;

	stackIterator<DataPoint> next(plot.data);
	DataPoint xy;

	while(next(xy)){
		double int_part;
		
		modf((xy.y - start)/width, &int_part);
		
		if(((int)int_part < numBin) && ((int)int_part >= 0))
			histArray[(int)int_part]++;
	}
	
	///////////////////////////
	// Build the histogram plot
	///////////////////////////
	// Give the plot a name
	hist.info.Format("Histogram of Set %d", id);

	// Set the plot style
	hist.pointType = SPlot::POINT_HISTOGRAM_X;
	hist.brush.lbStyle = BS_HOLLOW;
	hist.brush.lbColor = NO_COLOR;
	hist.lineColor = 0x00000000; // Black
	hist.lineType = SPlot::LINE_NONE;
	
	if(start <= 0.0)
		hist.canXLog = FALSE;
	else
		hist.canXLog = TRUE;

	hist.canYLog = FALSE;

	// Normalize the histogram if needed
	if(m_normalize){
		double norm = 0.0;
		
		for(i = 0;i < numBin;i++)
			norm += width*histArray[i];

		for(i = 0;i < numBin;i++)
			histArray[i] /= (float)norm;
	}

	for(i = 0;i < numBin;i++){
		if(m_center_bin)
			// Center the bin on the data point
			// x = left edge of bin
			xy.x = start + i*width;
		else
			// Align the left bin edge on the data point.
			// x = center of bin
			xy.x = start + (i + 0.5f)*width;

		xy.y = histArray[i];
		hist.data.Push(xy);
	}

	// "Terminate" the histogram by inserting a bin with zero counts
	// at the end.
	if(m_center_bin)
		// Center the bin on the data point
		// x = left edge of bin
		xy.x = start + numBin*width;
	else
		// Align the left bin edge on the data point.
		// x = center of bin
		xy.x = start + (numBin + 0.5f)*width;

	xy.y = 0.0f;
	hist.data.Push(xy);

	// Clean up
	delete [] histArray;

	return hist;
}

void CHistogramDialog::OnBinWidthRadio() 
{
	if(use_num_bin){
		CheckRadioButton(IDC_BIN_WIDTH_RADIO,	// First button
						 IDC_NUM_BIN_RADIO,		// Last bottom	
						 IDC_BIN_WIDTH_RADIO);	// Button to check

		GetDlgItem(IDC_WIDTH)->EnableWindow(TRUE);
		GetDlgItem(IDC_NUM_BIN)->EnableWindow(FALSE);

		use_num_bin = FALSE;
	}
}

void CHistogramDialog::OnNumBinRadio() 
{
	if(!use_num_bin){
		CheckRadioButton(IDC_BIN_WIDTH_RADIO,	// First button
						 IDC_NUM_BIN_RADIO,		// Last bottom	
						 IDC_NUM_BIN_RADIO);	// Button to check

		GetDlgItem(IDC_WIDTH)->EnableWindow(FALSE);
		GetDlgItem(IDC_NUM_BIN)->EnableWindow(TRUE);

		use_num_bin = TRUE;
	}
}

void CHistogramDialog::OnCheckNorm() 
{
	m_normalize = !m_normalize;
}
