//	(c) 2000-2002 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.

#include "stdafx.h"
#include "sgraph.h"
#include "sgraphDoc.h"
#include "LegendDialog.h"
#include "color.h"
#include "PrestoColor.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CLegendDialog dialog


CLegendDialog::CLegendDialog(CDocument* pDoc, CWnd* pParent /*=NULL*/)
	: CDialog(CLegendDialog::IDD, pParent)
{
	m_pDoc = pDoc;

	//{{AFX_DATA_INIT(CLegendDialog)
	m_show_bbox = FALSE;
	m_text = _T("");
	//}}AFX_DATA_INIT
}


void CLegendDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CLegendDialog)
	DDX_Control(pDX, IDC_FILL_COLOR, m_fill_color);
	DDX_Control(pDX, IDC_EDGE_COLOR, m_edge_color);
	DDX_Control(pDX, IDC_LEGEND_LIST, m_list);
	DDX_Check(pDX, IDC_BBOX_CHECK, m_show_bbox);
	DDX_Text(pDX, IDC_LEGEND_EDIT, m_text);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CLegendDialog, CDialog)
	//{{AFX_MSG_MAP(CLegendDialog)
		ON_LBN_SELCHANGE(IDC_LEGEND_LIST, OnSelectChange)
	ON_BN_CLICKED(IDC_LABLE_FONT_BUTTON, OnLableFontButton)
	ON_CBN_SELCHANGE(IDC_EDGE_COLOR, OnSelchangeEdgeColor)
	ON_CBN_SELCHANGE(IDC_FILL_COLOR, OnSelchangeFillColor)
	ON_BN_CLICKED(IDC_BBOX_CHECK, OnBboxCheck)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CLegendDialog message handlers

BOOL CLegendDialog::OnInitDialog()
{	
	int i;
	int plot_count;

	CDialog::OnInitDialog();

	// Initialize the list box
	PlotListBox::OnInitDialog(m_list, m_pDoc);

	// Cast the document pointer
	SGraph *m_ptr = &(((CSgraphDoc *)m_pDoc)->
		graph[((CSgraphDoc *)m_pDoc)->selected_graph]);

	tmp_legend = new STextObject [m_ptr->plot.Length()];

	for(i = 0, plot_count = 0;i < m_ptr->plot.Length();i++){
		if((plot_count <= MAX_LIST_BOX_ENTRIES) && (m_ptr->plot[i].isDrawn)){	
			// Make a copy of the legend
			tmp_legend[i] = m_ptr->plot[i].legend;

			plot_count++;
		}
	}

	if(PlotListBox::SelectedPlot() >= 0){
		m_text = tmp_legend[PlotListBox::SelectedPlot()].GetText();
	}

	// Set the state of the various check boxes
	m_show_bbox = m_ptr->legend.isBoundingBox;

	// Only enable the color combo boxes if we're displaying a legend
	// bounding box.
	if(m_show_bbox){
		GetDlgItem(IDC_FILL_COLOR)->EnableWindow(TRUE);
		GetDlgItem(IDC_EDGE_COLOR)->EnableWindow(TRUE);
	}
	else{
		GetDlgItem(IDC_FILL_COLOR)->EnableWindow(FALSE);
		GetDlgItem(IDC_EDGE_COLOR)->EnableWindow(FALSE);
	}

	// Set the fill color
	int tmp_color = ColorToIndex(m_ptr->legend.fillColor);

	if(tmp_color < 0){
		// Custom color
		customFillColor = -tmp_color;
		m_fill_color.SetCurSel(S_CLR_CUSTOM);
	}
	else{
		customFillColor = -1;
		m_fill_color.SetCurSel(tmp_color);
	}

	// Set the edge color
	tmp_color = ColorToIndex(m_ptr->legend.edgeColor);

	if(tmp_color < 0){
		// Custom color
		customEdgeColor = -tmp_color;
		m_edge_color.SetCurSel(S_CLR_CUSTOM);
	}
	else{
		customEdgeColor = -1;
		m_edge_color.SetCurSel(tmp_color);
	}

	font = m_ptr->legend.font;
	fontColor = m_ptr->legend.fontColor;

	// Set the variables to their potentially new values
	UpdateData(FALSE);

	return TRUE;
}

void CLegendDialog::OnSelectChange()
{
	// Read in all of the variables from the dialog box
	UpdateData();
	
	if(PlotListBox::SelectedPlot() >= 0)
		tmp_legend[PlotListBox::SelectedPlot()].SetText(m_text);

	PlotListBox::OnSelectChange();

	if(PlotListBox::SelectedPlot() >= 0){
		m_text = tmp_legend[PlotListBox::SelectedPlot()].GetText();
	}

	// Set the variables to their potentially new values
	UpdateData(FALSE);
}

void CLegendDialog::OnOK() 
{
	int i;
	BOOL draw_legend = FALSE;

	// Read in all of the variables from the dialog box
	UpdateData();

	if(PlotListBox::SelectedPlot() >= 0)
		// Update the text of the selected plot
		tmp_legend[PlotListBox::SelectedPlot()].SetText(m_text, font, fontColor);

	// Cast the document pointer
	SGraph *m_ptr = &(((CSgraphDoc *)m_pDoc)->
		graph[((CSgraphDoc *)m_pDoc)->selected_graph]);

	for(i = 0;i < m_ptr->plot.Length();i++)
		if(m_ptr->plot[i].isDrawn && (i != PlotListBox::SelectedPlot())){
			// Reformat all of the legends to allow for changes in
			// font and/or font color
			tmp_legend[i].SetFont(font, fontColor);
		}
	

	for(i = 0;i < m_ptr->plot.Length();i++)
		if(m_ptr->plot[i].isDrawn){
			// Update all of the legends
			m_ptr->plot[i].legend = tmp_legend[i];

			// Only draw a legend if the user has specified at least
			// one plot legend (i.e. string length > 0).
			if(m_ptr->plot[i].legend.GetText().GetLength() > 0)
				draw_legend = TRUE; 
		}

	// Free the tmp_legend array
	delete [] tmp_legend;
	
	m_ptr->legend.isDrawn = draw_legend;

	if(!draw_legend)
		// If the legend is not being drawn, it cannot be 
		// selected.
		m_ptr->legend.isSelected = FALSE;

	m_ptr->legend.isBoundingBox = m_show_bbox;

	if(m_fill_color.GetCurSel() == S_CLR_CUSTOM){
		m_ptr->legend.fillColor = (COLORREF)(customFillColor);
	}
	else{
		m_ptr->legend.fillColor = IndexToColor(m_fill_color.GetCurSel());
	}

	if(m_edge_color.GetCurSel() == S_CLR_CUSTOM){
		m_ptr->legend.edgeColor = (COLORREF)(customEdgeColor);
	}
	else{
		m_ptr->legend.edgeColor = IndexToColor(m_edge_color.GetCurSel());
	}

	m_ptr->legend.font = font;
	m_ptr->legend.fontColor = fontColor;

	CDialog::OnOK();
}

void CLegendDialog::OnLableFontButton() 
{
	CFontDialog fDlg(&font);

	fDlg.m_cf.rgbColors = fontColor;

	if(fDlg.DoModal() == IDOK)
		fontColor = fDlg.GetColor();
}

void CLegendDialog::OnCancel() 
{
	delete [] tmp_legend;
	
	CDialog::OnCancel();
}

void CLegendDialog::OnSelchangeEdgeColor() 
{
	// Allow the user to specify a custom edge color
	if(m_edge_color.GetCurSel() == S_CLR_CUSTOM){
		COLORREF tmp_color;

		if((customEdgeColor == NO_COLOR) || (customEdgeColor > 0))
			tmp_color = customEdgeColor;
		else
			tmp_color = (COLORREF)(0);

		CPrestoColor pcDlg(tmp_color);

		if(pcDlg.DoModal() == IDOK)
			customEdgeColor = (int)(pcDlg.GetColor());
	}
}

void CLegendDialog::OnSelchangeFillColor() 
{
	// Allow the user to specify a custom fill color
	if(m_fill_color.GetCurSel() == S_CLR_CUSTOM){
		COLORREF tmp_color;

		if((customFillColor == NO_COLOR) || (customFillColor > 0))
			tmp_color = customFillColor;
		else
			tmp_color = (COLORREF)(0);

		CPrestoColor pcDlg(tmp_color);

		if(pcDlg.DoModal() == IDOK)
			customFillColor = (int)(pcDlg.GetColor());
	}
}

void CLegendDialog::OnBboxCheck() 
{
	m_show_bbox = !m_show_bbox;

	// Toggle the state of the color combo boxes
	if(m_show_bbox){
		GetDlgItem(IDC_FILL_COLOR)->EnableWindow(TRUE);
		GetDlgItem(IDC_EDGE_COLOR)->EnableWindow(TRUE);
	}
	else{
		GetDlgItem(IDC_FILL_COLOR)->EnableWindow(FALSE);
		GetDlgItem(IDC_EDGE_COLOR)->EnableWindow(FALSE);
	}
}
