//	(c) 2000-2002 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.

#include "stdafx.h"
#include "sgraph.h"
#include "sgraphdoc.h"
#include "sgraphview.h"
#include "MovieDialog.h"
#include "color.h"
#include "Excepts.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define		MAX_FPS		120

// Render in 24 bits-per-pixel color
#define		BPP			24
/////////////////////////////////////////////////////////////////////////////
// CMovieDialog dialog


CMovieDialog::CMovieDialog(CView* pView)
	: CDialog(CMovieDialog::IDD, NULL)
{
	m_pView = pView;
	render_all = TRUE;

	aviFile = NULL;
	aviStream = NULL;
	aviStreamCompressed = NULL;
	hDib = NULL;
	hOldDib = NULL;
	pBitmapBits = NULL;

	// Fill in the header for the video stream
	memset(&aviInfo, 0, sizeof(aviInfo));
	memset(&compressOpts, 0, sizeof(compressOpts));

	bitmapSize = 0;
	current_step = 0;
	frames_written = 0;

	//{{AFX_DATA_INIT(CMovieDialog)
	m_fps = 0;
	m_height = 0;
	m_name = _T("");
	m_start = 0;
	m_stop = 0;
	m_width = 0;
	//}}AFX_DATA_INIT
}

CMovieDialog::~CMovieDialog()
{
	RenderClose();
}

void CMovieDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CMovieDialog)
	DDX_Control(pDX, IDC_SPIN_WIDTH, m_spin_width);
	DDX_Control(pDX, IDC_SPIN_STOP, m_spin_stop);
	DDX_Control(pDX, IDC_SPIN_START, m_spin_start);
	DDX_Control(pDX, IDC_SPIN_HEIGHT, m_spin_height);
	DDX_Control(pDX, IDC_SPIN_FPS, m_spin_fps);
	DDX_Control(pDX, IDC_PROGRESS_MOVIE, m_progress);
	DDX_Text(pDX, IDC_EDIT_FPS, m_fps);
	DDX_Text(pDX, IDC_EDIT_HEIGHT, m_height);
	DDX_Text(pDX, IDC_EDIT_NAME, m_name);
	DDX_Text(pDX, IDC_EDIT_START, m_start);
	DDX_Text(pDX, IDC_EDIT_STOP, m_stop);
	DDX_Text(pDX, IDC_EDIT_WIDTH, m_width);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CMovieDialog, CDialog)
	//{{AFX_MSG_MAP(CMovieDialog)
	ON_BN_CLICKED(IDC_RADIO_ALL, OnRadioAll)
	ON_BN_CLICKED(IDC_RADIO_SELECTED, OnRadioSelected)
	ON_BN_CLICKED(IDC_BUTTON_BROWSE, OnButtonBrowse)
	ON_BN_CLICKED(ID_RECORD, OnRecord)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMovieDialog message handlers
BOOL CMovieDialog::OnInitDialog()
{
	CDialog::OnInitDialog();
	
	CSgraphView *pView = (CSgraphView *)(m_pView);
	
	CSgraphDoc* pDoc = pView->GetDocument();
	ASSERT_VALID(pDoc);

	// Set some sensible defaults

	// Start at plot zero
	m_start = 0;
	m_stop = pDoc->graph[pDoc->selected_graph].plot.Length() - 1;

	// A nice rate for a movie
	m_fps = 15;
	
	m_height = pView->extent.Height();
	m_width = pView->extent.Width();

	// Connect the spinners
	m_spin_fps.SetBuddy(GetDlgItem(IDC_EDIT_FPS));
	m_spin_width.SetBuddy(GetDlgItem(IDC_EDIT_WIDTH));
	m_spin_height.SetBuddy(GetDlgItem(IDC_EDIT_HEIGHT));
	m_spin_start.SetBuddy(GetDlgItem(IDC_EDIT_START));
	m_spin_stop.SetBuddy(GetDlgItem(IDC_EDIT_STOP));

	// Set the allowed spinner range
	m_spin_fps.SetRange32(1, MAX_FPS);
	m_spin_width.SetRange32(1, UD_MAXVAL);
	m_spin_height.SetRange32(1, UD_MAXVAL);
	m_spin_start.SetRange32(0, UD_MAXVAL);
	m_spin_stop.SetRange32(0, UD_MAXVAL);

	// Set the radio buttons
	if(render_all){
		CheckRadioButton(IDC_RADIO_ALL,			// First button
						 IDC_RADIO_SELECTED,	// Last bottom	
						 IDC_RADIO_ALL);

		GetDlgItem(IDC_EDIT_START)->EnableWindow(FALSE);
		GetDlgItem(IDC_EDIT_STOP)->EnableWindow(FALSE);
		GetDlgItem(IDC_SPIN_STOP)->EnableWindow(FALSE);
		GetDlgItem(IDC_SPIN_START)->EnableWindow(FALSE);
	}
	else{
		CheckRadioButton(IDC_RADIO_ALL,			// First button
						 IDC_RADIO_SELECTED,	// Last bottom	
						 IDC_RADIO_SELECTED);

		GetDlgItem(IDC_EDIT_START)->EnableWindow(TRUE);
		GetDlgItem(IDC_EDIT_STOP)->EnableWindow(TRUE);
		GetDlgItem(IDC_SPIN_STOP)->EnableWindow(TRUE);
		GetDlgItem(IDC_SPIN_START)->EnableWindow(TRUE);
	}

	UpdateData(FALSE);

	return TRUE;
}

void CMovieDialog::OnRadioAll() 
{
	if(!render_all){
		render_all = TRUE;
		
		GetDlgItem(IDC_EDIT_START)->EnableWindow(FALSE);
		GetDlgItem(IDC_EDIT_STOP)->EnableWindow(FALSE);
		GetDlgItem(IDC_SPIN_STOP)->EnableWindow(FALSE);
		GetDlgItem(IDC_SPIN_START)->EnableWindow(FALSE);
	}
}

void CMovieDialog::OnRadioSelected() 
{
	if(render_all){
		render_all = FALSE;
		
		GetDlgItem(IDC_EDIT_START)->EnableWindow(TRUE);
		GetDlgItem(IDC_EDIT_STOP)->EnableWindow(TRUE);
		GetDlgItem(IDC_SPIN_STOP)->EnableWindow(TRUE);
		GetDlgItem(IDC_SPIN_START)->EnableWindow(TRUE);
	}
}

void CMovieDialog::OnButtonBrowse() 
{
	CFileDialog fDlg(FALSE /* Save As*/ , "*.avi", m_name);

	fDlg.m_ofn.lpstrTitle = "Specify AVI file to create";
	fDlg.m_ofn.lpstrFilter = "AVI files\0*.avi\0\0";

	// Set the current directory
	char dirBuffer[1024];

	// If the buffer is not big enought, DON'T set the current 
	// directory
	if(GetCurrentDirectory(1023, dirBuffer) != 0)
		fDlg.m_ofn.lpstrInitialDir = dirBuffer;

	if(fDlg.DoModal() == IDOK){
		m_name = fDlg.GetPathName();

		UpdateData(FALSE);
	}
}

void CMovieDialog::OnRecord() 
{
	UpdateData(TRUE);

	CSgraphView *pView = (CSgraphView *)(m_pView);
	
	CSgraphDoc* pDoc = pView->GetDocument();
	ASSERT_VALID(pDoc);

	// Make sure that the user has enabled plot animation
	if(pDoc->graph[pDoc->selected_graph].animate_plots == FALSE){
		AfxMessageBox("Please enable plot animation in the Plot Options dialog "
			"before exporting a movie.");
		return;
	}

	int max_step = pDoc->graph[pDoc->selected_graph].plot.Length() - 1;

	if(render_all){
		current_step = 0;
		m_stop = max_step;
	}
	else{
		current_step = m_start;
	}
	
	if((current_step < 0) || (m_stop > max_step) || (current_step >= m_stop)){
		AfxMessageBox("Please enter a valid step range!");
		return;
	}
	
	m_progress.SetRange32(0, m_stop - current_step + 1);
	m_progress.SetStep(1);

	if((m_fps < 1) || (m_fps >= MAX_FPS)){
		CString errorMsg;

		errorMsg.Format("Please enter a frame rate between 1 and %d", MAX_FPS);
		AfxMessageBox(errorMsg);
		return;
	}

	if(m_width < 1){
		AfxMessageBox("Please enter a valid width");
		return;
	}

	if(m_height < 1){
		AfxMessageBox("Please enter a valid height");
		return;
	}

	int name_len = m_name.GetLength();
	if(name_len == 0){
		AfxMessageBox("Please enter a file name");
		return;
	}

	// The file name MUST end with ".avi"!
	if((name_len < 4) || 
	   (m_name[name_len - 4] != '.') || 
	   (tolower(m_name[name_len - 3]) != 'a') || 
	   (tolower(m_name[name_len - 2]) != 'v') ||
	   (tolower(m_name[name_len - 1]) != 'i')){
		AfxMessageBox("The AVI output file MUST end with a '.avi' extension.");
		return;
	}

	HCURSOR prevCursor = ::SetCursor(LoadCursor(NULL, IDC_WAIT));

	// Initialize the movie
	try{
	
		if(!RenderInit()){
			// The user has canceled the operation
			::SetCursor(prevCursor);
			return;
		}

		if(compressOpts.hic){
			if(!ICSeqCompressFrameStart(&compressOpts, &bmpInfoInput)){
				throw Error("Can't initialize compressor!");
			}
		}

		while(current_step <= m_stop){
			RenderPlot(current_step);
			
			current_step++;

			m_progress.StepIt();
		}

		ICSeqCompressFrameEnd(&compressOpts);
 
		RenderClose();

		::SetCursor(prevCursor);
	}
	catch(Error &e){

		AfxMessageBox(e.message());

		ICSeqCompressFrameEnd(&compressOpts);

		RenderClose();

		::SetCursor(prevCursor);
		OnCancel();
		return;
	}

	AfxMessageBox("Movie creation successfull!");
	OnCancel();
}

// Initialize the AVI rendering code. Return TRUE if successfull, false
// if the operation has been canceled by the user.
BOOL CMovieDialog::RenderInit()
{	
	// Perform some sanity checking
	if(m_width <= 0)
		throw Error("Frame width out of bounds", __FILE__, __LINE__);

	if(m_height <= 0)
		throw Error("Frame height out of bounds", __FILE__, __LINE__);

	if(m_fps <= 0)
		throw Error("Frame rate out of bounds", __FILE__, __LINE__);
	
	// We must have a valid pointer to a view
	if(!m_pView)
		throw Error("Bad view pointer", __FILE__, __LINE__);

	CSgraphView *pView = (CSgraphView *)(m_pView);

	// Make sure we are running on 1.1 
	if (HIWORD(VideoForWindowsVersion()) < 0x010a)
		throw Error("The current version of VideoForWindows is too old!",
			__FILE__, __LINE__);

	// Everything looks good so far. Open an AVI file and stream.
	AVIFileInit();

	// Note that this function requires that the filename end in ".avi"
	// (because the last argument is NULL)!
	if(AVIFileOpen(&aviFile, LPCTSTR(m_name), OF_CREATE | OF_WRITE, NULL) != AVIERR_OK)
		throw Error("Error opening the AVI file", __FILE__, __LINE__);

	// stream type
	aviInfo.fccType = streamtypeVIDEO;
	aviInfo.fccHandler = 0;
	aviInfo.dwScale = 1;

	// The frame rate
	aviInfo.dwRate = m_fps;

	// size = width*height*bytes-per-pixel
	aviInfo.dwSuggestedBufferSize = m_width*m_height*(BPP/8);

	// rectangle for stream
	SetRect(&aviInfo.rcFrame, 0, 0, m_width, m_height);

	// And create the stream;
	if(AVIFileCreateStream(aviFile, &aviStream, &aviInfo) != AVIERR_OK)
		throw Error("Error opening the AVI stream");

	// Initialize the input bitmap header information...
	memset(&bmpInfoInput, 0, sizeof(BITMAPINFO));
	bmpInfoInput.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
	
	bmpInfoInput.bmiHeader.biWidth = m_width;
	bmpInfoInput.bmiHeader.biHeight = m_height;
	bmpInfoInput.bmiHeader.biPlanes = 1;
	bmpInfoInput.bmiHeader.biBitCount = BPP;
	bmpInfoInput.bmiHeader.biCompression = BI_RGB;
	bmpInfoInput.bmiHeader.biSizeImage = bmpInfoInput.bmiHeader.biWidth * 
		bmpInfoInput.bmiHeader.biHeight * (bmpInfoInput.bmiHeader.biBitCount/8);

	memset(&compressOpts, 0, sizeof(compressOpts));

	// Set this flag to indicate that we've manually "adjusted" the
	// compressOpts variable.
	compressOpts.cbSize = sizeof(COMPVARS);

	// Pop up a dialog to let the user choose the form of the compression
	// (including the option for no compression).
	ICCompressorChoose(this->GetSafeHwnd(),      
		0,   
		&bmpInfoInput,
		aviStream,  
		&compressOpts,   
		"Compression Options");

	// If the user chooses "no compression", compressOpts.hic will be NULL but
	// fccHandler will equal mmioFOURCC('D', 'I', 'B', ' ').
	if((compressOpts.fccHandler != mmioFOURCC('D', 'I', 'B', ' ')) && !compressOpts.hic){
		// cancle the record command after cleaning up memroy and files
		RenderClose();
		return FALSE;
	}

	// Get the output format for the compressor (if needed)
	if(compressOpts.hic){
		// Initialize the output bitmap header information...
		memset(&bmpInfoOutput, 0, sizeof(BITMAPINFO));

		if(ICCompressGetFormat(compressOpts.hic, &bmpInfoInput, &bmpInfoOutput) != ICERR_OK){
			throw Error("Can't get compression format!");
		}
	}
 
	// For debugging purposes, grab the four chacter codes used to describe
	// compression formats
	//char fcc[4];
	//fcc[0] = (char)(compressOpts.fccHandler & 0x000000ff);
	//fcc[1] = (char)((compressOpts.fccHandler & 0x0000ff00) >> 8);
	//fcc[2] = (char)((compressOpts.fccHandler & 0x00ff0000) >> 16);
	//fcc[3] = (char)((compressOpts.fccHandler & 0xff000000) >> 24);

	AVICOMPRESSOPTIONS opts;
	
	// Set the AVICOMPRESSOPTIONS structure based on the contents of compressOpts
	opts.fccType = streamtypeVIDEO;
    opts.fccHandler = compressOpts.fccHandler;
    opts.dwKeyFrameEvery = 0;
    opts.dwQuality = compressOpts.lQ; 
    opts.dwBytesPerSecond = 0; 
    opts.dwFlags = AVICOMPRESSF_VALID; 
    opts.lpFormat = 0; 
    opts.cbFormat = 0; 
    opts.lpParms = compressOpts.lpState; 
    opts.cbParms = compressOpts.cbState; 
    opts.dwInterleaveEvery = 0; 
 
	if(AVIMakeCompressedStream(&aviStreamCompressed, aviStream, &opts, NULL) != AVIERR_OK)
		throw Error("Couldn't make compressed AVI stream!");

	///////////////////// Create the frame in memeory /////////////////////
	// Create a DIB in memory that we can render to.
	CDC *pDC = pView->GetDC();

	hDib = CreateDIBSection(pDC->GetSafeHdc(), &bmpInfoInput, DIB_RGB_COLORS, &pBitmapBits, 
		NULL, (DWORD)0);

	// Save the image size in bytes
	bitmapSize = bmpInfoInput.bmiHeader.biSizeImage;

	// Set the stream format
	if(AVIStreamSetFormat(aviStreamCompressed, 0, &(bmpInfoInput.bmiHeader), 
		sizeof(BITMAPINFOHEADER)) != AVIERR_OK)
		throw Error("Couldn't set compressed AVI format!");

	// Create memory device context
	renderDC.CreateCompatibleDC(pDC);

	hOldDib = renderDC.SelectObject(hDib);

	pView->ReleaseDC(pDC);

	return TRUE;
}

// Clean up the AVI rendering code
void CMovieDialog::RenderClose()
{

	if(compressOpts.hic){
		ICCompressorFree(&compressOpts);
		memset(&compressOpts, 0, sizeof(compressOpts));
	}

	// Select "out" the DIB before deleting it
	if(hOldDib){
		renderDC.SelectObject(hOldDib);
		hOldDib = NULL;
	}

	// Delete our DIB and device context
	if(hDib){
		DeleteObject(hDib);
		hDib = NULL;
	}

	pBitmapBits = NULL;

	bitmapSize = 0;

	if(aviStreamCompressed){
		AVIStreamRelease(aviStreamCompressed);
		aviStreamCompressed = NULL;
	}

	if(aviStream){
		AVIStreamRelease(aviStream);
		aviStream = NULL;
	}

	if(aviFile){
		AVIFileRelease(aviFile);
		aviFile = NULL;

		AVIFileExit();
	}

	// Fill in the header for the video stream
	memset(&aviInfo, 0, sizeof(aviInfo));

}

// Render a single frame of the AVI movie
void CMovieDialog::RenderPlot(int cur_plot)
{
	CSgraphView *pView = (CSgraphView *)(m_pView);

	CSgraphDoc* pDoc = pView->GetDocument();
	ASSERT_VALID(pDoc);

	int tmp_curr_plot = pDoc->graph[pDoc->selected_graph].curr_animate_plot;
	int i;

	// Make sure that plot animation is enabled
	pDoc->graph[pDoc->selected_graph].animate_plots = TRUE;
	pDoc->graph[pDoc->selected_graph].curr_animate_plot = cur_plot;

	// Force the OnDraw() function to render to our DIB (regardless of the
	// state of the double buffering).
	pView->isPrinting = TRUE;

	int tmp_width, tmp_height;

	// Save the current viewport dimensions
	tmp_width = pView->extent.Width();
	tmp_height = pView->extent.Height();

	pView->extent = CRect(0, 0, m_width, m_height);

	// Draw a background color
	// If the current background color is NO_COLOR, 
	// use the system window color.
	CBrush bkgrnd(pDoc->background_color == NO_COLOR ? 
		GetSysColor(COLOR_WINDOW): pDoc->background_color);

	renderDC.FillRect(pView->extent, &bkgrnd);

	// Draw to the device context.
	// The isPrinting flag disables the graph selection boxes
	for(i = 0;i < pDoc->annotate_text.Length();i++)
		pDoc->annotate_text[i].Printing(TRUE);

	for(i = 0;i < pDoc->num_graph;i++)
		pDoc->graph[i].Printing(TRUE);

	pView->OnDraw(&renderDC);

	for(i = 0;i < pDoc->annotate_text.Length();i++)
		pDoc->annotate_text[i].Printing(FALSE);

	for(i = 0;i < pDoc->num_graph;i++)
		pDoc->graph[i].Printing(FALSE);

	// Restore the original viewport dimensions
	pView->extent = CRect(0, 0, tmp_width, tmp_height);

	// Restore the "is printing" flag
	pView->isPrinting = FALSE;

	// Restore the current animiated plot
	pDoc->graph[pDoc->selected_graph].curr_animate_plot = tmp_curr_plot;

	//////////////////////// Compress the frame ////////////////////////
	
	if(compressOpts.hic){
		BOOL isKeyFrame;
		LONG compressedSize = 0L;
		VOID *compressBuffer;

		compressBuffer = ICSeqCompressFrame(&compressOpts, 0,  
			pBitmapBits, &isKeyFrame,  &compressedSize);
 
		if(!compressBuffer)
			throw Error("Error compressing frame");

		//////////////////////// Write out the compressed frame ////////////////////////
		// An open question (whose answer is most likely a mistake on my part) -- Why does
		// AVIStreamWrite() take aviStream and not aviStreamCompressed as the argument here?
		// I've tried passing aviStreamCompressed, but then AVIStreamWrite() exits with an error.
		if(AVIStreamWrite(aviStream,	// stream pointer
		   frames_written,
		   1,							// number to write
		   compressBuffer,				// pointer to the compressed data
		   compressedSize,				// size of this frame
		   AVIIF_KEYFRAME,				// flags....
		   NULL,
		   NULL) != AVIERR_OK){
			throw Error("Can't write movie frame!");
		}
	}
	else{
		//////////////////////// Write out the uncompressed frame ////////////////////////
		if(AVIStreamWrite(aviStream,	// stream pointer
		   frames_written,
		   1,							// number to write
		   pBitmapBits,					// pointer to the compressed data
		   bitmapSize,					// size of this frame
		   AVIIF_KEYFRAME,				// flags....
		   NULL,
		   NULL) != AVIERR_OK){
			throw Error("Can't write movie frame!");
		}
	}

	// Increment the frame counter
	frames_written++;
}