//	(c) 2000-2002 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.
#if !defined(AFX_NONLINFITDIALOG_H__CE379347_9E68_45DA_A47A_AC42E67E559C__INCLUDED_)
#define AFX_NONLINFITDIALOG_H__CE379347_9E68_45DA_A47A_AC42E67E559C__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
// NonLinFitDialog.h : header file
//

#include "equation/Equation.h"
#include "list.h"
#include "array.h"
#include "history.h"

// Include the TNT linear algebra libraries
#include "tnt/tnt.h"
#include "tnt/vec.h"
#include "tnt/cmat.h"
#include "tnt/lu.h"

using namespace std;
using namespace TNT;

/////////////////////////////////////////////////////////////////////////////
// CNonLinFitDialog dialog
enum {FIT_LEVENBERG_MARQUARDT = 0};

// Which stage of the curve fit are we in?
enum {FIT_FUNCTION = 0,		// The user must enter a function
	  FIT_SET_PARAM,		// The user must specify initial parameter values
	  FIT_ADJUST_PARAM,		// The user may adjust the parameters
	  FIT_CURVE_FIT};		// Fitting the curve

#define		FIT_TOL			1.0e-6f
#define		FIT_MAX_ITER	1000
#define		FIT_METHOD

// The number of "ticks" for the parameter slider
#define		FIT_SLIDER_SIZE		100

// The fractional size of the slider (in units of the parameter
// value) to use when an explicit value for the min and max
// have not been set.
#define		FIT_SLIDER_FRACT_SIZE		0.50f


// The default weight for each point in the chi^2 sum
#define		DEFAULT_SIGMA		1.0f

// Message semaphores
#define		WM_STOP_FIT			(WM_USER + 8)
#define		WM_UPDATE_FIT		(WM_USER + 9)

// a class for tracking fit parameters
class FitParam{
private:
public:
	FitParam()
	{
		index = -1;
		val = 1.0f;
		max_val = min_val = 1.0f;
		is_max = is_min = is_val = FALSE;
	}

	// An index into the equation class
	int index;

	// The parameter value
	float val;

	// Has the value been set?
	BOOL is_val;

	// Has a maximum value been set?
	BOOL is_max;

	// Has a minimum value been set?
	BOOL is_min;

	// Has the minimizer run up to an edge?
	BOOL max_contrained;
	BOOL min_contrained;

	// The max and min allowed parameter values
	float max_val;
	float min_val;

	// When adjusting the parameter values by hand, what should
	// the slider limits be?
	float adjust_max;
	float adjust_min;
};

class CNonLinFitDialog : public CDialog
{
// Construction
public:
	CNonLinFitDialog(CView *m_pView);

	CView *pView;

	// A pointer to the current graph
	SGraph *m_graph_ptr;

	BOOL OnInitDialog();

	void ActivateStage();

	void SetParamList(BOOL m_initList);
	void SetParamItem(int m_i, BOOL m_initList);

	// Save the current parameter entries from the dialog
	// edit boxes.
	void SaveCurParam();

	// Fill the parameter value edit boxes according to the current
	// parameter.
	void LoadCurParam();

	// Change the current stage of the fitting process
	BOOL ChangeStage();

	void SetSlider();

	// Plot a trial function and return chi^2
	float PlotFit();

	// Compute the Chi Square for the current parameter
	// set.
	float ChiSquare(BOOL m_display);

	void StartCurveFit();

	// Stop the curve fitting thread
	BOOL m_stop_fit;

	// Compute the curve fit using the Levenburg-Marquardt method
	void CurveFitLM();
	void ComputeAlpha(Matrix<float> &alpha, float lambda, 
		array<SEquation> &deriv);
	void ComputeBeta(Vector<float> &beta, array<SEquation> &deriv);

	// Compute a discrete derivative in case the analytic derivative fails!
	float DiscreteDeriv(int i, float x);
	
	void SaveEquationHistory();

	// Fit options
	int m_fit_step;
	int fit_method;
	int fit_iter;
	float fit_tol;
	
	// Number of points to use when saving a plot
	int num_save_plot_points;

	// Should we simplify analytic derivatives?
	BOOL fit_simplify;

	// Should we use y error bars to weight chi^2?
	BOOL fit_use_y_error_bar;

	// The currently selected parameter
	int cur_sel_param;

	// Which plot are we fitting?
	int fit_plot;

	// A list of parameters to fit
	list<FitParam> param;

	// The equation that we'll be fitting
	SEquation fit_eq;

// Dialog Data
	//{{AFX_DATA(CNonLinFitDialog)
	enum { IDD = IDD_NON_LIN_FIT_DIALOG };
	CTabCtrl	m_eq_tab;
	CListCtrl	m_param;
	CSliderCtrl	m_param_slider;
	CListBox	m_data;
	CString	m_chi2;
	CString	m_function;
	CString	m_max_param;
	CString	m_min_param;
	CString	m_value_param;
	//}}AFX_DATA


// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CNonLinFitDialog)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:

	// Generated message map functions
	//{{AFX_MSG(CNonLinFitDialog)
	afx_msg void OnNextButton();
	afx_msg void OnPrevButton();
	afx_msg void OnFitButton();
	afx_msg void OnClickParamList(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);
	afx_msg LRESULT StopCurveFit(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT UpdateCurveFit(WPARAM wParam, LPARAM lParam);
	virtual void OnCancel();
	afx_msg void OnOptionsButton();
	afx_msg void OnSelchangeDataList();
	afx_msg void OnSaveButton();
	afx_msg void OnSelchangeEquationTab(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnSelchangingEquationTab(NMHDR* pNMHDR, LRESULT* pResult);
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_NONLINFITDIALOG_H__CE379347_9E68_45DA_A47A_AC42E67E559C__INCLUDED_)
