//	(c) 2000-2002 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.

#include "stdafx.h"
#include "sgraph.h"
#include "NonLinFitOptions.h"
#include "NonLinFitDialog.h"
#include "PrestoUtil.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CNonLinFitOptions dialog


CNonLinFitOptions::CNonLinFitOptions(int method, int iter, float tol, BOOL simplify,
									 BOOL use_error_bar)
	: CDialog(CNonLinFitOptions::IDD, NULL)
{
	//{{AFX_DATA_INIT(CNonLinFitOptions)
	m_max_iter = _T("");
	m_tol = _T("");
	m_simplify = simplify;
	m_use_error_bar = use_error_bar;
	//}}AFX_DATA_INIT

	// Save pointers to the user inputs
	tmp_method = method;
	tmp_iter = iter;
	tmp_tol = tol;
}


void CNonLinFitOptions::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CNonLinFitOptions)
	DDX_Control(pDX, IDC_METHOD_COMBO, m_method);
	DDX_Text(pDX, IDC_ITER_EDIT, m_max_iter);
	DDX_Text(pDX, IDC_TOL_EDIT, m_tol);
	DDX_Check(pDX, IDC_SIMPLIFY_CHECK, m_simplify);
	DDX_Check(pDX, IDC_Y_ERROR_CHECK, m_use_error_bar);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CNonLinFitOptions, CDialog)
	//{{AFX_MSG_MAP(CNonLinFitOptions)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CNonLinFitOptions message handlers
BOOL CNonLinFitOptions::OnInitDialog()
{
	CDialog::OnInitDialog();

	switch(tmp_method){
		case FIT_LEVENBERG_MARQUARDT:
		default:
			// By default, use FIT_LEVENBERG_MARQUARDT as the 
			// minimization method
			
			m_method.SetCurSel(FIT_LEVENBERG_MARQUARDT);

			// Default fit tolerance
			m_tol.Format("%g", tmp_tol);

			// Default number of iterations
			m_max_iter.Format("%d", tmp_iter);

			break;
	}

	UpdateData(FALSE);

	return TRUE;
}

void CNonLinFitOptions::OnOK() 
{
	// Collect the selected method, tol and max steps
	UpdateData(TRUE);

	tmp_method = m_method.GetCurSel();
	
	float tmp_float;

	if(StringToFloat(m_max_iter, tmp_float) == FALSE){
		AfxMessageBox("Invalid Max Iterations");
		return;
	}

	if(tmp_float <= 0){
		AfxMessageBox("Invalid Max Iterations ( < 0)");
		return;
	}
		
	tmp_iter = (int)(tmp_float);

	if(StringToFloat(m_tol, tmp_tol) == FALSE){
		AfxMessageBox("Invalid Fit Tolerance");
		return;
	}

	CDialog::OnOK();
}
