//	(c) 2000-2002 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.
#include "stdafx.h"
#include "PlotList.h"
#include "sgraph.h"
#include "sgraphDoc.h"

void PlotListBox::OnInitDialog(CListBox &m_list, CDocument *m_pDoc)
{
	int i, plot_count;
	int max_string_length = 0;
	char buffer[1024];
	int first_plot = -1;

	// Save a point to the list and document
	list_ptr = &m_list;
	pDoc = m_pDoc;

	ASSERT_VALID(pDoc);

	if(list_ptr == NULL)
		throw Error("NULL list pointer", __FILE__, __LINE__);

	// Cast the document pointer
	SGraph *m_ptr = &(((CSgraphDoc *)pDoc)->
		graph[((CSgraphDoc *)pDoc)->selected_graph]);

	// Fill the list with the plot labels
	for(i = 0, plot_count = 0;i < m_ptr->plot.Length();i++){
		if((plot_count <= MAX_LIST_BOX_ENTRIES) && (m_ptr->plot[i].isDrawn)){
			// The set number is determined when the plots are
			// first created (or read in) - hence use the index number
			// to lable the plot, not the plot_count.
			sprintf(buffer,"Set #%d (N = %d) \"%s\"", i,
				m_ptr->plot[i].data.Length(), LPCTSTR(m_ptr->plot[i].info));
		
			// Keep track of the maximum string length (in pixels).
			// See comments below regarding this kludge.
			int len = strlen(buffer);
			CDC *pDC = list_ptr->GetDC();

			if(pDC->GetTextExtent(buffer, len).cx > max_string_length)
				max_string_length = pDC->GetTextExtent(buffer, len).cx;
			
			list_ptr->ReleaseDC(pDC);

			list_ptr->AddString(buffer);
			plot_count++;

			// Start with a blank slate (i.e. no plot selected).
			m_ptr->plot[i].isSelected = FALSE;

			// Record the first plot
			if(first_plot < 0)
				first_plot = i;
		}
	}
	
	// Explain to users that they've encountered a limitation of the 
	// MFC CListBox class!
	if(plot_count > MAX_LIST_BOX_ENTRIES){
		AfxMessageBox("The maximum number of plots that can be displayed in a\r\n"
			"ListBox has been reached - plot list has been truncated!");
	}

	if(first_plot >= 0){
		// Save the id of the selected plot
		selected_plot = first_plot;

		// Select the first plot (if there is one)
		m_ptr->plot[first_plot].isSelected = TRUE;

		// The first plot corresponds to the first string, which
		// we need to select. The selection method used depends on
		// the type of list box -- either single or extended selection.
		if(list_ptr->GetStyle() & LBS_EXTENDEDSEL){
			list_ptr->SetSel(0);	// Extended selection
		}
		else{
			list_ptr->SetCurSel(0); // Single selection
		}
	}

	// Set the width of the list box for proper horizontal
	// scrolling (a kludge to get around horizontal scroll
	// limitations of the CListBox).
	list_ptr->SetHorizontalExtent(max_string_length);
}


void PlotListBox::OnSelectChange()
{
	int i, plot_count;

	ASSERT_VALID(pDoc);

	if(list_ptr == NULL)
		throw Error("NULL list pointer", __FILE__, __LINE__);

	// Cast the document pointer
	SGraph *m_ptr = &(((CSgraphDoc *)pDoc)->
		graph[((CSgraphDoc *)pDoc)->selected_graph]);

	for(i = 0, plot_count = 0;i < m_ptr->plot.Length();i++){
		if(m_ptr->plot[i].isDrawn){
			if(list_ptr->GetSel(plot_count)){
				// Save the selected plot number
				selected_plot = i;

				m_ptr->plot[i].isSelected = TRUE;
			}
			else{
				m_ptr->plot[i].isSelected = FALSE;
			}
			plot_count ++;
		}
	}
}


