//	(c) 2000 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.

#include "stdafx.h"
#include "sgraph.h"
#include "sgraphDoc.h"
#include "PlotOptionsDialog.h"
#include "color.h"
#include "PrestoColor.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// We need to know the 16 bit status of the OS when allowing the user
// to select a lines style.
BOOL extern is_16_bit;

/////////////////////////////////////////////////////////////////////////////
// CPlotOptionsDialog dialog


CPlotOptionsDialog::CPlotOptionsDialog(CDocument* pDoc, CWnd* pParent /*=NULL*/)
	: CDialog(CPlotOptionsDialog::IDD, pParent)
{
	m_pDoc = pDoc;

	//{{AFX_DATA_INIT(CPlotOptionsDialog)
	m_animate_plots = FALSE;
	//}}AFX_DATA_INIT
}


void CPlotOptionsDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CPlotOptionsDialog)
	DDX_Control(pDX, IDC_SYMBOL_TYPE, m_symbol_type);
	DDX_Control(pDX, IDC_SYMBOL_SIZE, m_symbol_size);
	DDX_Control(pDX, IDC_SYMBOL_COLOR, m_symbol_color);
	DDX_Control(pDX, IDC_PLOT_LIST, m_plot_list);
	DDX_Control(pDX, IDC_LINE_TYPE, m_line_type);
	DDX_Control(pDX, IDC_LINE_SIZE, m_line_size);
	DDX_Control(pDX, IDC_LINE_COLOR, m_line_color);
	DDX_Check(pDX, IDC_ANIMATE_CHECK, m_animate_plots);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CPlotOptionsDialog, CDialog)
	//{{AFX_MSG_MAP(CPlotOptionsDialog)
	ON_BN_CLICKED(IDAPPLY, OnApply)
	ON_LBN_SELCHANGE(IDC_PLOT_LIST, OnSelectChange)
	ON_CBN_SELCHANGE(IDC_LINE_COLOR, OnSelchangeLineColor)
	ON_CBN_SELCHANGE(IDC_SYMBOL_COLOR, OnSelchangeSymbolColor)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CPlotOptionsDialog message handlers

void CPlotOptionsDialog::SetDefaultOptions()
{
	// No custom colors are loaded initially
	custom_symbol_color = -1;
	custom_line_color = -1;

	m_symbol_type.
		SetCurSel(SPlot::POINT_CIRCLE);
	m_symbol_size.
		SetCurSel(0);
	m_symbol_color.
		SetCurSel(0); // Black

	m_line_type.
		SetCurSel(SPlot::LINE_SOLID);
	m_line_size.
		SetCurSel(0);
	m_line_color.
		SetCurSel(0);	
}

BOOL CPlotOptionsDialog::OnInitDialog()
{	
	// Cast the document pointer
	SGraph *m_ptr = &(((CSgraphDoc *)m_pDoc)->
		graph[((CSgraphDoc *)m_pDoc)->selected_graph]);

	// Set the check box for the animate plot option
	m_animate_plots = m_ptr->animate_plots;

	CDialog::OnInitDialog();

	// Initialize the list box
	PlotListBox::OnInitDialog(m_plot_list, m_pDoc);

	if(PlotListBox::SelectedPlot() >= 0){
		// Select the first plot (if there is one)
		m_ptr->plot[PlotListBox::SelectedPlot()].isSelected = TRUE;
		
		// Read in the Selected plot options
		m_symbol_type.
			SetCurSel(m_ptr->plot[PlotListBox::SelectedPlot()].pointType);
		m_symbol_size.
			SetCurSel(m_ptr->plot[PlotListBox::SelectedPlot()].pointSize - 1);
		
		int tmp_color = ColorToIndex(m_ptr->plot[PlotListBox::SelectedPlot()].brush.lbColor);

		if(tmp_color < 0){
			// Custom color
			custom_symbol_color = -tmp_color;
			m_symbol_color.SetCurSel(S_CLR_CUSTOM);
		}
		else
			m_symbol_color.SetCurSel(tmp_color);

		m_line_type.
			SetCurSel(m_ptr->plot[PlotListBox::SelectedPlot()].lineType);
		m_line_size.
			SetCurSel(m_ptr->plot[PlotListBox::SelectedPlot()].lineSize - 1);
		
		tmp_color = ColorToIndex(m_ptr->plot[PlotListBox::SelectedPlot()].lineColor);

		if(tmp_color < 0){
			// Custom color
			custom_line_color = -tmp_color;
			m_line_color.SetCurSel(S_CLR_CUSTOM);
		}
		else
			m_line_color.SetCurSel(tmp_color);
	}
	else{
		SetDefaultOptions();
	}

	return TRUE;
}

void CPlotOptionsDialog::OnSelectChange()
{
	PlotListBox::OnSelectChange();

	if(PlotListBox::SelectedPlot() >= 0){

		// Cast the document pointer
		SGraph *m_ptr = &(((CSgraphDoc *)m_pDoc)->
			graph[((CSgraphDoc *)m_pDoc)->selected_graph]);

		m_symbol_type.SetCurSel(m_ptr->plot[PlotListBox::SelectedPlot()].pointType);
		m_symbol_size.SetCurSel(m_ptr->plot[PlotListBox::SelectedPlot()].pointSize - 1);

		int tmp_color = ColorToIndex(m_ptr->plot[PlotListBox::SelectedPlot()].brush.lbColor);

		if(tmp_color < 0){
			// Custom color
			custom_symbol_color = -tmp_color;
			m_symbol_color.SetCurSel(S_CLR_CUSTOM);
		}
		else
			m_symbol_color.SetCurSel(tmp_color);

		m_line_type.SetCurSel(m_ptr->plot[PlotListBox::SelectedPlot()].lineType);
		m_line_size.SetCurSel(m_ptr->plot[PlotListBox::SelectedPlot()].lineSize - 1);

		tmp_color = ColorToIndex(m_ptr->plot[PlotListBox::SelectedPlot()].lineColor);

		if(tmp_color < 0){
			// Custom color
			custom_line_color = -tmp_color;
			m_line_color.SetCurSel(S_CLR_CUSTOM);
		}
		else
			m_line_color.SetCurSel(tmp_color);

		return;
	}
	else{
		SetDefaultOptions();
	}
}

void CPlotOptionsDialog::OnCancel() 
{
	// Nothing to do here!
	
	CDialog::OnCancel();
}

void CPlotOptionsDialog::OnOK() 
{
	OnApply();

	CDialog::OnOK();
}

void CPlotOptionsDialog::OnApply()
{	
	UpdateData();

	// Cast the document pointer
	SGraph *m_ptr = &(((CSgraphDoc *)m_pDoc)->
		graph[((CSgraphDoc *)m_pDoc)->selected_graph]);

	m_ptr->animate_plots = m_animate_plots;

	BOOL update = FALSE;
	
	// Give a warning if the user attempts to use thick lines with
	// a non-solid pattern (but only on 16 bit systems).
	BOOL MFCLineBugWarning = is_16_bit;
	
	// Apply changes to all selected plots
	for(int i = 0;i < m_ptr->plot.Length();i++){
		if(m_ptr->plot[i].isSelected){
			update = TRUE;
			m_ptr->plot[i].pointType = m_symbol_type.GetCurSel();
			m_ptr->plot[i].pointSize = m_symbol_size.GetCurSel() + 1;

			if(m_symbol_color.GetCurSel() == S_CLR_CUSTOM)
				m_ptr->plot[i].brush.lbColor = (COLORREF)(custom_symbol_color);
			else
				m_ptr->plot[i].brush.lbColor = 
					IndexToColor(m_symbol_color.GetCurSel());

			if(m_ptr->plot[i].brush.lbColor == NO_COLOR)
				// The user has requested no color (i.e. no fill).
				m_ptr->plot[i].brush.lbStyle = BS_HOLLOW;
			else
				m_ptr->plot[i].brush.lbStyle = BS_SOLID;

			m_ptr->plot[i].lineType = m_line_type.GetCurSel();
			m_ptr->plot[i].lineSize = m_line_size.GetCurSel() + 1;
				
			if(MFCLineBugWarning && (m_ptr->plot[i].lineType != SPlot::LINE_SOLID) && 
				(m_ptr->plot[i].lineSize > 1)){

				// Turn off the warning
				MFCLineBugWarning = FALSE;
				AfxMessageBox("Warning! Only solid lines can have a size greater than 1."
					" Line will appear solid.");
			}

			if(m_line_color.GetCurSel() == S_CLR_CUSTOM)
				m_ptr->plot[i].lineColor = (COLORREF)(custom_line_color);
			else
				m_ptr->plot[i].lineColor = 
					IndexToColor(m_line_color.GetCurSel());
		}
	}

	if(update){
		m_pDoc->UpdateAllViews(NULL);
	}
	
}

void CPlotOptionsDialog::OnSelchangeLineColor() 
{
	// Allow the user to specify a custom line color
	if(m_line_color.GetCurSel() == S_CLR_CUSTOM){
		COLORREF tmp_color;

		if((custom_line_color == NO_COLOR) || (custom_line_color > 0))
			tmp_color = custom_line_color;
		else
			tmp_color = (COLORREF)(0);

		CPrestoColor pcDlg(tmp_color);

		if(pcDlg.DoModal() == IDOK)
			custom_line_color = (int)(pcDlg.GetColor());
		else
			// If the user decides not to pick a custom color
			// from the dialog box, reset the list box (and every
			// thing else) to its original state.
			OnSelectChange();
	}
}

void CPlotOptionsDialog::OnSelchangeSymbolColor() 
{
	// Allow the user to specify a custom symbol color
	if(m_symbol_color.GetCurSel() == S_CLR_CUSTOM){
		COLORREF tmp_color;

		if((custom_line_color == NO_COLOR) || (custom_line_color > 0))
			tmp_color = custom_symbol_color;
		else
			tmp_color = (COLORREF)(0);

		CPrestoColor pcDlg(tmp_color);

		if(pcDlg.DoModal() == IDOK)
			custom_symbol_color = (int)(pcDlg.GetColor());
		else
			// If the user decides not to pick a custom color
			// from the dialog box, reset the list box (and every
			// thing else) to its original state.
			OnSelectChange();
	}
}
