//	(c) 2000 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.

#include "stdafx.h"
#include "sgraph.h"
#include "sgraphDoc.h"
#include "PlotStatisticsDialog.h"
#include "TextOutputDialog.h"
#include <math.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CPlotStatisticsDialog dialog


CPlotStatisticsDialog::CPlotStatisticsDialog(CDocument* pDoc, 
											 CWnd* pParent /*=NULL*/)
	: CDialog(CPlotStatisticsDialog::IDD, pParent)
{
	m_pDoc = pDoc;

	//{{AFX_DATA_INIT(CPlotStatisticsDialog)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}


void CPlotStatisticsDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CPlotStatisticsDialog)
	DDX_Control(pDX, IDC_STAT_LIST, m_plot_list);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CPlotStatisticsDialog, CDialog)
	//{{AFX_MSG_MAP(CPlotStatisticsDialog)
	ON_BN_CLICKED(IDAPPLY, OnApply)
	ON_LBN_SELCHANGE(IDC_STAT_LIST, OnSelectChange)
	ON_BN_CLICKED(IDCLOSE, OnClose)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CPlotStatisticsDialog message handlers

BOOL CPlotStatisticsDialog::OnInitDialog()
{	
	CDialog::OnInitDialog();

	// Initialize the list box
	PlotListBox::OnInitDialog(m_plot_list, m_pDoc);

	return TRUE;
}

void CPlotStatisticsDialog::OnSelectChange()
{
	PlotListBox::OnSelectChange();
}

void CPlotStatisticsDialog::OnApply() 
{
	// Compute the plot statistics
	CString buffer, info;
	int i;
	
	// Cast the document pointer
	SGraph *m_ptr = &(((CSgraphDoc *)m_pDoc)->
		graph[((CSgraphDoc *)m_pDoc)->selected_graph]);

	for(i = 0;i < m_ptr->plot.Length();i++)
		if(m_ptr->plot[i].isDrawn && m_ptr->plot[i].isSelected){
			info.Format("Set #%d (N = %d) \"%s\"", i,
				m_ptr->plot[i].data.Length(), m_ptr->plot[i].info);
			buffer += "******* ";
			buffer += info;
			buffer += " *******\n";
			buffer += GetStatistics(m_ptr->plot[i]);
		}

	if(buffer.GetLength() == 0){
		// No plots were selected, hence there is no data to display
		AfxMessageBox("No Plots have been selected!");
	}
	else{
		CTextOutputDialog toDlg("Plot Statistics Results", buffer);

		toDlg.DoModal();
	}
}

void CPlotStatisticsDialog::OnClose() 
{
	CDialog::OnCancel();
}

CString CPlotStatisticsDialog::GetStatistics(SPlot &plot)
{
	int n; // The number of samples
	float x, y;
	double ave_x, ave_y;
	double sum_x, sum_y;
	double ave_xx, ave_yy;
	double std_dev_x, std_dev_y;
	double min_x, max_x, min_y, max_y;

	n = plot.data.Length();
	
	if(n == 0)
		return "No Data in Plot!\n";

	stackIterator<DataPoint> next(plot.data);
	DataPoint dp;
	
	next(dp);
	
	x = dp.x;
	y = dp.y;
	
	min_x = x;
	max_x = x;
	min_y = y;
	max_y = y;
	ave_x = x;
	ave_y = y;
	ave_xx = x*x;
	ave_yy = y*y;

	while(next(dp)){
		x = dp.x;
		y = dp.y;

		ave_x += x;
		ave_y += y;
		ave_xx += x*x;
		ave_yy += y*y;

		max_x = max(x, max_x);
		min_x = min(x, min_x);
		max_y = max(y, max_y);
		min_y = min(y, min_y);
	}
	
	sum_x = ave_x;
	sum_y = ave_y;

	ave_x /= n;
	ave_y /= n;
	ave_xx /= n;
	ave_yy /= n;

	std_dev_x = (ave_xx - ave_x*ave_x);

	if(n > 1){
		std_dev_x *= n/(n - 1.0f);
	}

	// Don't allow round off error to generate
	// sqrt(negative number).
	if(std_dev_x <= 0.0)
		std_dev_x = 0.0;
	else
		std_dev_x = sqrt(std_dev_x);

	std_dev_y = ave_yy - ave_y*ave_y;

	if(n > 1){
		std_dev_y *= n/(n - 1.0f);
	}

	// Don't allow round off error to generate
	// sqrt(negative number).
	if(std_dev_y <= 0.0)
		std_dev_y = 0.0;
	else
		std_dev_y = sqrt(std_dev_y);

	CString buffer;
	
	if(n > 1){
		buffer.Format("<x> = %g\n"
				  "<x^2> = %g\n"
				  "StdDev(x) = %g\n"
				  "min(x) = %g\n"
				  "max(x) = %g\n"
				  "sum(x) = %g\n\n"
				  "<y> = %g\n"
				  "<y^2> = %g\n"
				  "StdDev(y) = %g\n"
				  "min(y) = %g\n"
				  "max(y) = %g\n"
				  "sum(y) = %g\n\n",
		ave_x, ave_xx, std_dev_x, min_x, max_x, sum_x,
		ave_y, ave_yy, std_dev_y, min_y, max_y, sum_y);
	}
	else{ // n == 1
		buffer.Format("<x> = %g\n"
				  "<x^2> = %g\n"
				  "min(x) = %g\n"
				  "max(x) = %g\n"
				  "sum(x) = %g\n\n"
				  "<y> = %g\n"
				  "<y^2> = %g\n"
				  "min(y) = %g\n"
				  "max(y) = %g\n"
				  "sum(y) = %g\n\n",
		ave_x, ave_xx, min_x, max_x, sum_x,
		ave_y, ave_yy, min_y, max_y, sum_y);
	}	

	return buffer;
}
