//	(c) 2000 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.
#include "stdafx.h"
#include "sgraph.h"
#include "SelectRect.h"
#include <math.h>

CSelectRect::CSelectRect()
{
	angle = 0.0;
	x = 0;
	y = 0;

	// The default selection region is 0 X 0
	rect_size.cx = 0;
	rect_size.cy = 0;

}

CSelectRect::CSelectRect(CSelectRect &copy)
{
	angle = copy.angle;
	x = copy.x;
	y = copy.y;
	rect_size = copy.rect_size;
}

BOOL CSelectRect::Intersect(CPoint &xy)
{
	// Map xy into the rotated coordinate frame:
    //     |  c  s | | x |  =  | x_p | 
    //     | -s  c | | y |     | y_p |

	double c = cos(-angle); // <-- By the negative angle!
	double s = sin(-angle);

	int x_p, y_p;

	x_p = (int)(c*(xy.x - x) + s*(xy.y - y));
	y_p = (int)(-s*(xy.x - x) + c*(xy.y - y));
	
	if(x_p < -rect_size.cx)
		return FALSE;

	if(x_p > rect_size.cx)
		return FALSE;

	if(y_p < -rect_size.cy)
		return FALSE;

	if(y_p > rect_size.cy)
		return FALSE;

	return TRUE;
}

void CSelectRect::Draw(CDC *pDC)
{
	// Map xy into the rotated coordinate frame:
    //     |  c  s | | x |  =  | x_p | 
    //     | -s  c | | y |     | y_p |

	double c = cos(angle);
	double s = sin(angle);
	
	int tmp_x, tmp_y;
	int ll_x, ll_y;

	// Guard against round-off error complicating
	// double->int conversion
	if(fabs(c) < FP_PRECISION)
		c = 0.0;

	if(fabs(s) < FP_PRECISION)
		s = 0.0;

	ll_x = (int)(-c*rect_size.cx + s*rect_size.cy);
	ll_y = (int)(s*rect_size.cx + c*rect_size.cy);

	pDC->MoveTo(x + ll_x, y + ll_y); 

	tmp_x = (int)(c*rect_size.cx + s*rect_size.cy);
	tmp_y = (int)(-s*rect_size.cx + c*rect_size.cy);

	pDC->LineTo(x + tmp_x, y + tmp_y);

	tmp_x = (int)(c*rect_size.cx - s*rect_size.cy);
	tmp_y = (int)(-s*rect_size.cx - c*rect_size.cy);

	pDC->LineTo(x + tmp_x, y + tmp_y); 

	tmp_x = -(int)(c*rect_size.cx + s*rect_size.cy);
	tmp_y = -(int)(-s*rect_size.cx + c*rect_size.cy);

	pDC->LineTo(x + tmp_x, y + tmp_y);

	// Close the rectangle
	pDC->LineTo(x + ll_x, y + ll_y); 

}