//	(c) 2000 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.

#include "stdafx.h"
#include "sgraph.h"
#include "TextEditDialog.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CTextEditDialog dialog


CTextEditDialog::CTextEditDialog(const CString &m_title, CString &m_text, LOGFONT &m_font, 
					COLORREF m_fontColor, CWnd* pParent /* = NULL */)
	: CDialog(CTextEditDialog::IDD, pParent)
{
	title = m_title;
	text = m_text;
	font = m_font;
	fontColor = m_fontColor;

	angle = (int)(font.lfOrientation/10.0);

	//{{AFX_DATA_INIT(CTextEditDialog)
	m_orientation = _T("");
	//}}AFX_DATA_INIT
}


void CTextEditDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CTextEditDialog)
	DDX_Control(pDX, IDC_TEXT_EDIT, m_text_edit);
	DDX_Control(pDX, IDC_SPIN_ORIENTATION, m_spin_orientation);
	DDX_Text(pDX, IDC_EDIT_ORIENTATION, m_orientation);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CTextEditDialog, CDialog)
	//{{AFX_MSG_MAP(CTextEditDialog)
	ON_BN_CLICKED(IDC_TEXT_FONT, OnTextFont)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTextEditDialog message handlers
BOOL CTextEditDialog::OnInitDialog()
{
	CDialog::OnInitDialog();
	
	SetWindowText(title);

	m_text_edit.SetWindowText(text);

	// Set the spinner buddy
	m_spin_orientation.SetBuddy(GetDlgItem(IDC_EDIT_ORIENTATION));

	// Set the allowed spinner range
	m_spin_orientation.SetRange32(0, 360);

	m_orientation.Format("%d", angle);

	UpdateData(FALSE);

	// Give the text edit control the window focus
	GotoDlgCtrl(GetDlgItem(IDC_TEXT_EDIT));

	return FALSE; // Return false because we are manually setting the focus
}

void CTextEditDialog::OnTextFont() 
{
	CFontDialog fDlg(&font);
	
	// Customize the font dialog to show the current font color
	fDlg.m_cf.rgbColors = fontColor;

	if(fDlg.DoModal() == IDOK)
		fontColor = fDlg.GetColor();
}

CString &CTextEditDialog::GetText()
{
	return text;
}

LOGFONT& CTextEditDialog::GetFont()
{
	return font;
}

COLORREF CTextEditDialog::GetFontColor()
{
	return fontColor;
}

void CTextEditDialog::OnOK() 
{
	UpdateData(TRUE);

	angle = atoi(m_orientation);

	if((angle < 0) || (angle > 360)){
		AfxMessageBox("Please enter a valid orientation!");
		return;
	}

	m_text_edit.GetWindowText(text);
	font.lfOrientation = 10*angle;
	font.lfEscapement = font.lfOrientation;

	CDialog::OnOK();
}
