//	(c) 2000 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.

#include "stdafx.h"
#include "sgraph.h"
#include "TickLabelDialog.h"
#include "AxisDialog.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CTickLabelDialog dialog


CTickLabelDialog::CTickLabelDialog(CSgraphDoc* pDoc, int axis, 
								   CWnd* pParent /*=NULL*/)
	: CDialog(CTickLabelDialog::IDD, pParent)
{
	m_pDoc = pDoc;
	selected_axis = axis;

	//{{AFX_DATA_INIT(CTickLabelDialog)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}


void CTickLabelDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CTickLabelDialog)
	DDX_Control(pDX, IDC_FORMAT, m_format);
	DDX_Control(pDX, IDC_PRECISION, m_precision);
	DDX_Control(pDX, IDC_APPLY, m_apply);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CTickLabelDialog, CDialog)
	//{{AFX_MSG_MAP(CTickLabelDialog)
	ON_BN_CLICKED(IDC_BUTTON_FONT, OnButtonFont)
	ON_BN_CLICKED(IDC_RADIO_NORMAL, OnRadioNormal)
	ON_BN_CLICKED(IDC_RADIO_BOTH, OnRadioBoth)
	ON_BN_CLICKED(IDC_RADIO_OPPOSITE, OnRadioOpposite)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTickLabelDialog message handlers

BOOL CTickLabelDialog::OnInitDialog()
{
	CDialog::OnInitDialog();
	
	SAxisObject *normal_ptr, *opposite_ptr;
	
	if(selected_axis == CAxisDialog::XAxis){
		normal_ptr = &(m_pDoc->graph[m_pDoc->selected_graph].bottomAxis);

		opposite_ptr = &(m_pDoc->graph[m_pDoc->selected_graph].topAxis);

		m_label_location = 
			m_pDoc->graph[m_pDoc->selected_graph].x_tick_label_location;
	}
	else{
		normal_ptr = &(m_pDoc->graph[m_pDoc->selected_graph].leftAxis);

		opposite_ptr = &(m_pDoc->graph[m_pDoc->selected_graph].rightAxis);

		m_label_location = 
			m_pDoc->graph[m_pDoc->selected_graph].y_tick_label_location;
	}

	m_font = normal_ptr->tick_font;
	m_font_color = normal_ptr->tick_font_color;

	// Set the tick direction radio buttons from the the current axis
	switch(m_label_location){
		case SGraph::TICK_MARK_NORMAL:
			CheckRadioButton(IDC_RADIO_NORMAL,		// First button
							 IDC_RADIO_OPPOSITE,	// Last bottom	
							 IDC_RADIO_NORMAL);		// Button to check
			break;
		case SGraph::TICK_MARK_BOTH:
			CheckRadioButton(IDC_RADIO_NORMAL,		// First button
							 IDC_RADIO_OPPOSITE,	// Last bottom	
							 IDC_RADIO_BOTH);		// Button to check
			break;
		case SGraph::TICK_MARK_OPPOSITE:
			CheckRadioButton(IDC_RADIO_NORMAL,		// First button
							 IDC_RADIO_OPPOSITE,	// Last bottom	
							 IDC_RADIO_OPPOSITE);	// Button to check
			break;
	};

	// Set the selected axis combo box
	m_apply.SetCurSel(0 /* Current Axis */);
	
	// Set the precision combo box
	m_precision.SetCurSel(normal_ptr->tick_lable_precision);

	// Set the format combo box
	m_format.SetCurSel(normal_ptr->tick_lable_format);

	UpdateData(FALSE /* Initialize dialog */);

	return TRUE;
}


void CTickLabelDialog::OnButtonFont() 
{
	CFontDialog fDlg(&m_font);

	fDlg.m_cf.rgbColors = m_font_color;

	if(fDlg.DoModal() == IDOK){
		// Update the font color
		m_font_color = fDlg.m_cf.rgbColors;
	}
}

void CTickLabelDialog::OnRadioNormal() 
{
	m_label_location = SGraph::TICK_MARK_NORMAL;
}

void CTickLabelDialog::OnRadioBoth() 
{
	m_label_location = SGraph::TICK_MARK_BOTH;
}

void CTickLabelDialog::OnRadioOpposite() 
{
	m_label_location = SGraph::TICK_MARK_OPPOSITE;
}

void CTickLabelDialog::UpdateLabels(int graph, int axis)
{
	SAxisObject *normal_ptr, *opposite_ptr;

	if(axis == CAxisDialog::XAxis){
		normal_ptr = &(m_pDoc->graph[graph].bottomAxis);
		opposite_ptr = &(m_pDoc->graph[graph].topAxis);
		m_pDoc->graph[graph].x_tick_label_location = m_label_location;

		switch(m_label_location){
			case SGraph::TICK_MARK_NORMAL:
				normal_ptr->isTickLableDrawn = TRUE;
				opposite_ptr->isTickLableDrawn = FALSE;
				break;
			case SGraph::TICK_MARK_BOTH:
				normal_ptr->isTickLableDrawn = TRUE;
				opposite_ptr->isTickLableDrawn = TRUE;
				break;
			case SGraph::TICK_MARK_OPPOSITE:
				normal_ptr->isTickLableDrawn = FALSE;
				opposite_ptr->isTickLableDrawn = TRUE;
				break;
		};
	}
	else{
		normal_ptr = &(m_pDoc->graph[graph].leftAxis);
		opposite_ptr = &(m_pDoc->graph[graph].rightAxis);
		m_pDoc->graph[graph].y_tick_label_location = m_label_location;

		switch(m_label_location){
			case SGraph::TICK_MARK_NORMAL:
				normal_ptr->isTickLableDrawn = TRUE;
				opposite_ptr->isTickLableDrawn = FALSE;
				break;
			case SGraph::TICK_MARK_BOTH:
				normal_ptr->isTickLableDrawn = TRUE;
				opposite_ptr->isTickLableDrawn = TRUE;
				break;
			case SGraph::TICK_MARK_OPPOSITE:
				normal_ptr->isTickLableDrawn = FALSE;
				opposite_ptr->isTickLableDrawn = TRUE;
				break;
		};
	}

	// Set the font and color
	normal_ptr->tick_font = m_font;
	normal_ptr->tick_font_color = m_font_color;
	opposite_ptr->tick_font = m_font;
	opposite_ptr->tick_font_color = m_font_color;

	// Set the precision
	normal_ptr->tick_lable_precision =  opposite_ptr->tick_lable_precision = 
		m_precision.GetCurSel();
		
	// Set the format
	normal_ptr->tick_lable_format =  opposite_ptr->tick_lable_format = 
		m_format.GetCurSel();

}

void CTickLabelDialog::OnOK() 
{
	// What graphs and axes do we apply the changes to?
	int apply = m_apply.GetCurSel();

	// The four different possibilities for the apply combo box
	if(apply == 0 /* Current Graph */){
		UpdateLabels(m_pDoc->selected_graph, selected_axis);
	}
	
	if(apply == 1 /* All Axes, Current Graph */){
		UpdateLabels(m_pDoc->selected_graph, CAxisDialog::XAxis);
		UpdateLabels(m_pDoc->selected_graph, CAxisDialog::YAxis);
	}
	
	if(apply == 2 /* Current Axis, All Graphs */){
		for(int i = 0;i < m_pDoc->num_graph;i++)
			UpdateLabels(i, selected_axis);
	}

	if(apply == 3 /* All Axes, All Graphs */){
		for(int i = 0;i < m_pDoc->num_graph;i++){
			UpdateLabels(i, CAxisDialog::XAxis);
			UpdateLabels(i, CAxisDialog::YAxis);
		}		
	}

	CDialog::OnOK();
}
