//	(c) 2000 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.
#include "stdafx.h"
#include "color.h"

// A color lookup table
int ColorToIndex(COLORREF color) 
{
	if(color == NO_COLOR)
		return S_CLR_NONE; // None (i.e. no color)

	int r, g, b;

	r = color & 0x000000ff;
	g = (color >> 8) & 0x000000ff;
	b = (color >> 16) & 0x000000ff;

	if((r == 0) && (g == 0) && (b == 0))
		return S_CLR_BLACK; // Black
	
	if((r == 255) && (g == 255) && (b == 255))
		return S_CLR_WHITE; // White
	
	if((r == 255) && (g == 0) && (b == 0))
		return S_CLR_RED; // Red
	
	if((r == 0) && (g == 255) && (b == 0))
		return S_CLR_GREEN; // Green
	
	if((r == 0) && (g == 0) && (b == 255))
		return S_CLR_BLUE; // Blue
	
	if((r == 0) && (g == 255) && (b == 255))
		return S_CLR_CYAN; // Cyan

	if((r == 100) && (g == 100) && (b == 100))
		return S_CLR_GREY; // Grey
	
	//////////////////////////////////////////////////////////////////
	// Handle custom colors in an unusual way!
	//////////////////////////////////////////////////////////////////
	// Is the color valid?
	if((r < 256) && (r >= 0) && 
	   (g < 256) && (g >= 0) &&
	   (b < 256) && (b >= 0))
		// Return a custom color. The negative sign indicates that
		// the color is (a) valid and (b) not one of the standard
		// colors.
		return -(int)(color);
	//////////////////////////////////////////////////////////////////

	// Default - return Black
	return S_CLR_BLACK;
}

// A color lookup table
COLORREF IndexToColor(int color)
{
	if(color == S_CLR_BLACK)
		return 0x00000000; // Black
	
	if(color == S_CLR_WHITE)
		return 0x00ffffff; // White
	
	if(color == S_CLR_RED)
		return 0x000000ff; // Red
	
	if(color == S_CLR_GREEN)
		return 0x0000ff00; // Green
	
	if(color == S_CLR_BLUE)
		return 0x00ff0000; // Blue
	
	if(color == S_CLR_CYAN)
		return 0x00ffff00; // Cyan

	if(color == S_CLR_GREY)
		return 0x00646464; // Grey
	
	if(color == S_CLR_NONE)
		// Are we allowed to write to the high order
		// bytes of a COLORREF structure
		// (i.e. NO_COLOR == 0xff000000)?
		return NO_COLOR; // No Color

	// Is this a custom color?
	if(color < 0)
		return (COLORREF)(-color);

	// Default - return Black
	return 0x00000000;
}