//	(c) 2000-2002 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.
#include "EquationUtil.h"
#include <math.h>
#include <float.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <stdio.h>

void EqElemUnary::Set(list<EqToken> &m_token, EqElem *m_p, list<EqToken> &varstack)
{
	// Set the parent pointer
	P = m_p;

	if(m_token[0].token_type == UNARY_L_BRACE){
		unary_op = UNARY_PAREN;

		int nextbrace = MatchUnaryBrace(m_token, 0);

		// Copy the list
		list<EqToken> tmp;

		// Remove the outermost '('.
		m_token.Remove(0);
		nextbrace --;

		for(int i = 0;i < nextbrace;i++){
			tmp.Push(m_token[0]);

			// Remove these elements as we go
			m_token.Remove(0);
		}

		// Remove the outermost ')'.
		m_token.Remove(0);

		// The first element of tmp must be a value or unary operator
		if(isUnaryOp(tmp[0].token_type)){
			C = new EqElemUnary;
			C->Set(tmp, this, varstack);
			C->Push(tmp, varstack);
			return;
		}

		// Its a value
		C = new EqElemVal;
		C->Set(tmp, this, varstack);
		C->Push(tmp, varstack);
		return;
	}
	else{
		// Not a brace
		unary_op = m_token[0].token_type;
		m_token.Remove(0);
	}
}

void EqElemUnary::Push(list<EqToken> &m_token, list<EqToken> &varstack)
{
	if(m_token.Length() == 0)
		// Nothing to do!
		return;

	if(C == NULL){
		// Must be a unary op or value
		if(isUnaryOp(m_token[0].token_type)){
			C = new EqElemUnary;
		}
		else{
			C = new EqElemVal;
		}

		C->Set(m_token, this, varstack);
		C->Push(m_token, varstack);
		return;
	}

	// C != NULL
	if(P == NULL){
		// Allocate space for the parent element (must be a binary op)
		if(!isBinaryOp(m_token[0].token_type))
			throw "NULL P pointer and unary->!binary Push";

		P = new EqElemBinary;

		((EqElemBinary*)P)->L = this;

		P->Set(m_token, NULL, varstack);
		P->Push(m_token, varstack);
		return;
	}

	//////////////////////////////////////////////////////////////////////
	// Make sure proper order of operations is maintained between binary
	// operators (i.e. ^,/,*,+,-).
	//////////////////////////////////////////////////////////////////////
	if(isBinaryOp(P->GetType())){
		if((P->GetType() == BINARY_ADD) || (P->GetType() == BINARY_SUB)){
			if((m_token[0].token_type == BINARY_DIV) || 
			   (m_token[0].token_type == BINARY_MULTI) ||
			   (m_token[0].token_type == BINARY_POW)){
				
				// Insert the new operator
				EqElem *tmp_P = P;

				P = new EqElemBinary;
				
				// Attach the old parent to this new element
				((EqElemBinary*)tmp_P)->R = P;

				// Attach the new parent to this element
				((EqElemBinary*)P)->L = this;

				P->Set(m_token, tmp_P, varstack);
				P->Push(m_token, varstack);
				return;
			}
		}
		
		if((P->GetType() == BINARY_DIV) || (P->GetType() == BINARY_MULTI)){
			if(m_token[0].token_type == BINARY_POW){
				
				// Insert the new operator
				EqElem *tmp_P = P;

				P = new EqElemBinary;
				
				// Attach the old parent to this new element
				((EqElemBinary*)tmp_P)->R = P;

				// Attach the new parent to this element
				((EqElemBinary*)P)->L = this;

				P->Set(m_token, tmp_P, varstack);
				P->Push(m_token, varstack);
				return;
			}
		}

		P->Push(m_token, varstack);
		return;
	}
	else{ // Unary operator

		// Is the parent a unary brace or unary '-'?
		if((P->GetType() == UNARY_PAREN) || 
		  ((P->GetType() == UNARY_NEG) &&
		  ((m_token[0].token_type == BINARY_DIV) || 
		   (m_token[0].token_type == BINARY_MULTI) ||
		   (m_token[0].token_type == BINARY_POW)))){
			if(!isBinaryOp(m_token[0].token_type))
				throw "Missing a binary operator between braces or before '-'";

			// The brace operator must get special treatment
			// Insert the new operator
			EqElem *tmp_P = P;

			P = new EqElemBinary;

			// Attach the old parent to this new element
			UNARY(tmp_P)->C = P;

			// Make the current element the left operatand
			BINARY(P)->L = this;

			P->Set(m_token, tmp_P, varstack);
			P->Push(m_token, varstack);
			return;
		}

		P->Push(m_token, varstack);
	}
}

double EqElemUnary::Eval(double *m_eval_stack)
{

	if(safe_mode){
		if(C == NULL)
			throw "Unary C is NULL";
	}

	switch (unary_op){
		case UNARY_NEG:
			return -C->Eval(m_eval_stack);
		case UNARY_PLUS:
			return C->Eval(m_eval_stack);
		case UNARY_PAREN:
			return C->Eval(m_eval_stack);
		case UNARY_SQRT:

			if(safe_mode){
				double tmp = sqrt(C->Eval(m_eval_stack));

				if(!_finite(tmp))
					throw "Unary sqrt of negative number";

				return tmp;
			}

			return sqrt(C->Eval(m_eval_stack));
		case UNARY_COS:
			return cos(C->Eval(m_eval_stack));
		case UNARY_SIN:
			return sin(C->Eval(m_eval_stack));
		case UNARY_TAN:
			return tan(C->Eval(m_eval_stack));
		case UNARY_ACOS:
			return acos(C->Eval(m_eval_stack));
		case UNARY_ASIN:
			return asin(C->Eval(m_eval_stack));
		case UNARY_ATAN:
			return atan(C->Eval(m_eval_stack));
		case UNARY_SEC:
			return sec(C->Eval(m_eval_stack));
		case UNARY_CSC:
			return csc(C->Eval(m_eval_stack));
		case UNARY_COT:
			return cot(C->Eval(m_eval_stack));
		case UNARY_COSH:
			return cosh(C->Eval(m_eval_stack));
		case UNARY_SINH:
			return sinh(C->Eval(m_eval_stack));
		case UNARY_TANH:
			return tanh(C->Eval(m_eval_stack));
		case UNARY_EXP:
			return exp(C->Eval(m_eval_stack));
		case UNARY_LOG:

			if(safe_mode){
				double tmp = log(C->Eval(m_eval_stack));

				if(!_finite(tmp))
					throw "Unary log of <= 0";

				return tmp;
			}

			return log(C->Eval(m_eval_stack));
		case UNARY_LOG10:

			if(safe_mode){
				double tmp = log10(C->Eval(m_eval_stack));

				if(!_finite(tmp))
					throw "Unary log10 of <= 0";

				return tmp;
			}

			return log10(C->Eval(m_eval_stack));
		case UNARY_ABS:
			return fabs(C->Eval(m_eval_stack));
	};

	throw "Unknown unary operator";

	return -1;
}

#ifdef  _DEBUG
void EqElemUnary::DebugEval(FILE *fout, list<EqToken> &varstack)
{

	if(safe_mode){
		if(C == NULL)
			throw "Unary C is NULL";
	}

	switch (unary_op){
		case UNARY_NEG:
			fprintf(fout, "-(");
			C->DebugEval(fout, varstack);
			fprintf(fout, ")");
			return;
		case UNARY_PLUS:
			//fprintf(fout, "(");
			C->DebugEval(fout, varstack);
			//fprintf(fout, ")");
			return;
		case UNARY_PAREN:
			//fprintf(fout, "(");
			C->DebugEval(fout, varstack);
			//fprintf(fout, ")");
			return;
		case UNARY_SQRT:
			fprintf(fout, "Sqrt[");
			C->DebugEval(fout, varstack);
			fprintf(fout, "]");
			return;
		case UNARY_COS:
			fprintf(fout, "Cos[");
			C->DebugEval(fout, varstack);
			fprintf(fout, "]");
			return;
		case UNARY_SIN:
			fprintf(fout, "Sin[");
			C->DebugEval(fout, varstack);
			fprintf(fout, "]");
			return;
		case UNARY_TAN:
			fprintf(fout, "Tan[");
			C->DebugEval(fout, varstack);
			fprintf(fout, "]");
			return;
		case UNARY_ACOS:
			fprintf(fout, "Acos[");
			C->DebugEval(fout, varstack);
			fprintf(fout, "]");
			return;
		case UNARY_ASIN:
			fprintf(fout, "Asin[");
			C->DebugEval(fout, varstack);
			fprintf(fout, "]");
			return;
		case UNARY_ATAN:
			fprintf(fout, "Atan[");
			C->DebugEval(fout, varstack);
			fprintf(fout, "]");
			return;
		case UNARY_SEC:
			fprintf(fout, "Sec[");
			C->DebugEval(fout, varstack);
			fprintf(fout, "]");
			return;
		case UNARY_CSC:
			fprintf(fout, "Csc[");
			C->DebugEval(fout, varstack);
			fprintf(fout, "]");
			return;
		case UNARY_COT:
			fprintf(fout, "Cot[");
			C->DebugEval(fout, varstack);
			fprintf(fout, "]");
			return;
		case UNARY_COSH:
			fprintf(fout, "Cosh[");
			C->DebugEval(fout, varstack);
			fprintf(fout, "]");
			return;
		case UNARY_SINH:
			fprintf(fout, "Sinh[");
			C->DebugEval(fout, varstack);
			fprintf(fout, "]");
			return;
		case UNARY_TANH:
			fprintf(fout, "Tanh[");
			C->DebugEval(fout, varstack);
			fprintf(fout, "]");
			return;
		case UNARY_EXP:
			fprintf(fout, "Exp[");
			C->DebugEval(fout, varstack);
			fprintf(fout, "]");
			return;
		case UNARY_LOG:
			fprintf(fout, "Log[");
			C->DebugEval(fout, varstack);
			fprintf(fout, "]");
			return;
		case UNARY_LOG10:
			fprintf(fout, "Log10[");
			C->DebugEval(fout, varstack);
			fprintf(fout, "]");
			return;
		case UNARY_ABS:
			fprintf(fout, "Abs[");
			C->DebugEval(fout, varstack);
			fprintf(fout, "]");
			return;
	};

	throw "Unknown unary operator";
}
#endif // _DEBUG

double EqElemUnary::sec(double m_x)
{
	m_x = 1.0/cos(m_x);

	if(safe_mode){
		if(!_finite(m_x))
			throw "Unary sec divide by zero";
	}

	return m_x;
}

double EqElemUnary::csc(double m_x)
{
	m_x = 1.0/sin(m_x);

	if(safe_mode){
		if(!_finite(m_x))
			throw "Unary csc divide by zero";
	}

	return m_x;
}

double EqElemUnary::cot(double m_x)
{
	m_x = tan(m_x);

	if(safe_mode){
		if(!_finite(m_x))
			throw "Unary cot divide by zero";
	}

	return m_x;
}

EqElem* EqElemUnary::Copy(EqElem *m_p)
{
	EqElem* tmp = new EqElemUnary;

	((EqElemUnary*)tmp)->unary_op = unary_op;

	// Recursively copy the tree elements
	((EqElemUnary*)tmp)->P = m_p;

#ifdef _DEBUG
	if(C == NULL)
		throw "NULL C in EqElemUnary::Copy";
#endif // _DEBUG

	((EqElemUnary*)tmp)->C = C->Copy(tmp);

	return tmp;
}

BOOL EqElemUnary::IsEqual(EqElem *m_elem)
{
#ifdef _DEBUG
	if(m_elem == NULL)
		throw "NULL pointer in EqElemUnary::IsEqual";
#endif // _DEBUG

	if(m_elem->GetType() != unary_op)
		return FALSE;

	if(C->IsEqual(UNARY(m_elem)->C))
		return TRUE;

	return FALSE;
}


EqElem* EqElemUnary::Deriv(int m_var_index, EqElem *m_p)
{
	// The differentiation of a unary operator has the general
	// form of a binary operator with right element given by
	// dx/dp, i.e.
	// D[U(x), p] = (dU/dx)*(dx/dp)
	EqElem* tmpBinary = new EqElemBinary;
	EqElem* tmp;

	tmpBinary->P = m_p;

	BINARY(tmpBinary)->binary_op = BINARY_MULTI;

	// Set the right child
	BINARY(tmpBinary)->R = C->Deriv(m_var_index, tmpBinary);

	// Set the left element, L = (dU/dx)(x)
	switch (unary_op){
		// L = -1 (L is EqElemVal)
		case UNARY_NEG:
			BINARY(tmpBinary)->L = MakeConst(-1.0, tmpBinary);

			break;
		// L = 1 (L is EqElemVal)
		case UNARY_PLUS:
			BINARY(tmpBinary)->L = MakeConst(1.0, tmpBinary);

			break;
		// L = 1 (L is EqElemVal)
		case UNARY_PAREN:
			BINARY(tmpBinary)->L = MakeConst(1.0, tmpBinary);
			break;
		// L = 0.5/sqrt(x) (L is EqElemBinary)
		case UNARY_SQRT:
			tmp = BINARY(tmpBinary)->L = new EqElemBinary(tmpBinary);
			
			BINARY(tmp)->binary_op = BINARY_DIV;
			BINARY(tmp)->L = MakeConst(0.5, tmp);

			BINARY(tmp)->R = new EqElemUnary(tmp);

			UNARY(BINARY(tmp)->R)->unary_op = UNARY_SQRT;
			UNARY(BINARY(tmp)->R)->C = C->Copy(BINARY(tmp)->R);
			break;
		// L = -(sin(x)) (L is EqElemUnary)
		case UNARY_COS:
			tmp = BINARY(tmpBinary)->L = new EqElemUnary(tmpBinary);
			UNARY(tmp)->unary_op = UNARY_NEG;
			
			UNARY(tmp)->C = new EqElemUnary(tmp);
			tmp = UNARY(tmp)->C;

			UNARY(tmp)->unary_op = UNARY_SIN;
			UNARY(tmp)->C = C->Copy(tmp);
			break;
		// L = cos(x) (L is EqElemUnary)
		case UNARY_SIN:
			tmp = BINARY(tmpBinary)->L = new EqElemUnary(tmpBinary);
			UNARY(tmp)->unary_op = UNARY_COS;
			UNARY(tmp)->C = C->Copy(tmp);
			break;
		// L = sec(x)^2 (L is EqElemBinary)
		case UNARY_TAN:
			tmp = BINARY(tmpBinary)->L = new EqElemBinary(tmpBinary);
			
			BINARY(tmp)->binary_op = BINARY_POW;
			BINARY(tmp)->R = MakeConst(2, tmp);

			BINARY(tmp)->L = new EqElemUnary(tmp);

			UNARY(BINARY(tmp)->L)->unary_op = UNARY_SEC;
			UNARY(BINARY(tmp)->L)->C = C->Copy(BINARY(tmp)->L);
			break;
		// L = (-1)/sqrt(1 - x^2) (L is EqElemBinary)
		case UNARY_ACOS:
			tmp = BINARY(tmpBinary)->L = new EqElemBinary(tmpBinary);
			
			BINARY(tmp)->binary_op = BINARY_DIV;
			BINARY(tmp)->L = MakeConst(-1.0, tmp);

			BINARY(tmp)->R = new EqElemUnary(tmp);

			UNARY(BINARY(tmp)->R)->unary_op = UNARY_SQRT;
			UNARY(BINARY(tmp)->R)->C = new EqElemBinary(BINARY(tmp)->R);

			tmp = UNARY(BINARY(tmp)->R)->C;

			BINARY(tmp)->binary_op = BINARY_SUB;
			BINARY(tmp)->L = MakeConst(1.0, tmp);
			BINARY(tmp)->R = new EqElemBinary(tmp);
			
			tmp = BINARY(tmp)->R;
			
			BINARY(tmp)->binary_op = BINARY_POW;
			BINARY(tmp)->R = MakeConst(2.0, tmp);
			BINARY(tmp)->L = C->Copy(tmp);
			break;
		// L = 1/sqrt(1 - x^2) (L is EqElemBinary)
		case UNARY_ASIN:
			tmp = BINARY(tmpBinary)->L = new EqElemBinary(tmpBinary);
			
			BINARY(tmp)->binary_op = BINARY_DIV;
			BINARY(tmp)->L = MakeConst(1.0, tmp);

			BINARY(tmp)->R = new EqElemUnary(tmp);

			UNARY(BINARY(tmp)->R)->unary_op = UNARY_SQRT;
			UNARY(BINARY(tmp)->R)->C = new EqElemBinary(BINARY(tmp)->R);

			tmp = UNARY(BINARY(tmp)->R)->C;

			BINARY(tmp)->binary_op = BINARY_SUB;
			BINARY(tmp)->L = MakeConst(1.0, tmp);
			BINARY(tmp)->R = new EqElemBinary(tmp);
			
			tmp = BINARY(tmp)->R;
			
			BINARY(tmp)->binary_op = BINARY_POW;
			BINARY(tmp)->R = MakeConst(2.0, tmp);
			BINARY(tmp)->L = C->Copy(tmp);
			break;
		// L = 1/(1 + x^2) (L is EqElemBinary)
		case UNARY_ATAN:
			tmp = BINARY(tmpBinary)->L = new EqElemBinary(tmpBinary);
			
			BINARY(tmp)->binary_op = BINARY_DIV;
			BINARY(tmp)->L = MakeConst(1.0, tmp);

			BINARY(tmp)->R = new EqElemBinary(tmp);

			tmp = BINARY(tmp)->R;
			
			BINARY(tmp)->binary_op = BINARY_ADD;
			BINARY(tmp)->L = MakeConst(1.0, tmp);

			BINARY(tmp)->R = new EqElemBinary(tmp);
			
			tmp = BINARY(tmp)->R;
			
			BINARY(tmp)->binary_op = BINARY_POW;
			BINARY(tmp)->R = MakeConst(2.0, tmp);
			BINARY(tmp)->L = C->Copy(tmp);
			break;
		// L = sec(x)*tan(x) (L is EqElemBinary)
		case UNARY_SEC:
			tmp = BINARY(tmpBinary)->L = new EqElemBinary(tmpBinary);
			
			BINARY(tmp)->binary_op = BINARY_MULTI;

			BINARY(tmp)->L = new EqElemUnary(tmp);

			UNARY(BINARY(tmp)->L)->unary_op = UNARY_SEC;
			UNARY(BINARY(tmp)->L)->C = C->Copy(BINARY(tmp)->L);

			BINARY(tmp)->R = new EqElemUnary(tmp);

			UNARY(BINARY(tmp)->R)->unary_op = UNARY_TAN;
			UNARY(BINARY(tmp)->R)->C = C->Copy(BINARY(tmp)->R);
			break;
		// L = -(cot(x)*csc(x)) (L is EqElemUnary)
		case UNARY_CSC:
			tmp = BINARY(tmpBinary)->L = new EqElemUnary(tmpBinary);
			
			UNARY(tmp)->unary_op = BINARY_SUB;

			UNARY(tmp)->C = new EqElemBinary(tmp);

			tmp = UNARY(tmp)->C;

			BINARY(tmp)->binary_op = BINARY_MULTI;

			BINARY(tmp)->L = new EqElemUnary(tmp);

			UNARY(BINARY(tmp)->L)->unary_op = UNARY_COT;
			UNARY(BINARY(tmp)->L)->C = C->Copy(BINARY(tmp)->L);

			BINARY(tmp)->R = new EqElemUnary(tmp);

			UNARY(BINARY(tmp)->R)->unary_op = UNARY_CSC;
			UNARY(BINARY(tmp)->R)->C = C->Copy(BINARY(tmp)->R);
			break;
		// L = -(csc(x)^2) (L is EqElemUnary)
		case UNARY_COT:
			tmp = BINARY(tmpBinary)->L = new EqElemUnary(tmpBinary);
			UNARY(tmp)->unary_op = UNARY_NEG;
			
			UNARY(tmp)->C = new EqElemBinary(tmp);
			tmp = UNARY(tmp)->C;

			BINARY(tmp)->binary_op = BINARY_POW;
			BINARY(tmp)->L = C->Copy(tmp);
			BINARY(tmp)->R = MakeConst(2.0, tmp);
			break;
		// L = sinh(x) (L is EqElemUnary)
		case UNARY_COSH:
			tmp = BINARY(tmpBinary)->L = new EqElemUnary(tmpBinary);
			UNARY(tmp)->unary_op = UNARY_SINH;
			
			UNARY(tmp)->C = C->Copy(tmp);
			break;
		// L = cosh(x) (L is EqElemUnary)
		case UNARY_SINH:
			tmp = BINARY(tmpBinary)->L = new EqElemUnary(tmpBinary);
			UNARY(tmp)->unary_op = UNARY_COSH;
			
			UNARY(tmp)->C = C->Copy(tmp);
			break;
		// L = sech(x)^2 (L is EqElemBinary)
		// note that sech(x) == 1/cosh(x)
		// L = cosh(x)^(-2)
		case UNARY_TANH:
			tmp = BINARY(tmpBinary)->L = new EqElemBinary(tmpBinary);
			
			BINARY(tmp)->binary_op = BINARY_POW;

			BINARY(tmp)->L = new EqElemUnary(tmp);

			UNARY(BINARY(tmp)->L)->unary_op = UNARY_COSH;
			UNARY(BINARY(tmp)->L)->C = C->Copy(BINARY(tmp)->L);

			BINARY(tmp)->R = MakeConst(-2.0, tmp);
			break;
		// L = exp(x) (L is EqElemUnary)
		case UNARY_EXP:
			tmp = BINARY(tmpBinary)->L = new EqElemUnary(tmpBinary);
			UNARY(tmp)->unary_op = UNARY_EXP;
			
			UNARY(tmp)->C = C->Copy(tmp);
			break;
		// L = 1/x (L is EqElemBinary)
		case UNARY_LOG:
			tmp = BINARY(tmpBinary)->L = new EqElemBinary(tmpBinary);
			
			BINARY(tmp)->binary_op = BINARY_DIV;

			BINARY(tmp)->L = MakeConst(1.0, tmp);

			BINARY(tmp)->R = C->Copy(tmp);
			break;
		// L = 1/(x*ln(10))
		//   = (1/ln(10))/x
		case UNARY_LOG10:
			tmp = BINARY(tmpBinary)->L = new EqElemBinary(tmpBinary);
			
			BINARY(tmp)->binary_op = BINARY_DIV;

			BINARY(tmp)->L = MakeConst(1.0/log(10.0), tmp);

			BINARY(tmp)->R = C->Copy(tmp);
			break;
		// Derivative not defined!!!
		// Clean up memory before throwing an error.
		case UNARY_ABS:
			//BINARY(tmpBinary)->L = MakeConst(0.0, tmpBinary);
			tmpBinary->CleanUp();
			delete tmpBinary;

			throw "Derivative of abs() not implemented!";
	};

	return tmpBinary;
}

void EqElemUnary::CleanUp()
{
	// Delete the child
	if(C){
		C->CleanUp();

		delete C;
		C = NULL;
	}
}