//	(c) 2000-2002 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.
#include "EquationUtil.h"
#include <math.h>
#include <float.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <stdio.h>

EqToken& EqToken::operator = (const EqToken& m_copy)
{
	if(id)
		delete [] id;

	id = new char [strlen(m_copy.id) + 1];

	numeric_val = m_copy.numeric_val;

	strcpy(id, m_copy.id);

	token_type = m_copy.token_type;

	return (*this);
}

int EqToken::GetOperator(char *m_input, int len)
{
	if(len == 2){
		if(strncmp(m_input, "ln", len) == 0){
			return UNARY_LOG;
		}
	}

	if(len == 3){
		if(strncmp(m_input, "cos", len) == 0){
			return UNARY_COS;
		}

		if(strncmp(m_input, "sin", len) == 0){
			return UNARY_SIN;
		}

		if(strncmp(m_input, "tan", len) == 0){
			return UNARY_TAN;
		}

		if(strncmp(m_input, "sec", len) == 0){
			return UNARY_SEC;
		}

		if(strncmp(m_input, "csc", len) == 0){
			return UNARY_CSC;
		}

		if(strncmp(m_input, "cot", len) == 0){
			return UNARY_COT;
		}

		if(strncmp(m_input, "exp", len) == 0){
			return UNARY_EXP;
		}

		if(strncmp(m_input, "log", len) == 0){
			return UNARY_LOG;
		}

		if(strncmp(m_input, "abs", len) == 0){
			return UNARY_ABS;
		}
	}

	if(len == 4){
		if(strncmp(m_input, "sqrt", len) == 0){
			return UNARY_SQRT;
		}

		if(strncmp(m_input, "acos", len) == 0){
			return UNARY_ACOS;
		}

		if(strncmp(m_input, "asin", len) == 0){
			return UNARY_ASIN;
		}

		if(strncmp(m_input, "acos", len) == 0){
			return UNARY_ACOS;
		}

		if(strncmp(m_input, "atan", len) == 0){
			return UNARY_ATAN;
		}

		if(strncmp(m_input, "cosh", len) == 0){
			return UNARY_COSH;
		}

		if(strncmp(m_input, "sinh", len) == 0){
			return UNARY_SINH;
		}

		if(strncmp(m_input, "tanh", len) == 0){
			return UNARY_TANH;
		}
	}

	if(len == 5){
		if(strncmp(m_input, "log10", len) == 0){
			return UNARY_LOG10;
		}
	}

	// Not an operator
	return -1;
}

BOOL EqToken::isNumber(char *m_input, int len)
{
	BOOL has_decimal = FALSE;

	// Is this string a special constant?
	switch(len){
		case 2:
			if(strncmp(m_input, "pi", len) == 0){
				numeric_val = 3.14159265359;
				return TRUE;
			}
	}

	for(int i = 0;i < len;i++){
		if(m_input[i] == '.'){
			if(has_decimal)
				return FALSE;
			has_decimal = TRUE;
		}
		else{
			if(!isdigit(m_input[i])){
				// Not a number
				return FALSE;
			}
		}
	}

	// Compute the numerical value of the string
	numeric_val = atof(m_input);

	return TRUE;
}

BOOL EqToken::isVariable(char *m_input, int len)
{
	
	// Varibles must be at least one character long
	if(len == 0)
		return FALSE;

	// Variables must start with a letter
	if(!isalpha(m_input[0]))
		return FALSE;

	// Variables can only contain letters and numbers,
	// no special characters (i.e. @3~ etc.).
	for(int i = 1;i < len;i++){
		if(!isalnum(m_input[i]))
			return FALSE;
	}

	return TRUE;
}

char *EqToken::Set(char *m_input, int len)
{
	switch(len){
		case 0: // An error has occured
			return "Zero length string in EqToken::Set";
		case 1: // This is an operator or variable
			if(isVariable(m_input, len)){ // A variable
				token_type = VALUE_VAR;
			}
			else{
				if(isNumber(m_input, len)){ // A single digit number
					token_type = VALUE_CONST;
				}else{ // An operator
					switch(m_input[0]){
						case '+':
							// By default, unary and binary '+' are initially
							// assigned BINARY_ADD.
							token_type = BINARY_ADD;
							break;
						case '-':
							// By default, unary and binary '-' are initially
							// assigned BINARY_SUB.
							token_type = BINARY_SUB;
							break;
						case '*':
							token_type = BINARY_MULTI;
							break;
						case '/':
							token_type = BINARY_DIV;
							break;
						case '^':
							token_type = BINARY_POW;
							break;
						case '(':
							token_type = UNARY_L_BRACE;
							break;
						case '[':
							token_type = UNARY_L_BRACE;
							break;
						case '{':
							token_type = UNARY_L_BRACE;
							break;
						case ')':
							token_type = UNARY_R_BRACE;
							break;
						case ']':
							token_type = UNARY_R_BRACE;
							break;
						case '}':
							token_type = UNARY_R_BRACE;
							break;
						default:
							// Not an operator -- an error has occured
							return "Invalid character";
					};
				}
			}
			break;

		case 2: // Could be an operator, variable or 2 digit number
		case 3: // Could be an operator, variable or 3 digit number
		case 4: // Could be an operator, variabel or 4 digit number
			token_type = GetOperator(m_input, len);

			if(token_type == -1){
				// Not an operator, is it a number?
				if(isNumber(m_input, len))
					token_type = VALUE_CONST;
				else{
					if(isVariable(m_input, len)){
						token_type = VALUE_VAR;
					}
					else{
						// Not an operator, variable or 
						// number -- an error has occured
						return "Unknown symbol";
					}
				}
			}	

			break;
		default:
			// Must be a number or variable
			if(isNumber(m_input, len))
				token_type = VALUE_CONST;
			else{
				if(isVariable(m_input, len)){
					token_type = VALUE_VAR;
				}
				else{
					// Not an operator, variable or 
					// number -- an error has occured
					return "Unknown symbol";
				}
			}
	};
	
	if(id)
		delete [] id;

	id = new char [len + 1];

	// Copy only len characters of the input string
	strncpy(id, m_input, len);
	id[len] = '\0';

	return NULL;
}