//	(c) 2000 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.
#include "stdafx.h"
#include "sgraph.h"
#include "parseOptions.h"
#include "color.h"

// Case Insensitive string handling routines
char *strstrCI(const char *target, const char *query);
int strncmpCI(const char *target, const char *query, int n);

int ImportOptions::Parse(char *buffer)
{
	// Find the first instance of a option command
	buffer = strchr(buffer, '@');

	// Does buffer contain an option command?
	if(buffer == NULL)
		return PARSE_OPTIONS_NO_DATA;

	int tmp = strlen(buffer);
	
	// Look for the keyword "set"
	char *ptr = strstrCI(buffer, "set[");

	if(ptr != NULL)
		return ParseSet(ptr);

	// Look for the keyword "graph"
	ptr = strstrCI(buffer, "graph.");

	if(ptr != NULL)
		return ParseGraph(ptr);

	return PARSE_OPTIONS_ERROR_MALFORMED;
}

int ImportOptions::ParseSet(char *buffer)
{
	if(strlen(buffer) < 4)
		return PARSE_OPTIONS_ERROR_MALFORMED;

	// Move the buffer pointer to the point after '['
	buffer += 4;

	// Find the closing brace
	char *ptr = strchr(buffer, ']');

	if(ptr == NULL)
		return PARSE_OPTIONS_ERROR_MALFORMED;

	// Terminate the buffer string and avance the pointer
	ptr[0] = '\0';
	ptr ++;

	// Get the index and make sure it is at least greater than
	// zero (a little sanity check never hurt anyone ...).
	if(strlen(buffer) == 0)
		return PARSE_OPTIONS_ERROR_BAD_INDEX;

	set_index = atoi(buffer);

	if(set_index < 0)
		return PARSE_OPTIONS_ERROR_BAD_INDEX;

	// In this section, enumerate all of the allowed commands
	if(strncmpCI(ptr, ".point.style", strlen(".point.style")) == 0){
		ptr += strlen(".point.style");
		
		// Skip any white space
		while(isspace(ptr[0]))
			ptr++;

		if(ptr[0] == '\0')
			return PARSE_OPTIONS_ERROR_BAD_POINT_STYLE;
		
		if(strncmpCI(ptr, "circle", strlen("circle")) == 0){
			set_pointType = SPlot::POINT_CIRCLE;
			return PARSE_OPTIONS_DATA;
		}

		if(strncmpCI(ptr, "square", strlen("square")) == 0){
			set_pointType = SPlot::POINT_SQUARE;
			return PARSE_OPTIONS_DATA;
		}
		
		if(strncmpCI(ptr, "triangle", strlen("triangle")) == 0){
			set_pointType = SPlot::POINT_TRIANGLE;
			return PARSE_OPTIONS_DATA;
		}
		
		if(strncmpCI(ptr, "dot", strlen("dot")) == 0){
			set_pointType = SPlot::POINT_DOT;
			return PARSE_OPTIONS_DATA;
		}
		
		if(strncmpCI(ptr, "histogramx", strlen("histogramx")) == 0){
			set_pointType = SPlot::POINT_HISTOGRAM_X;
			return PARSE_OPTIONS_DATA;
		}
		
		if(strncmpCI(ptr, "histogramy", strlen("histogramy")) == 0){
			set_pointType = SPlot::POINT_HISTOGRAM_Y;
			return PARSE_OPTIONS_DATA;
		}
		
		if(strncmpCI(ptr, "vector", strlen("vector")) == 0){
			set_pointType = SPlot::POINT_VECTOR;
			return PARSE_OPTIONS_DATA;
		}

		if(strncmpCI(ptr, "none", strlen("none")) == 0){
			set_pointType = SPlot::POINT_NONE;
			return PARSE_OPTIONS_DATA;
		}

		// If we get here an error has occured!
		return PARSE_OPTIONS_ERROR_BAD_POINT_STYLE;
	}

	if(strncmpCI(ptr, ".point.color", strlen(".point.color")) == 0){
		ptr += strlen(".point.color");
		
		// Skip any white space
		while(isspace(ptr[0]))
			ptr++;

		if(ptr[0] == '\0')
			return PARSE_OPTIONS_ERROR_BAD_POINT_COLOR;
		
		if(strncmpCI(ptr, "black", strlen("black")) == 0){
			set_pointColor = S_CLR_BLACK;
			return PARSE_OPTIONS_DATA;
		}

		if(strncmpCI(ptr, "white", strlen("white")) == 0){
			set_pointColor = S_CLR_WHITE;
			return PARSE_OPTIONS_DATA;
		}
		
		if(strncmpCI(ptr, "red", strlen("red")) == 0){
			set_pointColor = S_CLR_RED;
			return PARSE_OPTIONS_DATA;
		}
		
		if(strncmpCI(ptr, "green", strlen("green")) == 0){
			set_pointColor = S_CLR_GREEN;
			return PARSE_OPTIONS_DATA;
		}
		
		if(strncmpCI(ptr, "blue", strlen("blue")) == 0){
			set_pointColor = S_CLR_BLUE;
			return PARSE_OPTIONS_DATA;
		}
		
		if(strncmpCI(ptr, "cyan", strlen("cyan")) == 0){
			set_pointColor = S_CLR_CYAN;
			return PARSE_OPTIONS_DATA;
		}
		
		if(strncmpCI(ptr, "grey", strlen("grey")) == 0){
			set_pointColor = S_CLR_GREY;
			return PARSE_OPTIONS_DATA;
		}
		
		if(strncmpCI(ptr, "none", strlen("none")) == 0){
			set_pointColor = S_CLR_NONE;
			return PARSE_OPTIONS_DATA;
		}

		if(strncmpCI(ptr, "custom", strlen("custom")) == 0){
			ptr += strlen("custom");
			
			return ParseRBGColor(ptr, set_pointColor);
		}

		// If we get here an error has occured!
		return PARSE_OPTIONS_ERROR_BAD_POINT_COLOR;
	}


	if(strncmpCI(ptr, ".point.size", strlen(".point.size")) == 0){
		ptr += strlen(".point.size");
		
		// Skip any white space
		while(isspace(ptr[0]))
			ptr++;

		if(ptr[0] == '\0')
			return PARSE_OPTIONS_ERROR_BAD_POINT_SIZE;
		
		set_pointSize = atoi(ptr);

		if((set_pointSize < 1) || (set_pointSize > MAX_POINT_SIZE))
			return PARSE_OPTIONS_ERROR_BAD_POINT_SIZE;
		
		return PARSE_OPTIONS_DATA;
	}

	if(strncmpCI(ptr, ".line.size", strlen(".line.size")) == 0){
		ptr += strlen(".line.size");

		// Skip any white space
		while(isspace(ptr[0]))
			ptr++;

		if(ptr[0] == '\0')
			return PARSE_OPTIONS_ERROR_BAD_LINE_SIZE;
		
		set_lineSize = atoi(ptr);

		if((set_lineSize < 1) || (set_lineSize > MAX_LINE_SIZE))
			return PARSE_OPTIONS_ERROR_BAD_LINE_SIZE;
		
		return PARSE_OPTIONS_DATA;
	}

	if(strncmpCI(ptr, ".line.style", strlen(".line.style")) == 0){
		ptr += strlen(".line.style");

		// Skip any white space
		while(isspace(ptr[0]))
			ptr++;

		if(ptr[0] == '\0')
			return PARSE_OPTIONS_ERROR_BAD_LINE_STYLE;
		
		if(strncmpCI(ptr, "solid", strlen("solid")) == 0){
			set_lineType = SPlot::LINE_SOLID;
			return PARSE_OPTIONS_DATA;
		}

		if(strncmpCI(ptr, "dotted", strlen("dotted")) == 0){
			set_lineType = SPlot::LINE_DOTTED;
			return PARSE_OPTIONS_DATA;
		}
		
		if(strncmpCI(ptr, "dashed", strlen("dashed")) == 0){
			set_lineType = SPlot::LINE_DASHED;
			return PARSE_OPTIONS_DATA;
		}
		
		if(strncmpCI(ptr, "dashdot", strlen("dashdot")) == 0){
			set_lineType = SPlot::LINE_DASHDOT;
			return PARSE_OPTIONS_DATA;
		}
		
		if(strncmpCI(ptr, "dashdotdot", strlen("dashdotdot")) == 0){
			set_lineType = SPlot::LINE_DASHDOTDOT;
			return PARSE_OPTIONS_DATA;
		}
		
		if(strncmpCI(ptr, "none", strlen("none")) == 0){
			set_lineType = SPlot::LINE_NONE;
			return PARSE_OPTIONS_DATA;
		}

		return PARSE_OPTIONS_DATA;
	}

	if(strncmpCI(ptr, ".line.color", strlen(".line.color")) == 0){
		ptr += strlen(".line.color");
		
		// Skip any white space
		while(isspace(ptr[0]))
			ptr++;

		if(ptr[0] == '\0')
			return PARSE_OPTIONS_ERROR_BAD_LINE_COLOR;
		
		if(strncmpCI(ptr, "black", strlen("black")) == 0){
			set_lineColor = S_CLR_BLACK;
			return PARSE_OPTIONS_DATA;
		}

		if(strncmpCI(ptr, "white", strlen("white")) == 0){
			set_lineColor = S_CLR_WHITE;
			return PARSE_OPTIONS_DATA;
		}
		
		if(strncmpCI(ptr, "red", strlen("red")) == 0){
			set_lineColor = S_CLR_RED;
			return PARSE_OPTIONS_DATA;
		}
		
		if(strncmpCI(ptr, "green", strlen("green")) == 0){
			set_lineColor = S_CLR_GREEN;
			return PARSE_OPTIONS_DATA;
		}
		
		if(strncmpCI(ptr, "blue", strlen("blue")) == 0){
			set_lineColor = S_CLR_BLUE;
			return PARSE_OPTIONS_DATA;
		}
		
		if(strncmpCI(ptr, "cyan", strlen("cyan")) == 0){
			set_lineColor = S_CLR_CYAN;
			return PARSE_OPTIONS_DATA;
		}
		
		if(strncmpCI(ptr, "grey", strlen("grey")) == 0){
			set_lineColor = S_CLR_GREY;
			return PARSE_OPTIONS_DATA;
		}
		
		if(strncmpCI(ptr, "none", strlen("none")) == 0){
			set_lineColor = S_CLR_NONE;
			return PARSE_OPTIONS_DATA;
		}

		if(strncmpCI(ptr, "custom", strlen("custom")) == 0){
			ptr += strlen("custom");
			return ParseRBGColor(ptr, set_lineColor);
		}

		// If we get here an error has occured!
		return PARSE_OPTIONS_ERROR_BAD_LINE_COLOR;
	}

	if(strncmpCI(ptr, ".errorbar.display", strlen(".errorbar.display")) == 0){
		ptr += strlen(".errorbar.display");
		
		// Skip any white space
		while(isspace(ptr[0]))
			ptr++;

		if(ptr[0] == '\0')
			return PARSE_OPTIONS_ERROR_BAD_ERROR_BAR;
		
		if(strncmpCI(ptr, "top", strlen("top")) == 0){
			set_errorBarDisplay = SPlot::BAR_TOP_LEFT;
			return PARSE_OPTIONS_DATA;
		}

		if(strncmpCI(ptr, "left", strlen("left")) == 0){
			set_errorBarDisplay = SPlot::BAR_TOP_LEFT;
			return PARSE_OPTIONS_DATA;
		}
		
		if(strncmpCI(ptr, "bottom", strlen("bottom")) == 0){
			set_errorBarDisplay = SPlot::BAR_BOTTOM_RIGHT;
			return PARSE_OPTIONS_DATA;
		}
		
		if(strncmpCI(ptr, "right", strlen("right")) == 0){
			set_errorBarDisplay = SPlot::BAR_BOTTOM_RIGHT;
			return PARSE_OPTIONS_DATA;
		}
		
		if(strncmpCI(ptr, "none", strlen("none")) == 0){
			set_errorBarDisplay = SPlot::BAR_NONE;
			return PARSE_OPTIONS_DATA;
		}

		// If we get here an error has occured!
		return PARSE_OPTIONS_ERROR_BAD_ERROR_BAR;
	}

	if(strncmpCI(ptr, ".info", strlen(".info")) == 0){
		ptr += strlen(".info");
		
		// Skip any white space
		while(isspace(ptr[0]))
			ptr++;
		
		set_info = CString(ptr);
		set_info_flag = 1;

		return PARSE_OPTIONS_DATA;
	}

	// If we get here an error has occured!
	return PARSE_OPTIONS_ERROR_MALFORMED;
}

int ImportOptions::ParseGraph(char *buffer)
{
	buffer += strlen("graph.");

	if(strlen(buffer) == 0)
		return PARSE_OPTIONS_ERROR_MALFORMED;

	if(strncmpCI(buffer, "title", strlen("title")) == 0){
		buffer += strlen("title");

		// Skip white space
		while((buffer[0] == ' ') || (buffer[0] == '\t'))
			buffer ++;

		graph_title.Format("%s", buffer);

		return PARSE_OPTIONS_DATA;
	}

	if(strncmpCI(buffer, "xaxis.", strlen("xaxis.")) == 0){
		buffer += strlen("xaxis.");
		
		if(strncmpCI(buffer, "title", strlen("title")) == 0){
			buffer += strlen("title");

			// Skip white space
			while((buffer[0] == ' ') || (buffer[0] == '\t'))
				buffer ++;

			if(buffer == NULL)
				return PARSE_OPTIONS_ERROR_BAD_XAXIS_TILE;

			graph_xaxis_title.Format("%s", buffer);

			return PARSE_OPTIONS_DATA;
		}

		return PARSE_OPTIONS_ERROR_BAD_XAXIS_TILE;
	}

	if(strncmpCI(buffer, "yaxis.", strlen("yaxis.")) == 0){
		buffer += strlen("yaxis.");

		if(strncmpCI(buffer, "title", strlen("title")) == 0){
			buffer += strlen("title");

			// Skip white space
			while((buffer[0] == ' ') || (buffer[0] == '\t'))
				buffer ++;

			if(buffer == NULL)
				return PARSE_OPTIONS_ERROR_BAD_YAXIS_TILE;

			graph_yaxis_title.Format("%s", buffer);

			return PARSE_OPTIONS_DATA;
		}

		return PARSE_OPTIONS_ERROR_BAD_YAXIS_TILE;
	}

	// If we get here an error has occured!
	return PARSE_OPTIONS_ERROR_BAD_GRAPH_DATA;
}

ImportOptions& ImportOptions::operator=(const ImportOptions& copy)
{
	set_index = copy.set_index;

	// Only overwrite a variable if it is equal to -1
	// (i.e. uninitialized).
	if(set_pointType == -1)
		set_pointType = copy.set_pointType;

	if(set_pointSize == -1)
		set_pointSize = copy.set_pointSize;
	
	if(set_lineType == -1)
		set_lineType = copy.set_lineType;
	
	if(set_lineSize == -1)
		set_lineSize = copy.set_lineSize;
	
	if(set_lineColor == -1)
		set_lineColor = copy.set_lineColor;

	if(set_pointColor == -1)
		set_pointColor = copy.set_pointColor;

	if(set_errorBarDisplay == -1)
		set_errorBarDisplay = copy.set_errorBarDisplay;
	
	if(set_info_flag == -1){
		set_info_flag = copy.set_info_flag;
		set_info = copy.set_info;
	}

	graph_title = copy.graph_title;
	graph_xaxis_title = copy.graph_xaxis_title;
	graph_yaxis_title = copy.graph_yaxis_title;

	return (*this);
}

int ImportOptions::ParseRBGColor(char *ptr, int &clr)
{
	// Parse a string of the form "r g b"
	// where 0 <= r <= 255, 0 <= g <= 255, 0 <= b <= 255

	int r, g, b;

	if(sscanf(ptr, "%d %d %d", &r, &g, &b) != 3)
		return PARSE_OPTIONS_ERROR_BAD_CUSTOM_COLOR;
	
	// Check the bounds of each color index
	if((r < 0) || (r > 255) ||
	   (g < 0) || (g > 255) ||
	   (b < 0) || (b > 255))
	   return PARSE_OPTIONS_ERROR_BAD_CUSTOM_COLOR;

	// Convert the RGB values into a COLORREF structure (i.e. 32 bit int).
	clr = (b & 0x000000ff) << 8;
	clr = (clr | (g & 0x000000ff)) << 8;
	clr = clr | (r & 0x000000ff);

	// Negate the color value
	clr = -clr;

	return PARSE_OPTIONS_DATA;
}

void ImportOptionList::Add(ImportOptions &opt)
{
	// Don't add an index whose value is -1 (i.e. uninitialized).
	//if(opt.set_index == -1)
	//	return;

	for(int i = 0;i < opt_list.Length();i++)
		if((opt.set_index != -1) && 
		   (opt_list[i].set_index == opt.set_index)){
			opt_list[i] = opt;
			return;
		}
	
	// The given set index was not found in the list, so
	// add it to the list.
	opt_list.Push(opt);
}

int ImportOptionList::GetOption(int i, ImportOptions &opt)
{
	for(int j = 0;j < opt_list.Length();j++)
		if(opt_list[j].set_index == i){
			opt= opt_list[j];
			return TRUE;
		}

	// The requested set was NOT found!
	return FALSE;
}

CString ImportOptionList::GraphTitle()
{
	for(int j = 0;j < opt_list.Length();j++)
		if((opt_list[j].set_index == -1) && (opt_list[j].graph_title.GetLength() != 0)){
			return opt_list[j].graph_title;
		}

	// The graph title was NOT found!
	return CString();
}

CString ImportOptionList::XAxisTitle()
{
	for(int j = 0;j < opt_list.Length();j++)
		if((opt_list[j].set_index == -1) && (opt_list[j].graph_xaxis_title.GetLength() != 0)){
			return opt_list[j].graph_xaxis_title;
		}

	// The xaxis title was NOT found!
	return CString();
}

CString ImportOptionList::YAxisTitle()
{
	for(int j = 0;j < opt_list.Length();j++)
		if((opt_list[j].set_index == -1) && (opt_list[j].graph_yaxis_title.GetLength() != 0)){
			return opt_list[j].graph_yaxis_title;
			return TRUE;
		}

	// The yaxis title title was NOT found!
	return CString();
}

// Find the first occurance of m_query within m_target without
// regards to case.
char *strstrCI(const char *m_target, const char *m_query)
{
	int t_len = strlen(m_target), 
		q_len = strlen(m_query);

	char *target = new char [t_len + 1];
	char *query = new char [q_len + 1];

	int i;

	for(i = 0;i < t_len;i++)
		target[i] = tolower(m_target[i]);

	// NULL terminate the string
	target[i] = '\0';

	for(i = 0;i < q_len;i++)
		query[i] = tolower(m_query[i]);

	// NULL terminate the string
	query[i] = '\0';

	char *ptr = strstr(target, query);
	
	if(ptr != NULL){
		*ptr = '\0';
		i = strlen(target);
	}
	else
		i = -1;

	delete [] target;
	delete [] query;

	if(i == -1)
		return NULL;

	return (char*)(m_target + i);
}

// Compare m_query to m_target without regards to case.
int strncmpCI(const char *m_target, const char *m_query, int n)
{
	return _strnicmp(m_target, m_query, n);
}
