//	(c) 2000-2002 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.
#ifndef __SAXIS_OBJECT
#define __SAXIS_OBJECT

#include "sobject.h"

// A Graph axis class. Responsible for managing tick marks and tick labels.
class SAxisObject : public SObject{
private:

	// Hide the major_tick_space and minor_tick_space values (as private)
	// to implement "sticky" tick spaces.
	float major_tick_space, minor_tick_space;
public:
	// Orientations and directions
	enum {HORIZONTAL, VERTICAL, LEFT, RIGHT, UP, DOWN};
	
	// Tick lable formats
	enum {GENERAL_LABLE,			// i.e. 0.01
		  ENGINEERING_LABLE,		// i.e. 1.0e-2
		  SCIENTIFIC_LABLE};		// i.e. 1.0x10^{-2}

	// What kind of axis (i.e. linear or log)?
	int scaleType; 

	COLORREF axis_color;

	float line_length;
	int orientation;
	int isLineDrawn;
	
	// The major and minor tick lengths are in absolute
	// screen (display) dimensions
	int major_tick_length, minor_tick_length;

	// Are the tick mark spaces "sticky"? Sticky tick spaces only change
	// (a) when the user changes them and (b) when the plotting range increases
	// to the point where an order of magnitude more ticks would be required to
	// maintain the tick spacing.
	BOOL isStickyTickSpace;
	
	int direction;
	int isTickDrawn;
	int isGridDrawn;

	COLORREF grid_color;
	int grid_size;
	int grid_type;

	LOGFONT tick_font;
	COLORREF tick_font_color;

	int tick_lable_precision;
	float tick_min, tick_max;
	int isTickLableDrawn;
	int tick_lable_location;
	int tick_lable_format;

	virtual void Serialize(CArchive& ar);

	SAxisObject(float m_x, float m_y, float m_line_length, int m_orientation,
		float m_tick_min, float m_tick_max, int m_direction,
		int m_lable_location);

	void DrawLabel(CDC* pDC, CRect &extent, float label, float label_loc);
	void DrawExponent(CDC* pDC, char text[], int n, int x, int y);

	void Draw(CDC* pDC, CRect &extent);
	void DrawSelect(CDC* pDC, CRect &extent);
	void DrawTickMarks(CDC* pDC, CRect &extent, float major_dt, 
		float minor_dt, float l2fudge);
	void DrawGrid(CDC* pDC, CRect &extent, float grid_length);
	void DrawTickLabels(CDC* pDC, CRect &extent, float major_dt, 
		float minor_dt, float l2);

	CSelectRect SelectRect(CDC* pDC, CRect &extent);

	void PreparePrinting(CDC* pDC, long scaleMul, long scaleDiv, CRect &extent);
	void CleanUpPrinting(CDC* pDC, CRect &extent);

	float MajorTickSpace()
	{
		return major_tick_space;
	};

	void MajorTickSpace(float space, BOOL force_sticky = FALSE)
	{
		if(force_sticky || !isStickyTickSpace){
			major_tick_space = space;
		}
		else{
			// Only update the tick space if it's an order of
			// magnitude different than the current space
			if((space > 10.0f*major_tick_space) || (space < major_tick_space/10.0f)){
				major_tick_space = space;
			}
		}
	};

	float MinorTickSpace()
	{
		return minor_tick_space;
	};

	void MinorTickSpace(float space, BOOL force_sticky = FALSE)
	{
		if(force_sticky || !isStickyTickSpace){
			minor_tick_space = space;
		}
		else{
			// Only update the tick space if it's an order of
			// magnitude different than the current space
			if((space > 10.0f*minor_tick_space) || (space < minor_tick_space/10.0f)){
				minor_tick_space = space;
			}
		}
	};

	// These functions are identical to the SPlot::GetLineStyle functions.
	DWORD* GetLineStyle(int lt, int &db_size);
	int GetLineStyle(int lt);
};

#endif // __SAXIS_OBJECT