//	(c) 2000 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.
/////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_SGRAPHVIEW_H__828224EA_F061_11D3_A367_00C04F72D141__INCLUDED_)
#define AFX_SGRAPHVIEW_H__828224EA_F061_11D3_A367_00C04F72D141__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

// Note that afxpriv.h must be included to get the declaration
// of the CPreviewDC class. The following line should be included
// at the very top of your view's .CPP file
#include <afxpriv.h>
#include "AnimatePlotDialog.h"
#include "history.h"

// Enumerate the different (mutually exclusive) pointer states
enum {SG_POINTER_FREE = 0, SG_POINTER_ZOOM_START, SG_POINTER_ZOOM, 
	SG_POINTER_TEXT, SG_POINTER_SCROLL};

#define		WM_START_PIPE			(WM_USER + 4)
#define		WM_UPDATE_WYSIWYG		(WM_USER + 7)



class CSgraphView : public CView
{
protected: // create from serialization only
	CSgraphView();
	DECLARE_DYNCREATE(CSgraphView)
	
// Attributes
public:
	CSgraphDoc* GetDocument();

	void InitDefaults();

	// Does the clipboard contain data in format that prestoplot
	// can import?
	UINT GetClipFormat();

	// Stop reading from a data pipe
	void StopPipe();

	// The dimensions of the drawing area
	CRect extent;

	// Display cross hairs centered on the cursor?
	BOOL isCrossHair;
	CPoint lastCross;

	int pointerState;

	CPoint zoomStart, zoomEnd;
	CPoint scrollStart, scrollEnd;

	// Are we printing?
	BOOL isPrinting;

	// Are we in WYSIWYG mode?
	BOOL isWYSIWYG;
	
	// How big is the "printer"?
	CRect printerExtent;

	// The aspect ratio for WYSIWYG mode
	float aspectWYSIWYG;

	// Do we need to render a background color explicitly?
	BOOL render_background_color;

	// Have we selected a non-graph object (if so, get its pointer)?
	SObject *isSelected;
	BOOL isMoving;
	float move_x, move_y;

	CAnimatePlotDialog *capDlg;

	// History lists for user interface persistence
	// (none of these lists are saved to disk -- the persistance
	// is only for the lenght of the session).
	list<EquationInfo> eqHistory;
	list<ParametricEquationInfo> paraEqHistory;
	list<PolarEquationInfo> polarEqHistory;
	list<FitEquationInfo> fitEqHistory;

	void ForceRedraw();
	void InvalidateBuffer();

protected:
	// The arrow, cross, move and I-beam cursors
	HCURSOR m_hcurCurrent, m_hcurArrow, m_hcurCross, 
		m_hcurMove, m_hcurIBeam, m_hcurPipe;

	// Use double buffering? This variable is set in the
	// constructor and enables (or disables) double buffering
	// for screen rendering.
	BOOL doubleBuffer;

	CDC *m_pDC;
	CBitmap *drawBuffer;

	// Use ForceRedraw to repaint the window (instead of 
	// InvalidateRect - which is not aware of double buffering.
	BOOL flushBuffer;

	// Image export functions
	HBITMAP MakeDIB(CDC* pDC, int dib_width, int dib_height, int dib_bpp);
	void ExportImageBMP(const char* filename);
	void ExportImageBMP(const char* filename, int width, int height);
	void ExportImagePNG(const char* filename);
	void ExportImagePNG(const char* filename, int width, int height);

// Operations
public:

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CSgraphView)
	public:
	virtual void OnDraw(CDC* pDC);  // overridden to draw this view
	virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
	protected:
	virtual void OnPrint(CDC *pDC, CPrintInfo* pInfo);
	virtual BOOL OnPreparePrinting(CPrintInfo* pInfo);
	virtual void OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint);
	//}}AFX_VIRTUAL

	afx_msg LRESULT OnAnimatePlotClose(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT OnAnimatePlotAdjust(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT StartPipe(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT SetWYSIWYGAspectRatio(WPARAM wParam = 0, LPARAM lParam = 0);

// Implementation
public:
	virtual ~CSgraphView();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif
	
	BOOL SelectObject(CPoint point);

// Generated message map functions
protected:
	//{{AFX_MSG(CSgraphView)
	afx_msg void OnSize(UINT nType, int cx, int cy);
	afx_msg void OnMouseMove(UINT nFlags, CPoint point);
	afx_msg void OnPlotDefaultScale();
	afx_msg void OnPlotZoom();
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnLButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnMButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnMButtonUp(UINT nFlags, CPoint point);
	afx_msg BOOL OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message);
	afx_msg void OnLButtonDblClk(UINT nFlags, CPoint point);
	afx_msg void OnPlotArrange();
	afx_msg void OnEditCopy();
	afx_msg void OnDataStat();
	afx_msg void OnViewCrosshair();
	afx_msg void OnUpdateViewCrosshair(CCmdUI* pCmdUI);
	afx_msg void OnDataHist();
	afx_msg void OnGraphXAxis();
	afx_msg void OnGraphYAxis();
	afx_msg void OnPlotScaling();
	afx_msg void OnFileDataExport();
	afx_msg void OnFileExportAudio();
	afx_msg void OnDataDeriv();
	afx_msg void OnRButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnPlotUserText();
	afx_msg void OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnEditClear();
	afx_msg void OnUpdateEditClear(CCmdUI* pCmdUI);
	afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);
	afx_msg void OnDestroy();
	afx_msg BOOL OnEraseBkgnd(CDC* pDC);
	afx_msg void OnViewDoublebuffer();
	afx_msg void OnUpdateViewDoublebuffer(CCmdUI* pCmdUI);
	afx_msg void OnKillFocus(CWnd* pNewWnd);
	afx_msg void OnPlotGraphColor();
	afx_msg void OnPlotGraphSelect();
	afx_msg void OnDataIntegrate();
	afx_msg void OnAnimatePlot();
	afx_msg void OnUpdateAnimatePlot(CCmdUI* pCmdUI);
	afx_msg void OnViewRefresh();
	afx_msg void OnEditPaste();
	afx_msg void OnUpdateEditPaste(CCmdUI* pCmdUI);
	afx_msg void OnPlotLines();
	afx_msg void OnViewWysiwyg();
	afx_msg void OnUpdateViewWysiwyg(CCmdUI* pCmdUI);
	afx_msg void OnFileExportAnimation();
	afx_msg void OnViewProperties();
	afx_msg void OnEditCopyData();
	afx_msg BOOL OnMouseWheel(UINT nFlags, short zDelta, CPoint pt);
	afx_msg void OnDataFunc();
	afx_msg void OnDataNonLin();
	afx_msg void OnPageColor();
	afx_msg void OnDataParaFunc();
	afx_msg void OnDataPolarFunc();
	afx_msg void OnFileExportImage();
	afx_msg void OnPageOverlap();
	afx_msg void OnUpdatePageOverlap(CCmdUI* pCmdUI);
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

#ifndef _DEBUG  // debug version in sgraphView.cpp
inline CSgraphDoc* CSgraphView::GetDocument()
   { return (CSgraphDoc*)m_pDocument; }
#endif

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_SGRAPHVIEW_H__828224EA_F061_11D3_A367_00C04F72D141__INCLUDED_)
