//	(c) 2000-2002 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.
#ifndef __SLINE_OBJECT
#define __SLINE_OBJECT

#include "Excepts.h"

// The SLineObject class encapsulates user defined horizontal and vertical 
// lines.
class SLineObject{
private:
	int var_type; // X or Y
	int line_type;
	float c; // i.e. x or y = c
public:
	// Enumerate the line types. The order of the inumeration
	// must match the order of appearance in the plot
	// options dialog box.
	enum {LINE_SOLID = 0, LINE_DOTTED, LINE_DASHED, LINE_DASHDOT, 
		 LINE_DASHDOTDOT};	// <-- Differes from the Splot class by
							// the absence of a LINE_NONE type.

	enum {X = 0, Y};

	SLineObject()
	{
		SetVar(Y, 0.0f);
		SetLineType(LINE_SOLID);
	};

	inline void SetVar(int m_var_type, float m_var_value)
	{
		#ifdef _DEBUG
		if((m_var_type != X) && (m_var_type != Y))
			throw Error("Illegal type!", __FILE__, __LINE__);
		#endif // _DEBUG

		var_type = m_var_type;

		// Save the line value in both x and y.
		c = m_var_value;
	};

	inline void SetLineType(int m_line_type)
	{
		#ifdef _DEBUG
		if((m_line_type < LINE_SOLID) || (m_line_type >  LINE_DASHDOTDOT))
			throw Error("Illegal type!", __FILE__, __LINE__);
		#endif // _DEBUG

		line_type = m_line_type;
	};
	
	inline int GetVarType()
	{
		return var_type;
	};

	inline int GetLineType()
	{
		return line_type;
	};

	inline float GetValue()
	{
		return c;
	};

	void Serialize(CArchive& ar);

	// This function, GetLineStyle, is a kludge! It closely resembles the function of the
	// same name in the SPlot class, yet must take the isPrinting, printScaleMul, and
	// printScaleDiv variables as arguments (because SLineObject does not inherit 
	// SObject). Not happy with this but must finish thesis ...
	inline DWORD* GetLineStyle(int &db_size, BOOL isPrinting, 
							   int printScaleMul, int printScaleDiv)
	{
		DWORD *buffer = NULL;

		switch(line_type){
			case LINE_SOLID:
				db_size = 2;
				buffer = new DWORD[db_size];

				buffer[0] = 1;
				buffer[1] = 0;
				break;
			case LINE_DOTTED:
				db_size = 2;
				buffer = new DWORD[db_size];
				
				buffer[0] = 4;
				buffer[1] = 4;
				break;
			case LINE_DASHED:
				db_size = 2;
				buffer = new DWORD[db_size];

				buffer[0] = 18;
				buffer[1] = 8;
				break;
			case LINE_DASHDOT:
				db_size = 4;
				buffer = new DWORD[db_size];

				buffer[0] = 8;
				buffer[1] = 6;
				buffer[2] = 2;
				buffer[3] = 6;
				break;
			case LINE_DASHDOTDOT:
				db_size = 6;
				buffer = new DWORD[db_size];

				buffer[0] = 8;
				buffer[1] = 4;
				buffer[2] = 2;
				buffer[3] = 4;
				buffer[4] = 2;
				buffer[5] = 4;
				break;
		};
		
		if(isPrinting && buffer){
			for(int i = 0;i < db_size;i++)
				buffer[i] = MulDiv(buffer[i], printScaleMul, printScaleDiv);
		}

		return buffer;
	};
};

#endif // __SLINE_OBJECT