//	(c) 2000-2002 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.
#ifndef __SOBJECT
#define __SOBJECT

/////////////////////////////////////////////////////////
// Include symbols for the GUI elements. Not sure this
// file should live here (except that almost every other
// file includes sobject.h).
#include "resource.h"
/////////////////////////////////////////////////////////

#include "SelectRect.h"
#include "version.h"
#include "presto.h"

// ************************** Global variables **************************
// Are we running on a OS that only allows 16 bit screen coordinates 
// (i.e. 95 98)? If false, assume 32 bit screen coordiantes (i.e. NT and 2000).
extern BOOL is_16_bit;
// ************************ End Global variables ************************

/////////////////////////////////////////////////////////////////////////////
//	*** Helper functions ***
/////////////////////////////////////////////////////////////////////////////
void SerializeFont(LOGFONT &font, CArchive& ar);


// A fundametal "screen" object. This class provides a
// framework for most of the other classes in PrestoPlot.
class SObject{
protected:
	BOOL isPrinting;
public:
	BOOL isDrawn;
	BOOL isSelected;
	
	// The center of the object
	float x, y;
	
	// Version control (for disk IO)
	float version;

	// How to scale objects that are specified with
	// fixed screen coordinates. PrintSize = 
	// MulDiv(ScreenSize, printScaleMul, printScaleDiv).
	// These values are not saved or read by Serialize().
	int printScaleMul, printScaleDiv;

	// Allow all lines to be scaled by a common factor (lineScale > 0)
	static int lineScale;

	// Allow all fonts to be scaled by a common factor (fontScale > 0)
	static float fontScale;

	SObject(){
		isDrawn = TRUE;
		isSelected = FALSE;
		isPrinting = FALSE;
		printScaleMul = 1;
		printScaleDiv = 1;
		version = SGRAPH_VERSION;
		x = 0.0f;
		y = 0.0f;
	};

	virtual void Draw(CDC* pDC, CRect &extent) = 0;
	virtual void DrawSelect(CDC* pDC, CRect &extent) = 0;

	// What happens when the user attempts to delete this object?
	virtual void OnDelete()
	{
		// By default, do nothing.
	};

	// Is the point xy (in screen coordinates) within the 
	// selection region of this object?
	virtual BOOL Select(CPoint &xy, CDC* pDC, CRect &extent);
	virtual CSelectRect SelectRect(CDC* pDC, CRect &extent) = 0;

	virtual void PreparePrinting(CDC* pDC, long scaleMul, long scaleDiv, CRect &extent);
	virtual void CleanUpPrinting(CDC* pDC, CRect &extent);
	
	//virtual void ScaleFonts(CDC* pDC) = 0;

	virtual void Printing(BOOL p)
	{
		isPrinting = p;
	};

	virtual BOOL Printing()
	{
		return isPrinting;
	};

	virtual float UpdateX(float m_x)
	{
		return (x = m_x);
	};

	virtual float UpdateY(float m_y)
	{
		return (y = m_y);
	};


	virtual float MoveX(float m_dx)
	{
		return (x += m_dx);
	};

	virtual float MoveY(float m_dy)
	{
		return (y += m_dy);
	};

	virtual void Serialize(CArchive& ar);

	inline float ScreentoX(int x, CRect &extent){
		return (float)(x - extent.left)/extent.Width();
	};
	
	inline float ScreentoY(int y, CRect &extent){
		return (float)(extent.bottom - y)/extent.Height();
	};

	inline int XtoScreen(float x, CRect &extent)
	{
		// Compute the screen coordiantes as a float to
		// prevent overflowing an integer.
		float sx = (float)(x*extent.Width() + extent.left);

		// Do not step over the 16 bit limit for 
		// addressable screen coordinates!
		if(is_16_bit){
			if(sx > GDI_16_BIT_MAX)
				return int(GDI_16_BIT_MAX);

			if(sx < GDI_16_BIT_MIN)
				return int(GDI_16_BIT_MIN);
		}
		else{ // using 32 bit values
			if(sx > GDI_32_BIT_MAX)
				return int(GDI_32_BIT_MAX);

			if(sx < GDI_32_BIT_MIN)
				return int(GDI_32_BIT_MIN);
		}

		return int(sx);
	}
	
	inline int YtoScreen(float y, CRect &extent)
	{
		// Compute the screen coordiantes as a float to
		// prevent overflowing an integer.	
		float sy = (float)(extent.bottom - y*extent.Height());

		// Do not step over the 16 bit limit for addressable 
		// screen coordinates!
		if(is_16_bit){
			if(sy > GDI_16_BIT_MAX)
				return int(GDI_16_BIT_MAX);

			if(sy < GDI_16_BIT_MIN)
				return int(GDI_16_BIT_MIN);
		}
		else{ // using 32 bit values
			if(sy > GDI_32_BIT_MAX)
				return int(GDI_32_BIT_MAX);

			if(sy < GDI_32_BIT_MIN)
				return int(GDI_32_BIT_MIN);
		}

		return int(sy);
	}
};

#endif // __SOBJECT