//	(c) 2000-2002 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.
#include "stdafx.h"
#include "splot.h"
#include "color.h"

/////////////////////////////////////////////////////////////////////////////
//	*** SPlot ***
/////////////////////////////////////////////////////////////////////////////
SPlot::SPlot()
{
	// Unless requested, NEVER empty the source plot
	// when copying!
	empty_on_copy = FALSE;

	x_scaleType = SCALE_LINEAR;
	y_scaleType = SCALE_LINEAR;
	
	display_x_error_bar = BAR_NONE;
	display_y_error_bar = BAR_NONE;
	error_bar_hat_size = 3;
	error_bar_line_type = LINE_SOLID;
	error_bar_line_size = 1;

	canXLog = FALSE;
	canYLog = FALSE;

	pointType = POINT_NONE;
	pointSize = 3;
	lineColor = 0x00000000; //Black

	lineType = LINE_SOLID;
	lineSize = 1;
	brush.lbColor = 0x00000000; //Black
	brush.lbStyle = BS_HOLLOW;

	info = ""; // No info yet

	// Default graph bounds
	minX = 0.0f;
	maxX = 1.0f;
	minY = 0.0f;
	maxY = 1.0f;
}

void SPlot::MoveDataOnCopy(BOOL m_flag)
{
	empty_on_copy = m_flag;
}

SPlot &SPlot::operator=(SPlot &m_copy)
{
	// Copy the base class
	SObject::operator=(m_copy);

	// Always set the empty_on_copy flags
	empty_on_copy = FALSE;

	// The m_copy variable "decides" if data should
	// be copied or moved.
	if(m_copy.empty_on_copy){
		// Use shallow reference semantics
		data.Move(m_copy.data);
		yErrorBar.Move(m_copy.yErrorBar);
		xErrorBar.Move(m_copy.xErrorBar);
	}
	else{
		data = m_copy.data;
		yErrorBar = m_copy.yErrorBar;
		xErrorBar = m_copy.xErrorBar;
	}

	display_y_error_bar = m_copy.display_y_error_bar;
	display_x_error_bar = m_copy.display_x_error_bar;

	error_bar_hat_size = m_copy.error_bar_hat_size;
	error_bar_line_type = m_copy.error_bar_line_type;
	error_bar_line_size = m_copy.error_bar_line_size;

	x1 = m_copy.x1;
	y1 = m_copy.y1;
	x2 = m_copy.x2;
	y2 = m_copy.y2;
	
	x_scaleType = m_copy.x_scaleType;
	y_scaleType = m_copy.y_scaleType;
	
	canXLog = m_copy.canXLog;
	canYLog = m_copy.canYLog;

	pointType = m_copy.pointType;
	pointSize = m_copy.pointSize;
	
	brush = m_copy.brush;

	lineType = m_copy.lineType;
	lineSize = m_copy.lineSize;
	lineColor = m_copy.lineColor;

	minX = m_copy.minX;
	maxX = m_copy.maxX;
	minY = m_copy.minY;
	maxY = m_copy.maxY;
	
	info = m_copy.info;

	legend = m_copy.legend;

	return (*this);
}

// Return the range of X values. Error bars are ignored.
// Return 0.0f if there are no data points in the plot.
float SPlot::GetRangeX()
{
	stackIterator<DataPoint> next(data);

	DataPoint xy;
	float x_low, x_high;

	if(next(xy)){
		x_low = x_high = xy.x;
	}
	else 
		return 0.0f;

	while(next(xy)){

		if(xy.x > x_high)
			x_high = xy.x;
		
		if(xy.x < x_low)
			x_low = xy.x;
	}

	return x_high - x_low;
}

// Return TRUE if there is data in the plot, FALSE otherwise.
// If there is NO data in the plot, the minX and maxX values
// should not be used!
BOOL SPlot::UpdateBoundsX()
{
	stackIterator<DataPoint> next(data);
	stackIterator<float> nextXBar(xErrorBar);

	DataPoint xy;
	float ebar;

	if(next(xy)){
		if(nextXBar(ebar)){
			minX = xy.x - ebar;
			maxX = xy.x + ebar;
		}
		else{
			minX = maxX = xy.x;
		}
	}
	else 
		return FALSE;

	while(next(xy)){
		if(!nextXBar(ebar))
			ebar = 0.0;

		if((xy.x + ebar) > maxX)
			maxX = xy.x + ebar;
		
		if((xy.x - ebar) < minX)
			minX = xy.x - ebar;
	}

	return TRUE;
}

// Return TRUE if there is data in the plot, FALSE otherwise.
// If there is NO data in the plot, the minY and maxY values
// should not be used!
BOOL SPlot::UpdateBoundsY()
{
	stackIterator<DataPoint> next(data);
	stackIterator<float> nextYBar(yErrorBar);

	DataPoint xy;
	float ebar;

	if(next(xy)){
		if(nextYBar(ebar)){
			minY = xy.y - ebar;
			maxY = xy.y + ebar;
		}
		else{
			minY = maxY = xy.y;
		}
	}
	else 
		return FALSE;

	while(next(xy)){
		if(!nextYBar(ebar))
			ebar = 0.0;

		if((xy.y + ebar) > maxY)
			maxY = xy.y + ebar;
		
		if((xy.y - ebar) < minY)
			minY = xy.y - ebar;
	}

	return TRUE;
}

// Return TRUE if there is data in the plot, FALSE otherwise.
// If there is NO data in the plot, the min and max values
// should not be used!
BOOL SPlot::UpdateBoundsXY()
{
	stackIterator<DataPoint> next(data);
	stackIterator<float> nextXBar(xErrorBar);
	stackIterator<float> nextYBar(yErrorBar);

	DataPoint xy;
	float ebar;

	if(next(xy)){
		if(nextXBar(ebar)){
			minX = xy.x - ebar;
			maxX = xy.x + ebar;
		}
		else{
			minX = maxX = xy.x;
		}
		
		if(nextYBar(ebar)){
			minY = xy.y - ebar;
			maxY = xy.y + ebar;
		}
		else{
			minY = maxY = xy.y;
		}
	}
	else 
		return FALSE;

	while(next(xy)){
		if(!nextXBar(ebar))
			ebar = 0.0;

		if((xy.x + ebar) > maxX)
			maxX = xy.x + ebar;
		
		if((xy.x - ebar) < minX)
			minX = xy.x - ebar;
		
		if(!nextYBar(ebar))
			ebar = 0.0;

		if((xy.y + ebar) > maxY)
			maxY = xy.y + ebar;
		
		if((xy.y - ebar) < minY)
			minY = xy.y - ebar;
	}

	return TRUE;
}

int SPlot::PlotX(float x, CRect &extent)
{
	if(x_scaleType == SCALE_LINEAR)
		return XtoScreen((x - minX)*(x2 - x1)/(maxX - minX) + x1, extent);

	return XtoScreen(((float)(log10(x)) - minX)*(x2 - x1)/(maxX - minX) + x1, extent);
}

int SPlot::PlotY(float y, CRect &extent)
{
	if(y_scaleType == SCALE_LINEAR)
		return YtoScreen((y - minY)*(y2 - y1)/(maxY - minY) + y1, extent);

	return YtoScreen(((float)(log10(y)) - minY)*(y2 - y1)/(maxY - minY) + y1, extent);
}

// Map local definitions of line styles to the windows definitions.
int SPlot::GetLineStyle(int lt)
{
	switch(lt){
		case LINE_SOLID:
			return PS_SOLID;
		case LINE_DOTTED:
			return PS_DOT;
		case LINE_DASHED:
			return PS_DASH;
		case LINE_DASHDOT:
			return PS_DASHDOT;
		case LINE_DASHDOTDOT:
			return PS_DASHDOTDOT;
	};
	
	return PS_NULL;
}

DWORD* SPlot::GetLineStyle(int lt, int &db_size)
{
	DWORD *buffer = NULL;

	switch(lt){
		case LINE_SOLID:
			db_size = 2;
			buffer = new DWORD[db_size];

			buffer[0] = 1;
			buffer[1] = 0;
			break;
		case LINE_DOTTED:
			db_size = 2;
			buffer = new DWORD[db_size];
			
			buffer[0] = 4;
			buffer[1] = 4;
			break;
		case LINE_DASHED:
			db_size = 2;
			buffer = new DWORD[db_size];

			buffer[0] = 18;
			buffer[1] = 8;
			break;
		case LINE_DASHDOT:
			db_size = 4;
			buffer = new DWORD[db_size];

			buffer[0] = 8;
			buffer[1] = 6;
			buffer[2] = 2;
			buffer[3] = 6;
			break;
		case LINE_DASHDOTDOT:
			db_size = 6;
			buffer = new DWORD[db_size];

			buffer[0] = 8;
			buffer[1] = 4;
			buffer[2] = 2;
			buffer[3] = 4;
			buffer[4] = 2;
			buffer[5] = 4;
			break;
	};
	
	if(isPrinting && buffer){
		for(int i = 0;i < db_size;i++)
			buffer[i] = MulDiv(buffer[i], printScaleMul, printScaleDiv);
	}

	return buffer;
}

BOOL SPlot::CheckBounds(DataPoint &p)
{
	// Does the point to be plotted lie within the min/max bounds?
	if(p.x < minX)
		return FALSE;

	if(p.x > maxX)
		return FALSE;
	
	if(p.y < minY)
		return FALSE;
			
	if(p.y > maxY)
		return FALSE;
	
	return TRUE;
}

void SPlot::PreparePrinting(CDC* pDC, long scaleMul, long scaleDiv, CRect &extent)
{
	SObject::PreparePrinting(pDC, scaleMul, scaleDiv, extent);

	legend.PreparePrinting(pDC, scaleMul, scaleDiv, extent);

	pointSize = MulDiv(pointSize, printScaleMul, printScaleDiv);
	error_bar_hat_size = MulDiv(error_bar_hat_size, printScaleMul, printScaleDiv);
}

void SPlot::CleanUpPrinting(CDC* pDC, CRect &extent)
{
	
	pointSize = MulDiv(pointSize, printScaleDiv, printScaleMul);
	error_bar_hat_size = MulDiv(error_bar_hat_size, printScaleDiv, printScaleMul);
	
	legend.CleanUpPrinting(pDC, extent);

	SObject::CleanUpPrinting(pDC, extent);
}

void SPlot::Draw(CDC* pDC, CRect &extent)
{
	if(!isDrawn)
		return;
	
	// Draw the line
	if(lineType != LINE_NONE){
		BOOL use_gdi_path;
		BOOL use_user_pen;

		CPen *oldpen;
		CPen *linepen = NULL;
		DWORD *dashBuffer = NULL;

		// What kind of pen do we need to construct?
		use_user_pen = (lineType != LINE_SOLID) && (lineSize*lineScale != 1) && (!is_16_bit);

		// Will we need to render the plot as a GDI path?
		use_gdi_path = isPrinting || use_user_pen || is_16_bit;

		if(use_user_pen){
			// Create the pen for drawing the lines. A few hoops must be jumped 
			// through in order to render thick, non-solid lines.
			LOGBRUSH penBrush;

			penBrush.lbStyle = BS_SOLID;
			penBrush.lbColor = lineColor;

			int dashSize;
			dashBuffer = GetLineStyle(lineType, dashSize);

			// Use the extented form of the CPen constructor to enable the use
			// of the thick dashed lines.
			linepen = new CPen(PS_GEOMETRIC | PS_ENDCAP_SQUARE | PS_JOIN_BEVEL | PS_USERSTYLE, 
				lineSize*lineScale, &penBrush, dashSize, dashBuffer);
		}
		else{
			// Create the pen for drawing the lines. Use a plain old CPen for this
			// task (as it is quite fast!).
			linepen = new CPen(GetLineStyle(lineType), lineSize*lineScale, lineColor);
		}

		// Make the current pen the state pen
		oldpen = pDC->SelectObject(linepen); 

		stackIterator<DataPoint> next(data);
		DataPoint xy;

		next(xy);

		if(use_gdi_path){
			// We need to use a GDI path to insure that the appearance of non-solid lines
			// does not depend on the density of points in the path. There can be 
			// SERIOUS performance degradation when rendering many points as a GDI path.
			pDC->BeginPath();
		}

		pDC->MoveTo(PlotX(xy.x, extent), PlotY(xy.y, extent));

		while(next(xy))
			pDC->LineTo(PlotX(xy.x, extent), PlotY(xy.y, extent));

		if(use_gdi_path){
			pDC->EndPath();
			pDC->StrokePath();
		}

		// Clean up the dashBuffer
		if(dashBuffer)
			delete [] dashBuffer;

		// Restore the original pen
		pDC->SelectObject(oldpen);

		// Clean up the linepen
		if(linepen)
			delete linepen;
	}

	// Draw the x error bars
	if(!xErrorBar.IsEmpty() && (display_x_error_bar != BAR_NONE)){
		// Always use the solid line style
		CPen linepen(GetLineStyle(error_bar_line_type), 
					 error_bar_line_size*lineScale, 
					 lineColor);
		
		// Make the current pen the state pen
		CPen *oldpen = pDC->SelectObject(&linepen); 
		stackIterator<DataPoint> next(data);
		stackIterator<float> nextError(xErrorBar);
		DataPoint xy;
		float ebar;
		int tmp;

		switch(display_x_error_bar){
			case BAR_BOTH:
				while(next(xy) && nextError(ebar)){
					tmp = PlotY(xy.y, extent);

					pDC->MoveTo(PlotX(xy.x - ebar, extent), tmp);
					pDC->LineTo(PlotX(xy.x + ebar, extent), tmp);

					// Draw the error bar "hats"
					tmp = PlotX(xy.x - ebar, extent);
					pDC->MoveTo(tmp, PlotY(xy.y, extent) - error_bar_hat_size);
					pDC->LineTo(tmp, PlotY(xy.y, extent) + error_bar_hat_size + 1);

					tmp = PlotX(xy.x + ebar, extent);
					pDC->MoveTo(tmp, PlotY(xy.y, extent) - error_bar_hat_size);
					pDC->LineTo(tmp, PlotY(xy.y, extent) + error_bar_hat_size + 1);
				}
				break;
			case BAR_TOP_LEFT:
				while(next(xy) && nextError(ebar)){
					tmp = PlotY(xy.y, extent);
					pDC->MoveTo(PlotX(xy.x - ebar, extent), tmp);
					pDC->LineTo(PlotX(xy.x, extent), tmp);

					// Draw the error bar "hats"
					tmp = PlotX(xy.x - ebar, extent);
					pDC->MoveTo(tmp, PlotY(xy.y, extent) - error_bar_hat_size);
					pDC->LineTo(tmp, PlotY(xy.y, extent) + error_bar_hat_size + 1);
				}
				break;
			case BAR_BOTTOM_RIGHT:
				while(next(xy) && nextError(ebar)){
					tmp = PlotY(xy.y, extent);
					pDC->MoveTo(PlotX(xy.x, extent), tmp);
					pDC->LineTo(PlotX(xy.x + ebar, extent), tmp);

					// Draw the error bar "hats"
					tmp = PlotX(xy.x + ebar, extent);
					pDC->MoveTo(tmp, PlotY(xy.y, extent) - error_bar_hat_size);
					pDC->LineTo(tmp, PlotY(xy.y, extent) + error_bar_hat_size + 1);
				}
				break;
		};

		// Restore the pen
		pDC->SelectObject(oldpen); 
	}

	// Draw the y error bars
	if(!yErrorBar.IsEmpty() && (display_y_error_bar != BAR_NONE)){
		// Always use the solid line style
		CPen linepen(GetLineStyle(error_bar_line_type), 
					 error_bar_line_size*lineScale, 
					 lineColor);
		
		// Make the current pen the state pen
		CPen *oldpen = pDC->SelectObject(&linepen); 
		stackIterator<DataPoint> next(data);
		stackIterator<float> nextError(yErrorBar);
		DataPoint xy;
		float ebar;
		int tmp;

		switch(display_y_error_bar){
			case BAR_BOTH:
				while(next(xy) && nextError(ebar)){
					tmp = PlotX(xy.x, extent);
					pDC->MoveTo(tmp, PlotY(xy.y - ebar, extent));
					pDC->LineTo(tmp, PlotY(xy.y + ebar, extent));

					// Draw the error bar "hats"
					tmp = PlotY(xy.y - ebar, extent);
					pDC->MoveTo(PlotX(xy.x, extent) - error_bar_hat_size, tmp);
					pDC->LineTo(PlotX(xy.x, extent) + error_bar_hat_size + 1, tmp);

					tmp = PlotY(xy.y + ebar, extent);
					pDC->MoveTo(PlotX(xy.x, extent) - error_bar_hat_size, tmp);
					pDC->LineTo(PlotX(xy.x, extent) + error_bar_hat_size + 1, tmp);
				}
				break;
			case BAR_TOP_LEFT:
				while(next(xy) && nextError(ebar)){
					tmp = PlotX(xy.x, extent);
					pDC->MoveTo(tmp, PlotY(xy.y, extent));
					pDC->LineTo(tmp, PlotY(xy.y + ebar, extent));

					// Draw the error bar "hats"
					tmp = PlotY(xy.y + ebar, extent);
					pDC->MoveTo(PlotX(xy.x, extent) - error_bar_hat_size, tmp);
					pDC->LineTo(PlotX(xy.x, extent) + error_bar_hat_size + 1, tmp);
				}
				break;
			case BAR_BOTTOM_RIGHT:
				while(next(xy) && nextError(ebar)){
					tmp = PlotX(xy.x, extent);
					pDC->MoveTo(tmp, PlotY(xy.y, extent));
					pDC->LineTo(tmp, PlotY(xy.y - ebar, extent));

					// Draw the error bar "hats"
					tmp = PlotY(xy.y - ebar, extent);
					pDC->MoveTo(PlotX(xy.x, extent) - error_bar_hat_size, tmp);
					pDC->LineTo(PlotX(xy.x, extent) + error_bar_hat_size + 1, tmp);
				}
				break;
		};

		// Restore the pen
		pDC->SelectObject(oldpen); 
	}

	if(pointType != POINT_NONE){
		// Create the pen for drawing the symbols
		// Note that we use the line color here if the point type
		// is anything other than dot or vector).
		CPen pointpen(PS_SOLID, 1*lineScale, 
			((pointType == POINT_DOT) || (pointType == POINT_VECTOR)) 
			? brush.lbColor : lineColor);
	
		// Make the current pen the point pen
		CPen *oldpen = pDC->SelectObject(&pointpen); 
		
		// Create the brush for drawing the symbols
		CBrush pointbrush;
		pointbrush.CreateBrushIndirect(&brush);

		// Make the current brush the point brush
		CBrush *oldbrush = pDC->SelectObject(&pointbrush); 

		stackIterator<DataPoint> next(data);
		DataPoint xy;

		CPoint pointUpTriangle[3];

		// Store current point in {x, y} and the last
		// point in {x1, y1}.
		int x, y, x1, y1, zero;

		// If we're drawing a vector plot, have we accumulated both
		// {x, y} and {x1, y1}?
		BOOL vector_status = FALSE;

		// Histograms read the data with an offset
		if(((pointType == POINT_HISTOGRAM_X) || (pointType == POINT_HISTOGRAM_Y)) &&
			next(xy)){ 
			
			x1 = PlotX(xy.x, extent);
			y1 = PlotY(xy.y, extent);

			// Draw a base line for the histogram
			if(pointType == POINT_HISTOGRAM_X){
				pDC->MoveTo(PlotX(minX, extent), PlotY(0.0, extent));
				pDC->LineTo(PlotX(maxX, extent), PlotY(0.0, extent));
			}
			else{ //(pointType == POINT_HISTOGRAM_Y)
				pDC->MoveTo(PlotX(0.0, extent), PlotY(minY, extent));
				pDC->LineTo(PlotX(0.0, extent), PlotY(maxY, extent));
			}
		}

		while(next(xy)){
			x = PlotX(xy.x, extent);
			y = PlotY(xy.y, extent);
			
			switch(pointType){
				case POINT_CIRCLE:
					// Since the Ellipse function does not extend to the
					// lower right boundary, we must increment this boundary
					// by 1 to make the ellipse centered about the data point.
					pDC->Ellipse(x - pointSize, y - pointSize,
							 x + pointSize + 1, y + pointSize + 1);
					break;
				case POINT_SQUARE:
					// Since the Rectangle function does not extend to the
					// lower right boundary, we must increment this boundary
					// by 1 to make the rectangle centered about the data point.
					pDC->Rectangle(x - pointSize, y - pointSize,
							 x + pointSize + 1, y + pointSize + 1);
					break;
				case POINT_TRIANGLE:
					// Draw a point up equilateral triangle that
					// is circumscribed by a circle of radius
					// pointSize		

					// The vertcies of the triangle (counter clockwise).
					pointUpTriangle[0].x = x;
					pointUpTriangle[0].y = (long)(y - pointSize);
					
					pointUpTriangle[1].x = (long)(x - pointSize*0.866025f);
					pointUpTriangle[1].y = (long)(y + pointSize*0.5f);

					pointUpTriangle[2].x = (long)(x + pointSize*0.866025f);
					pointUpTriangle[2].y = (long)(y + pointSize*0.5f);

					pDC->Polygon(pointUpTriangle, 3);
					break;
				case POINT_DOT:
					// There are several ways to draw a dot:
					// (1) Draw the smallest circle we can
					// pDC->Ellipse(x - 1, y - 1, x + 1, y + 1);
					// This has a large cost overhead.
					//
					// (2) Draw a single pixel
					// pDC->SetPixel(x, y, brush.lbColor);
					// Fine, except that programs that render metafiles as
					// vector based drawing (i.e. Illustrator) can't render
					// single pixels.

					// (3) Draw a very small line segment!
					// This is just as fast as (2), but has the advantage that
					// line segements show up under vector formats!
					pDC->MoveTo(x, y);
					pDC->LineTo(x + 1, y + 1);

					break;

				// A quick note for drawing rectangles!
				// The rectangle extends up to, but does not 
				// include, the right and bottom coordinates. 
				// This means that the height of the rectangle 
				// is y2 - y1 and the width of the rectangle is 
				// x2 - x1. Hence the +1's used below.
				case POINT_HISTOGRAM_X:
					zero = PlotY(0.0, extent);
					
					if(x < x1)
						pDC->Rectangle(x, max(zero, y) + 1, 
							x1 + 1, min(zero, y));
					else
						pDC->Rectangle(x1, max(zero, y1) + 1, 
							x + 1, min(zero, y1));

					x1 = x;
					y1 = y;
					break;
				case POINT_HISTOGRAM_Y:
					zero = PlotX(0.0, extent);
					
					if(y > y1)
						pDC->Rectangle(min(x, zero), y + 1, 
							max(x, zero) + 1, y1);
					else
						pDC->Rectangle(min(x1, zero), y1 + 1, 
							max(x1, zero) + 1, y);

					x1 = x;
					y1 = y;
					break;
				case POINT_VECTOR:
					
					if(vector_status){
						// Note that plot data access "tail first" so we
						// must "reverse" the head-tail point order when
						// passing data to DrawVector
						//DrawVector(pDC, x1, y1, x, y); <-- tail ... head
						DrawVector(pDC, x, y, x1, y1);// <-- head ... tail
						vector_status = FALSE;
					}
					else{
						// Reset the status flag to collect another
						// vector pair
						vector_status = TRUE;
					}

					x1 = x;
					y1 = y;
					break;
			};
		}

		// Restore the pen and brush
		pDC->SelectObject(oldpen); 
		pDC->SelectObject(oldbrush); 
	}
}

void SPlot::DrawVector(CDC* pDC, int x_tail, int y_tail, int x_head, int y_head)
{
	const double barb_angle = 0.26; // In radians

	const double x = 2*pointSize,
				 y = 2*pointSize;

	int tmp_x1, tmp_y1;
	int tmp_x2, tmp_y2;
	int dx, dy;
	double angle;
	float s, c;

	dx = x_head - x_tail;
	dy = y_head - y_tail;

	if(x_tail == x_head){
		if(y_head > y_tail)
			angle = 0.5*PI;
		else
			angle = -0.5*PI;
	}
	else
		angle = atan((double)(dy)/dx);

	if(dx < 0){
		if(dy < 0){
			angle = angle - PI;
		}
		else{
			angle = PI + angle;
		}
	}

	s = (float)(sin(angle - barb_angle));
	c = (float)(cos(angle - barb_angle));

	tmp_x1 = (int)(-x*c + y*s) + x_head;
	tmp_y1 = -(int)(x*s + y*c) + y_head;

	s = (float)(sin(angle + barb_angle));
	c = (float)(cos(angle + barb_angle));

	tmp_x2 = (int)(-x*c - y*s) + x_head;
	tmp_y2 = -(int)(x*s - y*c) + y_head;

	// The vector Shaft
	pDC->MoveTo(x_tail, y_tail);
	pDC->LineTo(x_head, y_head);

	// The vector barbs
	pDC->MoveTo(x_head, y_head);
	pDC->LineTo(tmp_x1, tmp_y1);

	pDC->MoveTo(x_head, y_head);
	pDC->LineTo(tmp_x2, tmp_y2);
}

void SPlot::DrawSelect(CDC* pDC, CRect &extent)
{
	// Do nothing!
}

CSelectRect SPlot::SelectRect(CDC* pDC, CRect &extent)
{
	// Plots can not be selected
	// (return an "empty" rectangle).
	return CSelectRect();
}

// Smallest distance (squared) from a point in the plot to
// the specified point.
double SPlot::HitTest(int m_x, int m_y, float &f_x, float &f_y, CRect &extent)
{
	stackIterator<DataPoint> next(data);
	DataPoint xy;
	int x;
	int y;
	double dist2, dx, dy;

	if(next(xy)){
		x = PlotX(xy.x, extent);
		y = PlotY(xy.y, extent);

		dx = m_x - x;
		dy = m_y - y;
		
		f_x = xy.x;
		f_y = xy.y;

		dist2 = dx*dx + dy*dy;
	}
	else{
		// Empty set
		return -1.0;
	}

	while(next(xy)){
		x = PlotX(xy.x, extent);
		y = PlotY(xy.y, extent);

		dx = m_x - x;
		dy = m_y - y;

		double tmp_dist2 = dx*dx + dy*dy;

		if(tmp_dist2 < dist2){
			dist2 = tmp_dist2;
			f_x = xy.x;
			f_y = xy.y;
		}
	}

	return dist2;
}

void SPlot::Serialize(CArchive& ar)
{
	SObject::Serialize(ar);
	
	// Save\load the data
	data.Serialize(ar);

	// Save\load the error bars (and fix bug with version < 1.05 
	// that didn't save the x error bars, but saved the y error bars 
	// twice instead!).
	if(ar.IsStoring() || version > 1.04f)
		xErrorBar.Serialize(ar);
	else
		yErrorBar.Serialize(ar);

	yErrorBar.Serialize(ar);

	if(version > 1.05f)
		legend.Serialize(ar);

	if(ar.IsStoring()){
		ar << display_y_error_bar;
		ar << display_x_error_bar;
		ar << error_bar_hat_size;
		ar << error_bar_line_type;
		ar << error_bar_line_size;
		ar << x_scaleType;
		ar << y_scaleType;
		ar << canXLog;
		ar << canYLog;
		ar << x1;
		ar << y1;
		ar << x2;
		ar << y2;
		ar << pointType;
		ar << pointSize;
		ar << brush.lbColor;
		ar << lineType;
		ar << lineSize;
		ar << lineColor;
		ar << minX;
		ar << maxX;
		ar << minY;
		ar << maxY;
		ar << info;

		// legend no longer stored as simple text as of 1.06
		//ar << legend;
	}
	else{
		ar >> display_y_error_bar;
		ar >> display_x_error_bar;

		// Woops - forgot to save error bar properties!
		if(version > 1.04f){
			ar >> error_bar_hat_size;
			ar >> error_bar_line_type;
			ar >> error_bar_line_size;
		}

		ar >> x_scaleType;
		ar >> y_scaleType;
		ar >> canXLog;
		ar >> canYLog;
		ar >> x1;
		ar >> y1;
		ar >> x2;
		ar >> y2;
		ar >> pointType;

		// With the advent of vector style (as of version 1.16)
		// we must correct the point type for older plots
		if(version < 1.155f){ // Compare to 1.155 to prevent round off problems!
			if(pointType == POINT_VECTOR)
				// What is now the value indicating POINT_VECTOR used
				// to be the value indicating POINT_NONE.
				pointType = POINT_NONE;
		}

		ar >> pointSize;
		ar >> brush.lbColor;
		ar >> lineType;
		ar >> lineSize;
		ar >> lineColor;
		ar >> minX;
		ar >> maxX;
		ar >> minY;
		ar >> maxY;
		ar >> info;

		if(version < 1.06f){
			// The old way of storing a legend
			CString tmp;

			ar >> tmp;

			legend.SetText(tmp);
		}

		// If these values are ever negative, an error must have
		// occured reading in the file.
		if((pointSize < 0) || (lineSize < 0) || 
		   (x_scaleType < 0) || (y_scaleType < 0) || 
		   (pointType < 0) || (lineType < 0))
			AfxThrowArchiveException(CArchiveException::badIndex,
				ar.m_pDocument->GetTitle());

		// Set the symbol brush
		if(brush.lbColor == NO_COLOR)
			brush.lbStyle = BS_HOLLOW;
		else
			brush.lbStyle = BS_SOLID;
	}
}

