//	(c) 2000-2002 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.
#ifndef __SPLOT
#define __SPLOT

#include "sobject.h"
#include "stextobject.h"
#include "data.h"
#include "stack.h"

// A plot of {x,y} data points.
class SPlot: public SObject{
private:
	// Empty the plot (data and error bar stacks) on copy.
	// This is a ephemeral flag and is not saved by Serialize().
	BOOL empty_on_copy;
public:
	// Enumerate the symbol types. The order of the inumeration
	// must match the order of appearance in the plot
	// options dialog box.
	enum{POINT_CIRCLE = 0, POINT_SQUARE, POINT_TRIANGLE, 
		 POINT_DOT, POINT_HISTOGRAM_X, POINT_HISTOGRAM_Y,
		 POINT_VECTOR, POINT_NONE};
	
	// Enumerate the line types. The order of the inumeration
	// must match the order of appearance in the plot
	// options dialog box.
	enum{LINE_SOLID = 0, LINE_DOTTED, LINE_DASHED, LINE_DASHDOT, 
		 LINE_DASHDOTDOT, LINE_NONE};

	SPlot();
	
	// The data
	stack<DataPoint> data;
	
	// Error bars
	stack<float> yErrorBar;
	stack<float> xErrorBar;

	enum {BAR_TOP_LEFT = 0, BAR_BOTTOM_RIGHT, BAR_BOTH, BAR_NONE};

	int display_y_error_bar, 
		display_x_error_bar;

	int error_bar_hat_size;
	int error_bar_line_type;
	int error_bar_line_size;

	// The plot bounding box
	float x1, y1, // Lower left corner
		  x2, y2; // Upper right corner
	
	// How will the data be scaled to the display (i.e. linear or log)?
	int x_scaleType, y_scaleType;
	
	// Do the X or Y data sets contain any number less than or equal to zero
	// (i.e. can we take the log). False by default, but determined at data
	// import.
	BOOL canXLog, canYLog;

	int pointType;
	int pointSize;

	// Symbol color and fill type info are stored in the
	// LOGBRUSH structure. A negative value for the color
	// field of the brush structure indicates no color.
	LOGBRUSH brush;

	int lineType;
	int lineSize;
	COLORREF lineColor;

	float minX, maxX, minY, maxY;
	
	// Store usefull info about the plot in this string (i.e. where
	// did the plot come from, how was it generated, etc).
	CString info;

	// The plot legend (which is used to store user defined lables for 
	// the graph legend).
	STextObject legend;

	// Allow both "shallow" and "deep" copy semantics
	// (under the control of empty_on_copy and Move()).
	void MoveDataOnCopy(BOOL m_flag);

	SPlot &operator=(SPlot &m_copy);

	virtual void Serialize(CArchive& ar);

	// Recompute min and max values
	BOOL UpdateBoundsX();
	BOOL UpdateBoundsY();
	BOOL UpdateBoundsXY();

	// Return the range of X values
	float GetRangeX();

	int GetLineStyle(int lt);
	DWORD* GetLineStyle(int lt, int &db_size);

	// Is point p within the plot bounds?
	inline BOOL CheckBounds(DataPoint &p);

	// Map the data points to the screen coordinates
	inline int PlotX(float x, CRect &extent);
	inline int PlotY(float y, CRect &extent);

	void Draw(CDC* pDC, CRect &extent);
	void DrawSelect(CDC* pDC, CRect &extent);
	void DrawVector(CDC* pDC, int x_tail, int y_tail, int x_head, int y_head);

	CSelectRect SelectRect(CDC* pDC, CRect &extent);

	// Smallest distance (squared) from a point in the plot to
	// the specified point.
	double HitTest(int m_x, int m_y, float &f_x, float &f_y, CRect &extent);

	void PreparePrinting(CDC* pDC, long scaleMul, long scaleDiv, CRect &extent);
	void CleanUpPrinting(CDC* pDC, CRect &extent);
};

#endif // __SPLOT