//	(c) 2000 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.
// FILO singly-linked list class

#ifndef __STACK
#define	__STACK
#include <stdlib.h>
#include "Excepts.h"

// Forward declation of the iterator
template <class B> class stackIterator; 

template <class A>
class stackelem{
	public:
		stackelem *next;
		A value;
	
		stackelem(const A& elem)
		{
			value = elem;
			next = NULL;
		};
};

template <class B>
class stack{
	private:
		stackelem<B> *head;
		int length;
		
	public:
		friend class stackIterator<B>;

		stack() 
		{ 
	    	length = 0;
			head = NULL;
		};
	    
		~stack() 
		{ 
	    	while(!IsEmpty())
	        	Pop();

			head = NULL;
		};
		
		// Puch thing onto the top of the list
		void Push(const B &thing);
		
		// Remove the last element of the list.
		// DOES NOT return the value of the popped element.
		void Pop();
	
		int Length(){return length;};
	
		int IsEmpty() {return !length;};

#ifdef _WINDOWS
		void Serialize(CArchive& ar);
#endif //_WINDOWS

		stack<B>& operator=(const stack<B> &copy);

		// Move the memory from m_copy to the target stack
		void Move(stack<B> &m_copy);
};

#ifdef _WINDOWS
template <class B>
void stack<B>::Serialize(CArchive& ar)
{
	B tmp;

	if(ar.IsStoring()){
		stackIterator<B> next(*this);
		ar << length;

		while(next(tmp))
			tmp.Serialize(ar);
	}
	else{
		int len;
		
		// Save length with a temporary variable
		ar >> len;

		for(int i = 0;i < len;i++){
			tmp.Serialize(ar);
			Push(tmp);
		}
	}
}
#endif // _WINDOWS

template <class B>
stack<B>& stack<B>::operator=(const stack<B> &copy)
{
	// Clean up the current stack
	while(!IsEmpty())
		Pop();

	stackIterator<B> next(copy);
	B tmp;

	while(next(tmp))
		Push(tmp);

	return (*this);
}

template <class B>
void stack<B>::Move(stack<B> &m_copy)
{
	// Clean the curren stack
	while(!IsEmpty())
		Pop();

	// Copy the POINTER from the m_copy variable
	head = m_copy.head;
	length = m_copy.length;

	// Update the m_copy variable
	m_copy.head = NULL;
	m_copy.length = 0;
}

template <class B>
void stack<B>::Pop()
{
	if(head != NULL){
		stackelem<B> *tmp = head;
		head = head->next;

		delete tmp;
		length--;
	}
	else
		throw Error("Trying to pop an empty stack!", 
			__FILE__, __LINE__);
};

template <class B>
void stack<B>::Push(const B &thing)
{
	stackelem<B> *tmp = new stackelem<B>(thing);

	tmp->next = head;
	head = tmp;

	length++;
}

template <class B>
class stackIterator {
private:
	stackelem<B> *curr;

public:
	stackIterator(const stack<B>& theStack)
	{
		curr = theStack.head;
	};

	inline int operator()(B& elem)
	{
		if(curr != NULL){
			elem = curr->value;
			curr = curr->next;

			return TRUE;
		}

		return FALSE;
	};

	inline int operator()(B*& elem)
	{
		if(curr != NULL){
			elem = &(curr->value);
			curr = curr->next;

			return TRUE;
		}

		return FALSE;
	};

	inline void reset(stack<B>& theStack)
	{
		curr = theStack.head;
	};

	inline stackelem<B> *current()
	{
		return curr;
	};
};

#ifdef _WINDOWS
// Specific instances of stack::Serialize live here
// (for classes that do not have a Serialize member function).
inline void stack<float>::Serialize(CArchive& ar)
{
	float tmp;

	if(ar.IsStoring()){
		stackIterator<float> next(*this);
		ar << length;

		while(next(tmp))
			ar << tmp;
	}
	else{
		int len;
		
		// Save length with a temporary variable
		ar >> len;

		for(int i = 0;i < len;i++){
			ar >> tmp;
			Push(tmp);
		}
	}
}
#endif // _WINDOWS

#endif // __STACK
