//	(c) 2000-2002 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.
#ifndef __STEXT_OBJECT
#define __STEXT_OBJECT

#include "sobject.h"
#include "Text.h"

// The STextObject object encapsulates user defined text (that can be rendered
// to the screen).
class STextObject : public SObject{
private:
	// The raw text string
	CString text;

	// The formated text
	TextLabel label_text;

	// Font
	LOGFONT font;
	COLORREF fontColor;

public:

	inline void SetText(CString &m_text)
	{
		text = m_text;

		label_text.SetLabel(text, font, fontColor);
	};

	inline void SetText(CString &m_text, LOGFONT &m_font)
	{
		text = m_text;
		font = m_font;

		label_text.SetLabel(text, font, fontColor);
	};

	inline void SetText(CString &m_text, LOGFONT &m_font, COLORREF &m_color)
	{
		text = m_text;
		font = m_font;
		fontColor = m_color;

		label_text.SetLabel(text, font, fontColor);
	};


	inline void SetText(char *m_text)
	{
		text.Format("%s", m_text);

		label_text.SetLabel(text, font, fontColor);
	};

	inline void SetText(char *m_text, LOGFONT &m_font)
	{
		text.Format("%s", m_text);
		font = m_font;

		label_text.SetLabel(text, font, fontColor);
	};

	inline void SetText(char *m_text, LOGFONT &m_font, COLORREF &m_color)
	{
		text.Format("%s", m_text);
		font = m_font;
		fontColor = m_color;

		label_text.SetLabel(text, font, fontColor);
	};

	// This function allows the font size to be changed
	// (but is inefficient in that it rebuilds the formatted
	// text structure). If possible, set the font at the same time the
	// text is being set (via one of the SetText functions).
	inline SetFontSize(int m_pt_size)
	{
		font.lfHeight = -m_pt_size;

		// Rebuild the formated text
		label_text.SetLabel(text, font, fontColor);
	};

	inline SetFont(LOGFONT &m_font, COLORREF m_color)
	{
		font = m_font;
		fontColor = m_color;

		// Rebuild the formated text
		label_text.SetLabel(text, font, fontColor);
	};

	inline CString GetText()
	{
		return text;
	};

	inline int Length()
	{
		return text.GetLength();
	}

	inline CSize GetExtent(CDC* pDC)
	{
		CRect tmp = label_text.GetExtent(pDC);
		return CSize(tmp.Width(), tmp.Height());
	};

	STextObject();
	STextObject(CString m_text, float m_x, float m_y,
		int orientation = 0);

	void Draw(CDC* pDC, CRect &extent);
	void DrawSelect(CDC* pDC, CRect &extent);

	CSelectRect SelectRect(CDC* pDC, CRect &extent);

	void PreparePrinting(CDC* pDC, long scaleMul, long scaleDiv, CRect &extent);
	void CleanUpPrinting(CDC* pDC, CRect &extent);
	
	virtual void Serialize(CArchive& ar);

	void OnDelete();

	// Pop up a modal dialog to allow the user to set the
	// text and font.
	BOOL ModalSet(const CString &title);
};

#endif // __STEXT_OBJECT